"""
Shyft hydrology python api providing basic types
"""
from __future__ import annotations
from builtins import list as DoubleVector
import numpy
import shyft.time_series
from shyft.time_series import GeoPoint
from shyft.time_series import GeoPointVector
from shyft.time_series import TimeAxis
from shyft.time_series import TsVector
import typing
from . import pt_gs_k
from . import pt_hps_k
from . import pt_hs_k
from . import pt_ss_k
from . import pt_st_hbv
from . import pt_st_k
from . import r_pm_gs_k
from . import r_pm_st_k
from . import r_pmv_st_k
from . import r_pt_gs_k
__all__ = ['ABS_DIFF', 'ARegionEnvironment', 'ActualEvapotranspirationCalculate_step', 'ActualEvapotranspirationParameter', 'ActualEvapotranspirationResponse', 'BOBYQA', 'BTKParameter', 'CELL_CHARGE', 'CalibrationOption', 'CalibrationStatus', 'CatchmentPropertyType', 'CellEnvironment', 'CellStateId', 'DISCHARGE', 'DREAM', 'DoubleVector', 'DrmClient', 'DrmServer', 'EXPONENTIAL', 'FlowAdjustResult', 'FreeWaterEvaporationCalculator', 'FreeWaterEvaporationResponse', 'GAUSSIAN', 'GLOBAL', 'GammaSnowCalculator', 'GammaSnowParameter', 'GammaSnowResponse', 'GammaSnowState', 'GeoCellData', 'GeoCellDataClient', 'GeoCellDataModel', 'GeoCellDataModelVector', 'GeoCellDataServer', 'GeoCellDataVector', 'GeoPoint', 'GeoPointSource', 'GeoPointSourceVector', 'GeoPointVector', 'GlacierMeltParameter', 'GoalFunctionClient', 'GoalFunctionModel', 'GoalFunctionModelVector', 'GoalFunctionServer', 'HbvActualEvapotranspirationCalculate_step', 'HbvActualEvapotranspirationParameter', 'HbvActualEvapotranspirationResponse', 'HbvPhysicalSnowCalculator', 'HbvPhysicalSnowParameter', 'HbvPhysicalSnowResponse', 'HbvPhysicalSnowState', 'HbvSnowCalculator', 'HbvSnowParameter', 'HbvSnowResponse', 'HbvSnowState', 'HbvSoilCalculator', 'HbvSoilParameter', 'HbvSoilResponse', 'HbvSoilState', 'HbvTankCalculator', 'HbvTankParameter', 'HbvTankResponse', 'HbvTankState', 'IDWParameter', 'IDWPrecipitationParameter', 'IDWTemperatureParameter', 'InterpolationParameter', 'KLING_GUPTA', 'KalmanBiasPredictor', 'KalmanFilter', 'KalmanParameter', 'KalmanState', 'KirchnerCalculator', 'KirchnerParameter', 'KirchnerResponse', 'KirchnerState', 'LandTypeFractions', 'MethodStackParameter', 'NASH_SUTCLIFFE', 'OKCovarianceType', 'OKParameter', 'OptimizerMethod', 'PT_GS_K', 'PT_GS_K_OPT', 'PT_HPS_K', 'PT_HPS_K_OPT', 'PT_HS_K', 'PT_HS_K_OPT', 'PT_SS_K', 'PT_SS_K_OPT', 'PT_ST_HBV', 'PT_ST_K', 'PT_ST_K_HBV', 'PT_ST_K_OPT', 'ParameterClient', 'ParameterModel', 'ParameterModelVector', 'ParameterOptimizer', 'ParameterServer', 'PenmanMonteithCalculator', 'PenmanMonteithParameter', 'PenmanMonteithResponse', 'PenmanMonteithVegetationCalculator', 'PenmanMonteithVegetationParameter', 'PenmanMonteithVegetationResponse', 'PrecipitationCorrectionCalculator', 'PrecipitationCorrectionParameter', 'PrecipitationSource', 'PrecipitationSourceVector', 'PriestleyTaylorCalculator', 'PriestleyTaylorParameter', 'PriestleyTaylorResponse', 'RMSE', 'ROUTED_DISCHARGE', 'R_PMV_ST_K', 'R_PMV_ST_K_OPT', 'R_PM_GS_K', 'R_PM_GS_K_OPT', 'R_PM_ST_K', 'R_PM_ST_K_OPT', 'R_PT_GS_K', 'R_PT_GS_K_OPT', 'RadiationCalculator', 'RadiationParameter', 'RadiationResponse', 'RadiationSource', 'RadiationSourceVector', 'RegionModelType', 'RelHumSource', 'RelHumSourceVector', 'River', 'RiverNetwork', 'RoutingInfo', 'SCEUA', 'SNOW_COVERED_AREA', 'SNOW_WATER_EQUIVALENT', 'SkaugenCalculator', 'SkaugenParameter', 'SkaugenResponse', 'SkaugenState', 'SnowTilesCalculator', 'SnowTilesParameter', 'SnowTilesResponse', 'SnowTilesState', 'StateClient', 'StateModel', 'StateModelVector', 'StateServer', 'TargetSpecCalcType', 'TargetSpecificationPts', 'TargetSpecificationVector', 'TemperatureSource', 'TemperatureSourceVector', 'TimeAxis', 'TsTransform', 'TsVector', 'UHGParameter', 'WindSpeedSource', 'WindSpeedSourceVector', 'bayesian_kriging_temperature', 'catchment', 'cell', 'compute_geo_ts_values_at_time', 'create_precipitation_source_vector_from_np_array', 'create_radiation_source_vector_from_np_array', 'create_rel_hum_source_vector_from_np_array', 'create_temperature_source_vector_from_np_array', 'create_wind_speed_source_vector_from_np_array', 'glacier_melt_step', 'idw_precipitation', 'idw_radiation', 'idw_relative_humidity', 'idw_temperature', 'idw_wind_speed', 'make_uhg_from_gamma', 'ordinary_kriging', 'parse_cf_time', 'pt_gs_k', 'pt_hps_k', 'pt_hs_k', 'pt_ss_k', 'pt_st_hbv', 'pt_st_k', 'r_pm_gs_k', 'r_pm_st_k', 'r_pmv_st_k', 'r_pt_gs_k', 'shyftdata_dir', 'stat_scope', 'version']
class ARegionEnvironment:
    """
    Contains all geo-located sources to be used by a Shyft core model
    """
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @staticmethod
    def create_from_geo_ts_matrix(m: shyft.time_series.GeoTsMatrix, t: int = 0, e: int = 0, v: list[int] = []) -> ARegionEnvironment:
        """
        Construct ARegionEnvironment from a specified slice of a GeoTsMatrix
        
        Args:
            m (GeoTsMatrix): The matrix from which to extract temperature,precipitation,radiation,wind-speed,rel-hum data
        
            t (int): the forecast index to extract
        
            e (int): the ensemble index to extract
        
        Returns:
            ARegionEnvironment: ARegionEnvironment. A ready to use region environment for interpolation
        
            v (IntVector): variable indicies that in order of appearance selects temperature,precipitation,radiation,wind-speed,rel-hum.If empty/default, same as [0,1,2,3,4]
        """
    @staticmethod
    def deserialize(blob: list[str]) -> ARegionEnvironment:
        """
        convert a blob, as returned by .serialize() into a ARegionEnvironment
        """
    def __eq__(self, arg0: ARegionEnvironment) -> bool:
        ...
    def __init__(self) -> None:
        ...
    def __ne__(self, arg0: ARegionEnvironment) -> bool:
        ...
    def serialize(self) -> list[str]:
        """
        convert ARegionEnvironment into a binary blob that later can be restored with the .deserialize(blob) method
        """
    @property
    def precipitation(self) -> PrecipitationSourceVector:
        """
        PrecipitationSourceVector: precipitation sources
        """
    @precipitation.setter
    def precipitation(self, arg1: PrecipitationSourceVector) -> None:
        ...
    @property
    def radiation(self) -> RadiationSourceVector:
        """
        RadiationSourceVector: radiation sources
        """
    @radiation.setter
    def radiation(self, arg1: RadiationSourceVector) -> None:
        ...
    @property
    def rel_hum(self) -> RelHumSourceVector:
        """
        RelHumSourceVector: rel-hum sources
        """
    @rel_hum.setter
    def rel_hum(self, arg1: RelHumSourceVector) -> None:
        ...
    @property
    def temperature(self) -> TemperatureSourceVector:
        """
        TemperatureSourceVector: temperature sources
        """
    @temperature.setter
    def temperature(self, arg1: TemperatureSourceVector) -> None:
        ...
    @property
    def variables(self) -> tuple[tuple[str, TemperatureSourceVector], tuple[str, PrecipitationSourceVector], tuple[str, RadiationSourceVector], tuple[str, RelHumSourceVector], tuple[str, WindSpeedSourceVector]]:
        ...
    @property
    def wind_speed(self) -> WindSpeedSourceVector:
        """
        WindSpeedSourceVector: wind-speed sources
        """
    @wind_speed.setter
    def wind_speed(self, arg1: WindSpeedSourceVector) -> None:
        ...
class ActualEvapotranspirationParameter:
    ae_scale_factor: float
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, ae_scale_factor: float = 1.5) -> None:
        ...
class ActualEvapotranspirationResponse:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @property
    def ae(self) -> float:
        """
        float: actual evapotranspiration
        """
    @ae.setter
    def ae(self, arg0: float) -> None:
        ...
class BTKParameter:
    """
    BTKParameter class with time varying gradient based on day no
    """
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, arg0: BTKParameter) -> bool:
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, temperature_gradient: float, temperature_gradient_sd: float) -> None:
        """
        specifying default temp.grad(not used) and std.dev[C/100m]
        """
    @typing.overload
    def __init__(self, temperature_gradient: float, temperature_gradient_sd: float, sill: float = 25.0, nugget: float = 0.5, range: float = 200000.0, zscale: float = 20.0) -> None:
        ...
    def __ne__(self, arg0: BTKParameter) -> bool:
        ...
    def nug(self) -> float:
        """
        Nugget magnitude,default=0.5
        """
    def range(self) -> float:
        """
        Point where semivariogram flattens out,default=200000.0
        """
    def sill(self) -> float:
        """
        Value of semivariogram at range default=25.0
        """
    def temperature_gradient(self, p: shyft.time_series.UtcPeriod) -> float:
        """
        return default temp.gradient based on day of year calculated for midst of utcperiod p
        """
    def temperature_gradient_sd(self) -> float:
        """
        returns Prior standard deviation of temperature gradient in [C/m]
        """
    def zscale(self) -> float:
        """
        Height scale used during distance computations,default=20.0
        """
class CalibrationOption:
    """
    The CalibrationOption controls how the calibration optmizer is run regarding method and termination criterias.
    Notice that the different optmizers do have variations in the termination criterias.
    All optimization algorithms expect at least one-dimensional paramteter space.
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @staticmethod
    def bobyqa(max_iterations: int, tr_start: float, tr_stop: float) -> CalibrationOption:
        """
        Construct bobyqa options.
        
            max_iterations (int): stop after this number of iterations (default 1500).
        
            tr_start (float): BOBYQA: trust region start (default 0.1).
        
            tr_stop (float): trust region stop/end (default 1e-5).
        
        Returns:
            CalibrationOption: co. An instance of CalibrationOption for the relevant optmization algorithm.
        """
    @staticmethod
    def dream(max_iterations: int) -> CalibrationOption:
        """
        Construct dream options
        
            max_iterations (int): stop after this number of iterations (default 1500).
        
        Returns:
            CalibrationOption: co. An instance of CalibrationOption for the relevant optmization algorithm.
        """
    @staticmethod
    def global_search(max_iterations: int, time_limit: shyft.time_series.time, solver_epsilon: float) -> CalibrationOption:
        """
        Construct global search options
        
            max_iterations (int): stop after this number of iterations (default 1500).
        
            time_limit (float): don't run the global search longer than this (default 0, off).
        
            solver_epsilon (float): select which parameter optimizer to use (default 0.001).
        
        Returns:
            CalibrationOption: co. An instance of CalibrationOption for the relevant optmization algorithm.
        """
    @staticmethod
    def sceua(max_iterations: int, x_epsilon: float, y_epsilon: float) -> CalibrationOption:
        """
        Construct sceua options
        
            max_iterations (int): stop after this number of iterations (default 1500).
        
            x_epsilon (float): stop when normalized parameters x is within this range in variation.
        
            y_epsilon (float): stop when goal function is stable within this range (not improving any further).
        
        Returns:
            CalibrationOption: co. An instance of CalibrationOption for the relevant optmization algorithm.
        """
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, method: OptimizerMethod, max_iterations: int = 1500, time_limit: shyft.time_series.time = ..., solver_epsilon: float = 0.001, x_epsilon: float = 0.001, y_epsilon: float = 0.001, tr_start: float = 0.1, tr_stop: float = 1e-05) -> None:
        """
        Initialises a ParameterOptimizer instance.
        
            method (OptimizerMethod): select which parameter optimizer to use.
        
            max_iterations (int): stop after this number of iterations (default 1500).
        
            time_limit (float): GLOBAL: don't run the global search longer than this (default 0, off).
        
            solver_epsilon (float): GLOBAL: select which parameter optimizer to use (default 0.001).
        
            x_epsilon (float): SCEUA: stop when normalized parameters x is within this range in variation.
        
            y_epsilon (float): SCEUA: stop when goal function is stable within this range (not improving any further).
        
            tr_start (float): BOBYQA: trust region start (default 0.1).
        
            tr_stop (float): BOBYQA: trust region stop/end (default 1e-5).
        """
    @property
    def max_iterations(self) -> int:
        """
        int: stop after this number of iterations (default 1500)
        """
    @max_iterations.setter
    def max_iterations(self, arg0: int) -> None:
        ...
    @property
    def method(self) -> OptimizerMethod:
        """
        int: OptimizerMethod,  which parameter optimizer to use
        """
    @method.setter
    def method(self, arg0: OptimizerMethod) -> None:
        ...
    @property
    def solver_epsilon(self) -> float:
        """
        float: GLOBAL: select which parameter optimizer to use (default 0.001)
        """
    @solver_epsilon.setter
    def solver_epsilon(self, arg0: float) -> None:
        ...
    @property
    def time_limit(self) -> shyft.time_series.time:
        """
        time: GLOBAL: don't run the global search longer than this (default 0, off)
        """
    @time_limit.setter
    def time_limit(self, arg0: shyft.time_series.time) -> None:
        ...
    @property
    def tr_start(self) -> float:
        """
        float: BOBYQA: trust region start (default 0.1)
        """
    @tr_start.setter
    def tr_start(self, arg0: float) -> None:
        ...
    @property
    def tr_stop(self) -> float:
        """
        float: BOBYQA: trust region stop/end (default 1e-5)
        """
    @tr_stop.setter
    def tr_stop(self, arg0: float) -> None:
        ...
    @property
    def x_epsilon(self) -> float:
        """
        float: SCEUA: stop when normalized parameters x is within this range in variation
        """
    @x_epsilon.setter
    def x_epsilon(self, arg0: float) -> None:
        ...
    @property
    def y_epsilon(self) -> float:
        """
        float: SCEUA: stop when goal function is stable within this range (not improving any further)
        """
    @y_epsilon.setter
    def y_epsilon(self, arg0: float) -> None:
        ...
class CalibrationStatus:
    """
    The CalibrationStatus is returned from drms, or local call to the region model calibration object.It provides status, progress, and current trace of parameters and goal function values
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __repr__(self) -> str:
        ...
    def __str__(self) -> str:
        ...
    def result(self) -> typing.Any:
        """
        returns resulting calibrated parameter if available
        """
    def trace_goal_function_value(self, i: int) -> float:
        """
        returns the i'th goal function value
        """
    def trace_parameter(self, i: int) -> typing.Any:
        """
        returns the i'th parameter tried, corresponding to the 
        i'th trace_goal_function value
        """
    @property
    def running(self) -> bool:
        """
        bool: True if the calibration is still running
        """
    @property
    def trace_goal_function_values(self) -> list[float]:
        """
        DoubleVector: goal function values obtained so far
        """
    @property
    def trace_size(self) -> int:
        """
        int: returns the size of the parameter-trace
        
        See also:
            trace_goal_function_value,trace_parameter
        """
class CatchmentPropertyType:
    """
    Members:
    
      DISCHARGE
    
      SNOW_COVERED_AREA
    
      SNOW_WATER_EQUIVALENT
    
      ROUTED_DISCHARGE
    
      CELL_CHARGE
    """
    CELL_CHARGE: typing.ClassVar[CatchmentPropertyType]  # value = <CatchmentPropertyType.CELL_CHARGE: 4>
    DISCHARGE: typing.ClassVar[CatchmentPropertyType]  # value = <CatchmentPropertyType.DISCHARGE: 0>
    ROUTED_DISCHARGE: typing.ClassVar[CatchmentPropertyType]  # value = <CatchmentPropertyType.ROUTED_DISCHARGE: 3>
    SNOW_COVERED_AREA: typing.ClassVar[CatchmentPropertyType]  # value = <CatchmentPropertyType.SNOW_COVERED_AREA: 1>
    SNOW_WATER_EQUIVALENT: typing.ClassVar[CatchmentPropertyType]  # value = <CatchmentPropertyType.SNOW_WATER_EQUIVALENT: 2>
    __members__: typing.ClassVar[dict[str, CatchmentPropertyType]]  # value = {'DISCHARGE': <CatchmentPropertyType.DISCHARGE: 0>, 'SNOW_COVERED_AREA': <CatchmentPropertyType.SNOW_COVERED_AREA: 1>, 'SNOW_WATER_EQUIVALENT': <CatchmentPropertyType.SNOW_WATER_EQUIVALENT: 2>, 'ROUTED_DISCHARGE': <CatchmentPropertyType.ROUTED_DISCHARGE: 3>, 'CELL_CHARGE': <CatchmentPropertyType.CELL_CHARGE: 4>}
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
class CellEnvironment:
    """
    Contains all ts projected to a certain cell-model using interpolation step (if needed)
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self) -> None:
        ...
    def has_nan_values(self) -> bool:
        """
        scans all time-series for nan-values
        
        Returns:
            bool: has_nan. true if any nan is encounted, otherwise false
        """
    def init(self, ta: shyft.time_series.TimeAxisFixedDeltaT) -> None:
        """
        zero all series, set time-axis ta
        """
    @property
    def precipitation(self) -> shyft.time_series.TsFixed:
        """
        TsFixed: precipitation
        """
    @precipitation.setter
    def precipitation(self, arg0: shyft.time_series.TsFixed) -> None:
        ...
    @property
    def radiation(self) -> shyft.time_series.TsFixed:
        """
        TsFixed: radiation
        """
    @radiation.setter
    def radiation(self, arg0: shyft.time_series.TsFixed) -> None:
        ...
    @property
    def rel_hum(self) -> shyft.time_series.TsFixed:
        """
        TsFixed: relhum
        """
    @rel_hum.setter
    def rel_hum(self, arg0: shyft.time_series.TsFixed) -> None:
        ...
    @property
    def temperature(self) -> shyft.time_series.TsFixed:
        """
        TsFixed: temperature
        """
    @temperature.setter
    def temperature(self, arg0: shyft.time_series.TsFixed) -> None:
        ...
    @property
    def wind_speed(self) -> shyft.time_series.TsFixed:
        """
        TsFixed: wind speed
        """
    @wind_speed.setter
    def wind_speed(self, arg0: shyft.time_series.TsFixed) -> None:
        ...
class CellStateId:
    """
    Unique cell pseudo identifier of  a state
    
    A lot of Shyft models can appear in different geometries and
    resolutions.In a few rare cases we would like to store the *state* of
    a model.This would be typical for example for hydrological new - year 1 - sept in Norway.
    To ensure that each cell - state have a unique identifier so that we never risk
    mixing state from different cells or different geometries, we create a pseudo unique
    id that helps identifying unique cell - states given this usage and context.
    
    The primary usage is to identify which cell a specific identified state belongs to.
    """
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, arg0: CellStateId) -> bool:
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, cid: int, x: int, y: int, area: int) -> None:
        """
        construct CellStateId with specified parameters
        """
    def __ne__(self, arg0: CellStateId) -> bool:
        ...
    def __repr__(self) -> str:
        """
        string representation of cell_state_id
        """
    def __str__(self) -> str:
        """
        string representation of cell_state_id
        """
    @property
    def area(self) -> int:
        """
        int: area in [m^2]
        """
    @area.setter
    def area(self, arg0: int) -> None:
        ...
    @property
    def cid(self) -> int:
        """
        int: catchment identifier
        """
    @cid.setter
    def cid(self, arg0: int) -> None:
        ...
    @property
    def x(self) -> int:
        """
        int: x position in [m]
        """
    @x.setter
    def x(self, arg0: int) -> None:
        ...
    @property
    def y(self) -> int:
        """
        int: y position in [m]
        """
    @y.setter
    def y(self, arg0: int) -> None:
        ...
class DrmClient:
    """
    Distributed region model client provides all needed functionality to transfer
    Shyft region-models, and then run simulations/optimizations
    """
    total_clients: typing.ClassVar[int] = 0
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, host_port: str, timeout_ms: int) -> None:
        ...
    def adjust_q(self, mid: str, indexes: list[int], wanted_q: float, start_step: int = 0, scale_range: float = 3.0, scale_eps: float = 0.001, max_iter: int = 300, n_steps: int = 1) -> FlowAdjustResult:
        """
        State adjustment to achieve wanted/observed flow
        
        Args:
            mid (str): model identifier
        
            indexes (List[int]): cell ids that should be adjusted
        
            wanted_q (float): the average flow first time-step we want to achieve, in m**3/s
        
            start_step (int): time-axis start step index, default=0
        
            scale_range (float): default=3, min,max= scale0/scale_range.. scale0*scale_range
        
            scale_eps (float): default 1e-3, stop iteration when scale-change is less than this
        
            max_iter (int): default=300, stop searching for solution after this limit is reached
        
            n_steps (int): default=1, number of steps on time-axis to average to match wanted flow
        
        Returns:
            q_adjust_result: . obtained flow in m**3/s. This can deviate from wanted flow due to model and state constraints. q_r is optimized q and q_0 is unadjusted.
        """
    def cancel_calibration(self, mid: str) -> bool:
        """
        Cancels any ongoing not finished calibration for the specified model
        
        Args:
            mid (str): model identifier
        
        Returns:
            bool: . indicating success
        """
    def check_calibration(self, mid: str) -> CalibrationStatus:
        """
        Check ongoing calibration, return intermediate status with trace, or final status with trace and resulting parameters
        
        Args:
            mid (): model identifier
        
        Returns:
            : . Calibration Status class with information and results
        """
    def clone_model(self, mid: str, new_mid: str) -> bool:
        """
        clone the specified model
        
        Args:
            mid (str): original model
        
            new_mid (str): the model-id for the cloned model
        
        Returns:
            bool: . true if succeeded
        """
    def close(self) -> None:
        """
        Close the connection. It will automatically reopen if needed.
        """
    def copy_model(self, mid: str, new_mid: str) -> bool:
        """
        full copy of the specified model
        
        Args:
            mid (str): original model
        
            new_mid (str): the model-id for the copied model
        
        Returns:
            bool: . true if succeeded
        """
    def create_model(self, mid: str, mtype: RegionModelType, gcd: GeoCellDataVector) -> bool:
        """
        create the model
        
        Args:
            mid (str): model identifier
        
            mtype (enum rmodel_type): rmodel_type.pt_gs_k rmodel_type.pt_gs_k_opt or similar model type
        
            gcd (GeoCellDataVector): strongly typed list of GeoCellData
        
        Returns:
            bool: . true if succeeded
        """
    def fx(self, mid: str, fx_args: str) -> bool:
        """
        Invoke the serverside fx callback, on the exlusively locked model given by mid
        
        Args:
            mid (str): model identifier
        
            fx_args (str): args to the callback, typically a json string
        
        Returns:
            bool: . the server side evaluation result, as pr. callback return value
        """
    def get_catchment_parameter(self, mid: str, cid: int) -> typing.Any:
        """
        Get the effective catchment parameter for the specified catchment
        
        Args:
            mid (str): model identifier
        
            cid (int): catchment identifier
        
        Returns:
            XXXParameter: . The catchment parameter for specified catchment
        """
    def get_charge(self, mid: str, indexes: list[int], index_type: stat_scope = stat_scope.catchment) -> shyft.time_series.TimeSeries:
        """
        get the charge
        
        Args:
            mid (str): model identifier
        
            indexes (List[int]): indexes to be retrieved, the index_type tells if it's cells or catchment indexes
        
            index_type (stat_scope): stat_scope.(cell|catchment) denotes the type of indexes passed. cell returns sum of cell indexes, catchment returns sum of catchments
        
        Returns:
            TimeSeries: . returns sum  for catcment_ids
        """
    def get_description(self, mid: str) -> str:
        """
        returns the user specified description string, typically a json formatted string.
        
        Args:
            mid (str): model identifier
        
        Returns:
            str: . The  description(json string) for the specified model
        """
    def get_discharge(self, mid: str, indexes: list[int], index_type: stat_scope = stat_scope.catchment) -> shyft.time_series.TimeSeries:
        """
        get the discharge
        
        Args:
            mid (str): model identifier
        
            indexes (List[int]): indexes to be retrieved, the index_type tells if it's cells or catchment indexes
        
            index_type (stat_scope): stat_scope.(cell|catchment) denotes the type of indexes passed. cell returns sum of cell indexes, catchment returns sum of catchments
        
        Returns:
            TimeSeries: . returns sum  for catcment_ids
        """
    def get_geo_cell_data(self, mid: str) -> GeoCellDataVector:
        """
        returns the geo cell data vector for the specified model.
        
        Args:
            mid (str): model identifier
        
        Returns:
            GeoCellDataVector: . The GeoCellDataVector for the specified model
        """
    def get_interpolation_parameter(self, mid: str) -> InterpolationParameter:
        """
        returns the current interpolation parameters  used to interpolate region-env to the cells.
        
        Args:
            mid (str): model identifier
        
        Returns:
            InterpolationParameter: . The interpolation parameter for the specified model
        """
    def get_model_ids(self) -> list[str]:
        """
        returns a list of model ids (mids) that is alive and known at the remote server
        
        Returns:
            StringList: . Strongly typed list of strings naming the available models on the drms
        """
    def get_precipitation(self, mid: str, indexes: list[int], index_type: stat_scope = stat_scope.catchment) -> shyft.time_series.TimeSeries:
        """
        get the precipitation
        
        Args:
            mid (str): model identifier
        
            indexes (List[int]): indexes to be retrieved, the index_type tells if it's cells or catchment indexes
        
            index_type (stat_scope): stat_scope.(cell|catchment) denotes the type of indexes passed. cell returns sum of cell indexes, catchment returns sum of catchments
        
        Returns:
            TimeSeries: . returns sum  for catcment_ids
        """
    def get_radiation(self, mid: str, indexes: list[int], index_type: stat_scope = stat_scope.catchment) -> shyft.time_series.TimeSeries:
        """
        get the radiation
        
        Args:
            mid (str): model identifier
        
            indexes (List[int]): indexes to be retrieved, the index_type tells if it's cells or catchment indexes
        
            index_type (stat_scope): stat_scope.(cell|catchment) denotes the type of indexes passed. cell returns sum of cell indexes, catchment returns sum of catchments
        
        Returns:
            TimeSeries: . returns sum  for catcment_ids
        """
    def get_region_env(self, mid: str) -> ARegionEnvironment:
        """
        returns the current region-env for the specified model.
        
        Args:
            mid (str): model identifier
        
        Returns:
            ARegionEnvironment: . The region envirionment for the specified model
        """
    def get_region_parameter(self, mid: str) -> typing.Any:
        """
        Get the region model parameter settings
        
        Args:
            mid (str): model identifier
        
        Returns:
            XXXParameter: . The region parameter for the model
        """
    def get_rel_hum(self, mid: str, indexes: list[int], index_type: stat_scope = stat_scope.catchment) -> shyft.time_series.TimeSeries:
        """
        get the rel_hum
        
        Args:
            mid (str): model identifier
        
            indexes (List[int]): indexes to be retrieved, the index_type tells if it's cells or catchment indexes
        
            index_type (stat_scope): stat_scope.(cell|catchment) denotes the type of indexes passed. cell returns sum of cell indexes, catchment returns sum of catchments
        
        Returns:
            TimeSeries: . returns sum  for catcment_ids
        """
    def get_server_version(self) -> str:
        """
        returns the server version
        
        Returns:
            str: version. Server version string
        """
    def get_snow_sca(self, mid: str, indexes: list[int], index_type: stat_scope = stat_scope.catchment) -> shyft.time_series.TimeSeries:
        """
        get the snow_sca
        
        Args:
            mid (str): model identifier
        
            indexes (List[int]): indexes to be retrieved, the index_type tells if it's cells or catchment indexes
        
            index_type (stat_scope): stat_scope.(cell|catchment) denotes the type of indexes passed. cell returns sum of cell indexes, catchment returns sum of catchments
        
        Returns:
            TimeSeries: . returns sum  for catcment_ids
        """
    def get_snow_swe(self, mid: str, indexes: list[int], index_type: stat_scope = stat_scope.catchment) -> shyft.time_series.TimeSeries:
        """
        get the snow_swe
        
        Args:
            mid (str): model identifier
        
            indexes (List[int]): indexes to be retrieved, the index_type tells if it's cells or catchment indexes
        
            index_type (stat_scope): stat_scope.(cell|catchment) denotes the type of indexes passed. cell returns sum of cell indexes, catchment returns sum of catchments
        
        Returns:
            TimeSeries: . returns sum  for catcment_ids
        """
    def get_state(self, mid: str, indexes: list[int] = []) -> typing.Any:
        """
        Extract cell state for the optionaly specified catchment ids, cids
        
        Args:
            mid (str): model identifier
        
            indexes (IntVector): list of catchment-id's, if empty, extract all
        
        Returns:
            xStateWithIdVector: . strongly typed list of stack state with identifier for the cells
        """
    def get_temperature(self, mid: str, indexes: list[int], index_type: stat_scope = stat_scope.catchment) -> shyft.time_series.TimeSeries:
        """
        get the temperature
        
        Args:
            mid (str): model identifier
        
            indexes (List[int]): indexes to be retrieved, the index_type tells if it's cells or catchment indexes
        
            index_type (stat_scope): stat_scope.(cell|catchment) denotes the type of indexes passed. cell returns sum of cell indexes, catchment returns sum of catchments
        
        Returns:
            TimeSeries: . returns sum  for catcment_ids
        """
    def get_time_axis(self, mid: str) -> shyft.time_series.TimeAxis:
        """
        Get the current time-axis of the model
        
        Args:
            mid (str): model identifier
        
        Returns:
            TimeAxis: . The region model time-axis
        """
    def get_wind_speed(self, mid: str, indexes: list[int], index_type: stat_scope = stat_scope.catchment) -> shyft.time_series.TimeSeries:
        """
        get the wind speed
        
        Args:
            mid (str): model identifier
        
            indexes (List[int]): indexes to be retrieved, the index_type tells if it's cells or catchment indexes
        
            index_type (stat_scope): stat_scope.(cell|catchment) denotes the type of indexes passed. cell returns sum of cell indexes, catchment returns sum of catchments
        
        Returns:
            TimeSeries: . returns sum  for catcment_ids
        """
    def has_catchment_parameter(self, mid: str, cid: int) -> bool:
        """
        Get the current time-axis of the model
        
        Args:
            mid (str): model identifier
        
            cid (int): catchment identifier
        
        Returns:
            bool: . True if catchment has its own parameter setting
        """
    def is_calculated(self, mid: str, catchment_id: int) -> bool:
        """
        using the catchment_calculation_filter to decide if discharge etc. are calculated.
        
        Args:
            mid (str): model identifier
        
            cid (int): catchment id
        
        Returns:
            bool: . true if catchment id is calculated during runs
        """
    def is_cell_env_ts_ok(self, mid: str) -> bool:
        """
        check if all values in cell.env_ts for selected calculation-filter is non-nan, i.e. valid numbers
        
        Args:
            mid (str): model identifier
        
        Returns:
            bool: . true if ok, false if not ok
        """
    def remove_catchment_parameter(self, mid: str, cid: int) -> None:
        """
        Remove the specified catchments local parameter, and set it to the region parameter
        
        Args:
            mid (str): model identifier
        
            cid (int): catchment identifier
        """
    def remove_model(self, mid: str) -> bool:
        """
        remove the specified model
        
        Args:
            mid (str): model identifier
        
        Returns:
            bool: . true if succeeded
        """
    def rename_model(self, mid: str, new_mid: str) -> bool:
        """
        rename the specified model
        
        Args:
            mid (str): model identifier
        
            new_mid (str): the new wanted model-id for the model
        
        Returns:
            bool: . true if succeeded
        """
    def revert_to_initial_state(self, mid: str) -> bool:
        """
        revert model to initial state, notice that this 
        require that the set_current_state_as_initial_state() or run_cells() is called.
        The latter will set initial state if not already done.
        
        Args:
            mid (str): model identifier
        
        Returns:
            bool: . true if succeeded
        """
    def run_cells(self, mid: str, use_ncore: int = 0, start_step: int = 0, n_steps: int = 0) -> bool:
        """
        Run calculation over the specified time axis. Requires that cells and environment have been initialized and interpolated.
        
        Args:
            mid (str): model identifier
        
            use_ncore (int): default 0, means autodetect core, otherwise you can specify 1..n
        
            start_step (int): default 0, from start step of time-axis
        
            n_steps (int): default 0, means run entire time-axis, 1..n means number of steps from start_step
        
        Returns:
            bool: . true if succeeded
        """
    def run_interpolation(self, mid: str, ip_parameter: InterpolationParameter, ta: shyft.time_series.TimeAxis, r_env: ARegionEnvironment, best_effort: bool) -> bool:
        """
        initializes the cell environment and project region environment time_series to cells.
        
        Args:
            mid (str): model identifier
        
            ip_parameter (InterpolationParameter): interpolation parameter
        
            ta (TimeAxis): time axis
        
            r_env (ARegionEnvironment): region environment containing all geo-located sources
        
            best_effort (bool): best effort type interpolation
        
        Returns:
            bool: . true if succeeded
        """
    def set_catchment_calculation_filter(self, mid: str, catchment_id_list: list[int]) -> bool:
        """
        set catchment calculation filter
        
        Args:
            mid (str): model identifier
        
            catchment_id_list (List[int]): list of catchment indexes
        
        Returns:
            bool: . true if succeeded
        """
    def set_catchment_parameter(self, mid: str, parameter: pt_gs_k.PTGSKParameter | pt_ss_k.PTSSKParameter | pt_hs_k.PTHSKParameter | pt_hps_k.PTHPSKParameter | r_pm_gs_k.RPMGSKParameter | pt_st_k.PTSTKParameter | pt_st_hbv.PTSTHBVParameter | r_pt_gs_k.RPTGSKParameter | r_pm_st_k.RPMSTKParameter | r_pmv_st_k.RPMVSTKParameter, cid: int) -> bool:
        """
        set catchment parameter
        
        Args:
            mid (str): model identifier
        
            parameter (XXXXParameter): strongly typed method stack parameter
        
            cid (int): catchment id that should get the supplied parameter
        
        Returns:
            bool: . true if succeeded
        """
    def set_cell_environment(self, mid: str, ta: shyft.time_series.TimeAxis, r_env: ARegionEnvironment) -> bool:
        """
        Set the forcing data cell enviroment (cell.env_ts.* )
        
        The method initializes the cell environment, that keeps temperature, precipitation etc
        for all the cells.
        The region-model time-axis is set to the supplied time-axis, so that
        the the region model is ready to run cells, using this time-axis.
        
        There are strict requirements to the content of the `region_env` parameter:
        
         - rm.cells[i].mid_point()== region_env.temperature[i].mid_point() for all i
         - similar for precipitation,rel_hum,radiation,wind_speed
        
        So same number of forcing data, in the same order and geo position as the cells.
        Tip: If time_axis is equal to the forcing time-axis, it is twice as fast.
        
        
        Args:
            mid (str): model identifier
        
            ta (TimeAxis): specifies the time-axisfor the region-model, and thus the cells
        
            r_env (ARegionEnvironment): A region environment with ready to use forcing data for all the cells.
        
        Returns:
            bool: success. true if successfull, raises exception otherwise
        """
    def set_current_state_as_initial_state(self, mid: str) -> bool:
        """
        set current state as initial state for the region model.
        the method .revert_to_initial_state will restore this state.
        
        
        Args:
            mid (str): model identifier
        
        Returns:
            bool: . true if succeeded
        """
    def set_description(self, mid: str, description: str) -> bool:
        """
        Set the user specified description string, typically a json formatted string.
        
        Args:
            mid (str): model identifier
        
        Returns:
            bool: . True if successful
        """
    def set_region_parameter(self, mid: str, parameter: pt_gs_k.PTGSKParameter | pt_ss_k.PTSSKParameter | pt_hs_k.PTHSKParameter | pt_hps_k.PTHPSKParameter | r_pm_gs_k.RPMGSKParameter | pt_st_k.PTSTKParameter | pt_st_hbv.PTSTHBVParameter | r_pt_gs_k.RPTGSKParameter | r_pm_st_k.RPMSTKParameter | r_pmv_st_k.RPMVSTKParameter) -> bool:
        """
        set region parameter
        
        Args:
            mid (str): model identifier
        
            parameter (XXXXParameter): strongly typed method stack parameter
        
        Returns:
            bool: . true if succeeded
        """
    def set_snow_sca_swe_collection(self, mid: str, catchment_id: int, on_or_off: bool) -> bool:
        """
        enable/disable collection of snow sca|sca for calibration purposes
        
        Args:
            mid (str): model identifier
        
            catchment_id (int): to enable snow calibration for, -1 means turn on/off for all
        
            on_or_off (bool): on_or_off true|or false
        
        Returns:
            bool: . true if succeeded
        """
    def set_state(self, mid: str, state: pt_gs_k.PTGSKStateWithIdVector | pt_ss_k.PTSSKStateWithIdVector | pt_hs_k.PTHSKStateWithIdVector | pt_hps_k.PTHPSKStateWithIdVector | r_pm_gs_k.RPMGSKStateWithIdVector | pt_st_k.PTSTKStateWithIdVector | pt_st_hbv.PTSTHBVStateWithIdVector | r_pt_gs_k.RPTGSKStateWithIdVector | r_pm_st_k.RPMSTKStateWithIdVector | r_pmv_st_k.RPMVSTKStateWithIdVector) -> bool:
        """
        set the cell state
        
        Args:
            mid (str): model identifier
        
            state (StateWithIdVector): strongly typed list of stack state with id vector
        
        Returns:
            bool: . true if succeeded
        """
    def set_state_collection(self, mid: str, catchment_id: int, on_or_off: bool) -> bool:
        """
        enable state collection for specified or all cells
        
        Args:
            mid (str): model identifier
        
            catchment_id (int): to enable state collection for, -1 means turn on/off for all
        
            on_or_off (bool): on_or_off true|or false
        
        Returns:
            bool: . true if succeeded
        """
    def start_calibration(self, mid: str, p_start: pt_gs_k.PTGSKParameter | pt_ss_k.PTSSKParameter | pt_hs_k.PTHSKParameter | pt_hps_k.PTHPSKParameter | r_pm_gs_k.RPMGSKParameter | pt_st_k.PTSTKParameter | pt_st_hbv.PTSTHBVParameter | r_pt_gs_k.RPTGSKParameter | r_pm_st_k.RPMSTKParameter | r_pmv_st_k.RPMVSTKParameter, p_min: pt_gs_k.PTGSKParameter | pt_ss_k.PTSSKParameter | pt_hs_k.PTHSKParameter | pt_hps_k.PTHPSKParameter | r_pm_gs_k.RPMGSKParameter | pt_st_k.PTSTKParameter | pt_st_hbv.PTSTHBVParameter | r_pt_gs_k.RPTGSKParameter | r_pm_st_k.RPMSTKParameter | r_pmv_st_k.RPMVSTKParameter, p_max: pt_gs_k.PTGSKParameter | pt_ss_k.PTSSKParameter | pt_hs_k.PTHSKParameter | pt_hps_k.PTHPSKParameter | r_pm_gs_k.RPMGSKParameter | pt_st_k.PTSTKParameter | pt_st_hbv.PTSTHBVParameter | r_pt_gs_k.RPTGSKParameter | r_pm_st_k.RPMSTKParameter | r_pmv_st_k.RPMVSTKParameter, spec: TargetSpecificationVector, opt: CalibrationOption) -> bool:
        """
        Start calibration with specified parameters for the model given by 'mid'
        
        Args:
            mid (): model identifier
        
            p_start (): starting parameters
        
            p_min (): min-range for parameters
        
            p_max (): max-range for parameters
        
            spec (): goal function specification, including reference series and weights
        
            opt (): optimizer options, like BOBYQA, GLOBAL,SCEUA or DREAM etc.
        
        Returns:
            bool: . indicating success
        """
    @property
    def host_port(self) -> str:
        """
        str: Endpoint network address of the remote server.
        """
    @property
    def is_open(self) -> bool:
        """
        bool: If the connection to the remote server is (still) open.
        """
    @property
    def reconnect_count(self) -> int:
        """
        int: Number of reconnects to the remote server that have been performed.
        """
    @property
    def timeout_ms(self) -> int:
        """
        int: Timout for remote server operations, in number milliseconds.
        """
class DrmServer:
    """
    The DRMS, distributed region model server provides the server part that implements
    the needed functionality to create, run and get results from hydrology region-models
    .Together with the DrmClient, it allow the Shyft hydrology region-model operations to be
    scaled out on several servers, giving providing support for system and analysis of any size
    
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, config: shyft.time_series.ServerConfig = ...) -> None:
        """
        Creates a server object with stale connection detect parameters
        """
    def get_listening_port(self) -> int:
        """
        returns the port number it's listening at for serving incoming request
        """
    def get_max_connections(self) -> int:
        """
        returns the maximum number of connections to be served concurrently
        """
    def get_model(self, arg0: str) -> typing.Any:
        """
        returns the named model, so it can be inspected/modified
        WIP: needs to be sure that there is no activity on the model from the remote side
        
        
        Returns:
            Model: . Of the type as set by the client
        """
    def get_model_ids(self) -> list[str]:
        """
        returns a list of model ids (mids) that is alive and known in the server
        
        Returns:
            StringList: . Strongly typed list of strings naming the available models on the drms
        """
    def is_running(self) -> bool:
        """
        true if server is listening and running
        
        See also:
            start_server()
        """
    def set_listening_ip(self, ip: str) -> None:
        """
        set the listening port for the service
        
        Args:
            ip (str): ip or host-name to start listening on
        
        Returns:
            None: nothing. 
        """
    def set_listening_port(self, port_no: int) -> None:
        """
        set the listening port for the service
        
        Args:
            port_no (int): a valid and available tcp-ip port number to listen on.
            typically it could be 20000 (avoid using official reserved numbers)
        
        Returns:
            None: nothing. 
        """
    def set_max_connections(self, max_connect: int) -> None:
        """
        limits simultaneous connections to the server (it's multithreaded, and uses on thread pr. connect)
        
        Args:
            max_connect (int): maximum number of connections before denying more connections
        
        See also:
            get_max_connections()
        """
    def start_server(self) -> int:
        """
        start server listening in background, and processing messages
        
        See also:
            set_listening_port(port_no),is_running
        
        Returns:
            in: port_no. the port used for listening operations, either the value as by set_listening_port, or if it was unspecified, a new available port
        """
    def stop_server(self, ms: int = 2000) -> None:
        """
        stop serving connections, gracefully.
        
        See also:
            start_server()
        """
    @property
    def alive_connections(self) -> int:
        """
        int: returns currently alive connections to the server
        """
    @property
    def fx(self) -> typing.Callable[[str, str], bool]:
        """
        Callable[[str,str],bool]: server-side callable function(lambda) that takes two parameters:
        mid :  the model id
        fx_arg: arbitrary string to pass to the server-side function
        The server-side fx is called when the client (or web-api) invoke the c.fx(mid,fx_arg).
        The signature of the callback function should be callback(mid:str, fx_arg:str)->bool
        This feature is simply enabling the users to tailor server-side functionality in python!
        NOTE: Currently we do take exclusive lock on the model id, so that we avoid race conditions
        Examples:
        
        >>> from shyft.hydrology import DrmServer
        >>> s=DrmServer()
        >>> def my_fx(mid:str, fx_arg:str)->bool:
        >>>     print(f'invoked with mid={mid} fx_arg={fx_arg}')
        >>>   # note we can use captured Server s here!>>>     return True
        >>> # and then bind the function to the callback
        >>> s.fx=my_fx
        >>> s.start_server()
        >>> : # later using client from anywhere to invoke the call
        >>> fx_result=c.fx('my_model_id', 'my_args')
        """
    @fx.setter
    def fx(self, arg0: typing.Callable[[str, str], bool]) -> None:
        ...
class FlowAdjustResult:
    """
    The result type of region-model .adjust_state_to_target_flow(..) method
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self) -> None:
        ...
    @property
    def diagnostics(self) -> str:
        """
        str: If tuning failed, the diagnostics of failure, zero length/empty if success
        """
    @diagnostics.setter
    def diagnostics(self, arg0: str) -> None:
        ...
    @property
    def q_0(self) -> float:
        """
        float: The flow m3/s from selected catchments before tuning
        """
    @q_0.setter
    def q_0(self, arg0: float) -> None:
        ...
    @property
    def q_r(self) -> float:
        """
        float: The obtaioned flow m3/s after tuning
        """
    @q_r.setter
    def q_r(self, arg0: float) -> None:
        ...
class FreeWaterEvaporationCalculator:
    """
    Evapotranspiration model, Penman or combination method equation for open water, ponds and reservoirs, PM
    reference ::
    
        L. Dingman Penmann evapotranspiration
    [mm/s] units.
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def evaporation(self, response: FreeWaterEvaporationResponse, dt: shyft.time_series.time, swin_radiation: float, lw_radiation: float, tempmax: float, tempmin: float, rhumidity: float, water_area: float, albedo: float, elevation: float, windspeed: float) -> None:
        """
        calculates evapotranspiration, updating response
        """
class FreeWaterEvaporationResponse:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self) -> None:
        ...
    @property
    def et(self) -> float:
        """
        float: mm/h
        """
    @et.setter
    def et(self, arg0: float) -> None:
        ...
class GammaSnowCalculator:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self) -> None:
        ...
    def step(self, state: GammaSnowState, response: GammaSnowResponse, t: shyft.time_series.time, dt: shyft.time_series.time, parameter: GammaSnowParameter, temperature: float, radiation: float, precipitation: float, wind_speed: float, rel_hum: float, forest_fraction: float, altitude: float) -> None:
        """
        Step the snow model forward from time t to t+dt, given state, parameters and input.
        
        Updates the state and response upon return.
        
        
        Args:
            state (GammaSnowState):  param state state of type S,in/out, ref template parameters
        
            response (GammaSnowResponse): the response , out parameter.
        
            t (float): ttemperature degC, considered constant over timestep dt
        
            dt (time): length of time-step, seconds
        
            param (GammaSnowParameter):  response result of type R, output only, ref. template parameters
        
        
            radiation (float):  in W/m2, considered constant over timestep
        
            precipitation (float):  in mm/h
        
            wind_speed (float):  in m/s
        
            rel_hum (float):  range  0..1
        
            forest_fraction (float): range 0..1, influences calculation of effective snow_cv
        
            altitude (float):  0..x [m], influences calculation of effective_snow_cv
        """
class GammaSnowParameter:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, winter_end_day_of_year: int, initial_bare_ground_fraction: float = 0.04, snow_cv: float = 0.4, tx: float = -0.5, wind_scale: float = 2.0, wind_const: float = 1.0, max_water: float = 0.1, surface_magnitude: float = 30.0, max_albedo: float = 0.9, min_albedo: float = 0.6, fast_albedo_decay_rate: float = 5.0, slow_albedo_decay_rate: float = 5.0, snowfall_reset_depth: float = 0.4, glacier_albedo: float = False) -> None:
        ...
    def effective_snow_cv(self, forest_fraction: float, altitude: float) -> float:
        """
        returns the effective snow cv, taking the forest_fraction and altitude into the equations using corresponding factors
        """
    def is_snow_season(self, t: shyft.time_series.time) -> bool:
        """
        returns true if specified t is within the snow season, e.g. sept.. winder_end_day_of_year
        """
    def is_start_melt_season(self, t: shyft.time_series.time, dt: shyft.time_series.time = 0) -> bool:
        """
        true if specified interval t day of year is wind_end_day_of_year
        """
    @property
    def calculate_iso_pot_energy(self) -> bool:
        """
        float: Whether or not to calculate the potential energy flux,default=false
        """
    @calculate_iso_pot_energy.setter
    def calculate_iso_pot_energy(self, arg0: bool) -> None:
        ...
    @property
    def fast_albedo_decay_rate(self) -> float:
        """
        float: Albedo decay rate during melt [days],default=5.0
        """
    @fast_albedo_decay_rate.setter
    def fast_albedo_decay_rate(self, arg0: float) -> None:
        ...
    @property
    def glacier_albedo(self) -> float:
        """
        float: Glacier ice fixed albedo, default=0.4
        """
    @glacier_albedo.setter
    def glacier_albedo(self, arg0: float) -> None:
        ...
    @property
    def initial_bare_ground_fraction(self) -> float:
        """
        float: Bare ground fraction at melt onset,default= 0.04
        """
    @initial_bare_ground_fraction.setter
    def initial_bare_ground_fraction(self, arg0: float) -> None:
        ...
    @property
    def max_albedo(self) -> float:
        """
        float: Maximum albedo value,default=0.9
        """
    @max_albedo.setter
    def max_albedo(self, arg0: float) -> None:
        ...
    @property
    def max_water(self) -> float:
        """
        float: Maximum liquid water content,default=0.1
        """
    @max_water.setter
    def max_water(self, arg0: float) -> None:
        ...
    @property
    def min_albedo(self) -> float:
        """
        float: Minimum albedo value,default=0.6
        """
    @min_albedo.setter
    def min_albedo(self, arg0: float) -> None:
        ...
    @property
    def n_winter_days(self) -> int:
        """
        int: number of winter-days, default 221
        """
    @n_winter_days.setter
    def n_winter_days(self, arg0: int) -> None:
        ...
    @property
    def slow_albedo_decay_rate(self) -> float:
        """
        float: Albedo decay rate in cold conditions [days],default=5.0
        """
    @slow_albedo_decay_rate.setter
    def slow_albedo_decay_rate(self, arg0: float) -> None:
        ...
    @property
    def snow_cv(self) -> float:
        """
        float: Spatial coefficient variation of fresh snowfall, default= 0.4
        """
    @snow_cv.setter
    def snow_cv(self, arg0: float) -> None:
        ...
    @property
    def snow_cv_altitude_factor(self) -> float:
        """
        float: default=0.0, [1/m]
        	 the effective snow_cv gets an additional value of altitude[m]* snow_cv_altitude_factor
        """
    @snow_cv_altitude_factor.setter
    def snow_cv_altitude_factor(self, arg0: float) -> None:
        ...
    @property
    def snow_cv_forest_factor(self) -> float:
        """
        float: default=0.0, [ratio]
        	the effective snow_cv gets an additional value of geo.forest_fraction()*snow_cv_forest_factor
        """
    @snow_cv_forest_factor.setter
    def snow_cv_forest_factor(self, arg0: float) -> None:
        ...
    @property
    def snowfall_reset_depth(self) -> float:
        """
        float: Snowfall required to reset albedo [mm],default=5.0
        """
    @snowfall_reset_depth.setter
    def snowfall_reset_depth(self, arg0: float) -> None:
        ...
    @property
    def surface_magnitude(self) -> float:
        """
        float: Surface layer magnitude,default=30.0
        """
    @surface_magnitude.setter
    def surface_magnitude(self, arg0: float) -> None:
        ...
    @property
    def tx(self) -> float:
        """
        float: default= -0.5 [degC]
        """
    @tx.setter
    def tx(self, arg0: float) -> None:
        ...
    @property
    def wind_const(self) -> float:
        """
        float: Intercept in turbulent wind function,default=1.0
        """
    @wind_const.setter
    def wind_const(self, arg0: float) -> None:
        ...
    @property
    def wind_scale(self) -> float:
        """
        float: Slope in turbulent wind function default=2.0 [m/s]
        """
    @wind_scale.setter
    def wind_scale(self, arg0: float) -> None:
        ...
    @property
    def winter_end_day_of_year(self) -> int:
        """
        int: Last day of accumulation season,default= 100
        """
    @winter_end_day_of_year.setter
    def winter_end_day_of_year(self, arg0: int) -> None:
        ...
class GammaSnowResponse:
    """
    The response(output) from gamma-snow for one time-step
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self) -> None:
        ...
    @property
    def outflow(self) -> float:
        """
        float:Water out from the snow pack [mm/h]
        """
    @outflow.setter
    def outflow(self, arg0: float) -> None:
        ...
    @property
    def sca(self) -> float:
        """
        float:Snow covered area [0..1]
        """
    @sca.setter
    def sca(self, arg0: float) -> None:
        ...
    @property
    def storage(self) -> float:
        """
        float:Snow storage in [mm] over the area
        """
    @storage.setter
    def storage(self, arg0: float) -> None:
        ...
class GammaSnowState:
    """
    The state description of the GammaSnow routine
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, albedo: float = 0.4, lwc: float = 0.1, surface_heat: float = 30000.0, alpha: float = 1.26, sdc_melt_mean: float = 0.0, acc_melt: float = 0.0, iso_pot_energy: float = 0.0, temp_swe: float = 0.0) -> None:
        ...
    @property
    def acc_melt(self) -> float:
        """
        float:acc_melt (Accumulated melt depth) [mm],default = 0.0
        """
    @acc_melt.setter
    def acc_melt(self, arg0: float) -> None:
        ...
    @property
    def albedo(self) -> float:
        """
        float:albedo (Broadband snow reflectivity fraction),default = 0.4
        """
    @albedo.setter
    def albedo(self, arg0: float) -> None:
        ...
    @property
    def alpha(self) -> float:
        """
        float:alpha (Dynamic shape state in the SDC),default = 1.26
        """
    @alpha.setter
    def alpha(self, arg0: float) -> None:
        ...
    @property
    def iso_pot_energy(self) -> float:
        """
        float:iso_pot_energy (Accumulated energy assuming isothermal snow surface) [J/m2],default = 0.0
        """
    @iso_pot_energy.setter
    def iso_pot_energy(self, arg0: float) -> None:
        ...
    @property
    def lwc(self) -> float:
        """
        float:lwc (liquid water content) [mm],default = 0.1
        """
    @lwc.setter
    def lwc(self, arg0: float) -> None:
        ...
    @property
    def sdc_melt_mean(self) -> float:
        """
        float:sdc_melt_mean  (Mean snow storage at melt onset) [mm],default = 0.0
        """
    @sdc_melt_mean.setter
    def sdc_melt_mean(self, arg0: float) -> None:
        ...
    @property
    def surface_heat(self) -> float:
        """
        float:surface_heat (Snow surface cold content) [J/m2],default = 30000.0
        """
    @surface_heat.setter
    def surface_heat(self, arg0: float) -> None:
        ...
    @property
    def temp_swe(self) -> float:
        """
        float:temp_swe (Depth of temporary new snow layer during spring) [mm],default = 0.0
        """
    @temp_swe.setter
    def temp_swe(self, arg0: float) -> None:
        ...
class GeoCellData:
    """
    Represents common constant geo_cell properties across several possible models and cell assemblies.
    The idea is that most of our algorithms uses one or more of these properties,
    so we provide a common aspect that keeps this together.
    Currently it keep these items:
    - mid-point geo_point, (x,y,z) (default 0)
    - TIN data 
    - the area in m^2, (default 1000 x 1000 m^2)
    - land_type_fractions (unspecified=1)
    - catchment_id   def (-1)
    - radiation_slope_factor def 0.9
    - routing_info def(0,0.0), i.e. not routed and hydrological distance=0.0m
    """
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, arg0: GeoCellData) -> bool:
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, mid_point: shyft.time_series.GeoPoint, area: float, catchment_id: int, radiation_slope_factor: float = 0.9, land_type_fractions: LandTypeFractions = ..., routing_info: RoutingInfo = ...) -> None:
        """
        Constructs a GeoCellData with all parameters specified
        
        Args:
            mid_point (GeoPoint): specifies the x,y,z mid-point of the cell-area
        
            area (float): area in unit [m^2]
        
            catchment_id (int): catchment-id that this cell is a part of
        
            radiation_slope_factor (float): aspect dependent factor used to calculate the effective radiation for this cell,range 0.0..1.0
        
            land_type_fractions (LandTypeFractions): specifies the fractions of glacier, forrest, lake and reservoirs for this cell
        
            routing_info (RoutingInfo): Specifies the destination routing network-node and velocity for this cell
        """
    @typing.overload
    def __init__(self, p1: shyft.time_series.GeoPoint, p2: shyft.time_series.GeoPoint, p3: shyft.time_series.GeoPoint, epsg_id: int, catchment_id: int, land_type_fractions: LandTypeFractions = ..., routing_info: RoutingInfo = ...) -> None:
        """
        Constructs a TIN-based GeoCellData with all parameters specified, slope/aspect etc. is based on TIN
        
        Args:
            p1 (GeoPoint): specifies the first vertex in TIN
        
            p2 (GeoPoint): specifies the second vertex in TIN
        
            p3 (GeoPoint): specifies the third vertex in TIN
        
            epsg_id (int): specifies the geo projection as epsg-id, used for projection from cartesian coord to long-lat  algorithms that needs this(e.g. radiation, TIN-models)
        
            catchment_id (int): catchment-id that this cell is a part of
        
            land_type_fractions (LandTypeFractions): specifies the fractions of glacier, forrest, lake and reservoirs for this cell
        
            routing_info (RoutingInfo): Specifies the destination routing network-node and velocity for this cell
        """
    def __ne__(self, arg0: GeoCellData) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __str__(self) -> str:
        ...
    def area(self) -> float:
        """
        returns the effective area, as projected to the horisontal plane, in m^2
        """
    def aspect(self) -> float:
        """
        returns aspect, deg
        """
    def catchment_id(self) -> int:
        """
        returns the current catchment_id
        """
    def is_tin(self) -> bool:
        """
        returns true if the geo-cell is fully specified as a TIN model
        """
    def land_type_fractions_info(self) -> LandTypeFractions:
        """
        land_type_fractions
        """
    def mid_point(self) -> shyft.time_series.GeoPoint:
        """
        returns the mid_point
        """
    def radiation_slope_factor(self) -> float:
        ...
    def set_catchment_id(self, catchment_id: int) -> None:
        """
        Set the catchment_id to specified value
        Note: Only use this method *before* a region-model is created
        
        Args:
            catchment_id (int): Catchment-id for this cell
        """
    def set_land_type_fractions(self, ltf: LandTypeFractions) -> None:
        """
        set new LandTypeFractions
        """
    def set_tin_data(self, vertexes: shyft.time_series.GeoPointVector) -> None:
        """
        set TIN data from vector of vertexes
        """
    def slope(self) -> float:
        """
        returns slope, deg
        """
    def surface_area(self) -> float:
        """
        returns the real surface area in m^2
        """
    def vertexes(self) -> shyft.time_series.GeoPointVector:
        """
        returns vector of tin vertexes
        """
    @property
    def epsg_id(self) -> int:
        """
        int: the epsg-id that should consistently identify the geo projection for all cells in a region model
        """
    @epsg_id.setter
    def epsg_id(self, arg0: int) -> None:
        ...
    @property
    def routing_info(self) -> RoutingInfo:
        """
        RoutingInfo: routing information for the cell keep destination id and hydrological distance to destination
        """
    @routing_info.setter
    def routing_info(self, arg0: RoutingInfo) -> None:
        ...
class GeoCellDataClient:
    """
    The client api for the shyft.hydrology geo_cell_data model service.
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @staticmethod
    def get_model_infos(*args, **kwargs) -> list[shyft.time_series.ModelInfo]:
        """
        returns all or selected model-info objects based on model-identifiers(mids)
        
        Args:
            mids (IntVector): empty = all, or a list of known exisiting model-identifiers
        
            created_in (UtcPeriod): For which period you are interested in model-infos.
        
        Returns:
            ModelInfoVector: model_infos. Strongly typed list of ModelInfo
        """
    def __init__(self, host_port: str, timeout_ms: int) -> None:
        """
        Creates a python client that can communicate with the corresponding server
        """
    def close(self) -> None:
        """
        Close the connection. It will automatically reopen if needed.
        """
    def read_model(self, mid: int) -> GeoCellDataModel:
        """
        Read and return the model for specified model-identifier (mid)
        
        Args:
            mid (int): the model-identifer for the wanted model
        
        Returns:
            Model: m. The resulting model from the server
        """
    def read_models(self, mids: list[int]) -> GeoCellDataModelVector:
        """
        Read and return the model for specified model-identifier (mid)
        
        Args:
            mids (list[int]): A strongly typed list of ints, the model-identifers for the wanted models
        
        Returns:
            Model: m. The resulting model from the server
        """
    def remove_model(self, mid: int) -> int:
        """
        Remove the specified model bymodel-identifier (mid)
        
        Args:
            mid (int): the model-identifer for the wanted model
        
        Returns:
            int: ec. 0 or error-code?
        """
    def store_model(self, m: GeoCellDataModel, mi: shyft.time_series.ModelInfo) -> int:
        """
        Store the model to backend, if m.id==0 then a new unique model-info is created and used
        
        Args:
            m (Model): The model to store
        
            mi (ModelInfo): The model-info to store for the model
        
        Returns:
            int: mid. model-identifier for the stored model and model-info
        """
    def update_model_info(self, mid: int, mi: shyft.time_series.ModelInfo) -> bool:
        """
        Update the model-info for specified model-identifier(mid)
        
        Args:
            mid (int): model-identifer
        
            mi (ModelInfo): The new updated model-info
        
        Returns:
            bool: ok. true if success
        """
    @property
    def host_port(self) -> str:
        """
        str: Endpoint network address of the remote server.
        """
    @property
    def is_open(self) -> bool:
        """
        bool: If the connection to the remote server is (still) open.
        """
    @property
    def reconnect_count(self) -> int:
        """
        int: Number of reconnects to the remote server that have been performed.
        """
    @property
    def timeout_ms(self) -> int:
        """
        int: Timout for remote server operations, in number milliseconds.
        """
class GeoCellDataModel:
    """
    A model, with mandatory identifier and geo-cell-data vector.
    The optional filled in epsg, polygon and json members can provide
    useful information on how it was created, or can be presented.
    """
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, arg0: GeoCellDataModel) -> bool:
        ...
    def __init__(self) -> None:
        ...
    def __ne__(self, arg0: GeoCellDataModel) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __str__(self) -> str:
        ...
    @property
    def epsg_id(self) -> int:
        """
        int: EPSG code for the polygon coordinate reference system
        """
    @epsg_id.setter
    def epsg_id(self, arg0: int) -> None:
        ...
    @property
    def geo_cell_data(self) -> GeoCellDataVector:
        """
        GeoCellDataVector: a strongly typed list of geo-cell-data, the geo-personality of Shyft cells
        """
    @geo_cell_data.setter
    def geo_cell_data(self, arg0: GeoCellDataVector) -> None:
        ...
    @property
    def id(self) -> int:
        """
        int: unique model identifier
        """
    @id.setter
    def id(self, arg0: int) -> None:
        ...
    @property
    def json(self) -> str:
        """
        str: extra useful json formatted information
        """
    @json.setter
    def json(self, arg0: str) -> None:
        ...
    @property
    def polygon(self) -> shyft.time_series.GeoPointVector:
        """
        GeoPointVector: convex hull for the geo_cell_data
        """
    @polygon.setter
    def polygon(self, arg0: shyft.time_series.GeoPointVector) -> None:
        ...
class GeoCellDataModelVector:
    """
    A strongly typed list of GeoCellDataModel's
    """
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __bool__(self) -> bool:
        """
        Check whether the list is nonempty
        """
    @typing.overload
    def __delitem__(self, arg0: int) -> None:
        """
        Delete the list elements at index ``i``
        """
    @typing.overload
    def __delitem__(self, arg0: slice) -> None:
        """
        Delete list elements using a slice object
        """
    def __eq__(self, arg0: GeoCellDataModelVector) -> bool:
        ...
    @typing.overload
    def __getitem__(self, s: slice) -> GeoCellDataModelVector:
        """
        Retrieve list elements using a slice object
        """
    @typing.overload
    def __getitem__(self, arg0: int) -> GeoCellDataModel:
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, arg0: GeoCellDataModelVector) -> None:
        """
        Copy constructor
        """
    @typing.overload
    def __init__(self, arg0: typing.Iterable) -> None:
        ...
    def __iter__(self) -> typing.Iterator[GeoCellDataModel]:
        ...
    def __len__(self) -> int:
        ...
    def __ne__(self, arg0: GeoCellDataModelVector) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    @typing.overload
    def __setitem__(self, arg0: int, arg1: GeoCellDataModel) -> None:
        ...
    @typing.overload
    def __setitem__(self, arg0: slice, arg1: GeoCellDataModelVector) -> None:
        """
        Assign list elements using a slice object
        """
    def __str__(self) -> str:
        ...
    def append(self, x: GeoCellDataModel) -> None:
        """
        Add an item to the end of the list
        """
    def clear(self) -> None:
        """
        Clear the contents
        """
    @typing.overload
    def extend(self, L: GeoCellDataModelVector) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    @typing.overload
    def extend(self, L: typing.Iterable) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    def insert(self, i: int, x: GeoCellDataModel) -> None:
        """
        Insert an item at a given position.
        """
    @typing.overload
    def pop(self) -> GeoCellDataModel:
        """
        Remove and return the last item
        """
    @typing.overload
    def pop(self, i: int) -> GeoCellDataModel:
        """
        Remove and return the item at index ``i``
        """
    def size(self) -> int:
        ...
class GeoCellDataServer:
    """
    The server-side component for the shyft.hydrology geo_cell_data model repository.
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, root_dir: str, config: shyft.time_series.ServerConfig = ...) -> None:
        """
        Creates a server object that serves models from root_dir.
        The root_dir will be create if it does not exists.
        
        Args:
            root_dir (str): Path to the root-directory that keeps/will keep the model-files
        
            config (ServerConfig): Configuration of the server
        """
    def get_listening_port(self) -> int:
        """
        returns the port number it's listening at for serving incoming request
        """
    def get_max_connections(self) -> int:
        """
        returns the maximum number of connections to be served concurrently
        """
    def is_running(self) -> bool:
        """
        true if server is listening and running
        
        See also:
            start_server()
        """
    def set_listening_ip(self, ip: str) -> None:
        """
        set the listening port for the service
        
        Args:
            ip (str): ip or host-name to start listening on
        
        Returns:
            None: nothing. 
        """
    def set_listening_port(self, port_no: int) -> None:
        """
        set the listening port for the service
        
        Args:
            port_no (int): a valid and available tcp-ip port number to listen on.
            typically it could be 20000 (avoid using official reserved numbers)
        
        Returns:
            None: nothing. 
        """
    def set_max_connections(self, max_connect: int) -> None:
        """
        limits simultaneous connections to the server (it's multithreaded, and uses on thread pr. connect)
        
        Args:
            max_connect (int): maximum number of connections before denying more connections
        
        See also:
            get_max_connections()
        """
    def start_server(self) -> int:
        """
        start server listening in background, and processing messages
        
        See also:
            set_listening_port(port_no),is_running
        
        Returns:
            in: port_no. the port used for listening operations, either the value as by set_listening_port, or if it was unspecified, a new available port
        """
    def stop_server(self, timeout: int = 1000) -> None:
        """
        stop serving connections, gracefully.
        
        See also:
            start_server()
        """
    @property
    def stale_connection_close_count(self) -> int:
        """
        int: returns count of connection closed due to stale/no communication activity
        """
class GeoCellDataVector:
    """
    A vector, list, of GeoCellData
    """
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @staticmethod
    def deserialize(arg0: list[str]) -> typing.Any:
        """
        deserialize from a binary to GeoCellDataVector
        """
    def __bool__(self) -> bool:
        """
        Check whether the list is nonempty
        """
    @typing.overload
    def __delitem__(self, arg0: int) -> None:
        """
        Delete the list elements at index ``i``
        """
    @typing.overload
    def __delitem__(self, arg0: slice) -> None:
        """
        Delete list elements using a slice object
        """
    def __eq__(self, arg0: GeoCellDataVector) -> bool:
        ...
    @typing.overload
    def __getitem__(self, s: slice) -> GeoCellDataVector:
        """
        Retrieve list elements using a slice object
        """
    @typing.overload
    def __getitem__(self, arg0: int) -> GeoCellData:
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, arg0: GeoCellDataVector) -> None:
        """
        Copy constructor
        """
    @typing.overload
    def __init__(self, arg0: typing.Iterable) -> None:
        ...
    def __iter__(self) -> typing.Iterator[GeoCellData]:
        ...
    def __len__(self) -> int:
        ...
    def __ne__(self, arg0: GeoCellDataVector) -> bool:
        ...
    @typing.overload
    def __setitem__(self, arg0: int, arg1: GeoCellData) -> None:
        ...
    @typing.overload
    def __setitem__(self, arg0: slice, arg1: GeoCellDataVector) -> None:
        """
        Assign list elements using a slice object
        """
    def append(self, x: GeoCellData) -> None:
        """
        Add an item to the end of the list
        """
    def clear(self) -> None:
        """
        Clear the contents
        """
    @typing.overload
    def extend(self, L: GeoCellDataVector) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    @typing.overload
    def extend(self, L: typing.Iterable) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    def insert(self, i: int, x: GeoCellData) -> None:
        """
        Insert an item at a given position.
        """
    def polygon_buffer(self, distance: float) -> shyft.time_series.GeoPointVector:
        """
        Computes the polygon_buffer of tin points, or mid-points, using boost::geometry::buffer algorithm
        If number of points is large, currently > 50,  a reduction algorithm using convex_hull is first applied
        
        Args:
            distance (float): metric[m] buffer distance
        
        Returns:
            GeoPointVector: polygon. A closed polygon envelope around the cells in the model
        """
    @typing.overload
    def pop(self) -> GeoCellData:
        """
        Remove and return the last item
        """
    @typing.overload
    def pop(self, i: int) -> GeoCellData:
        """
        Remove and return the item at index ``i``
        """
    def serialize(self) -> typing.Any:
        """
        serialize to a binary byte vector
        """
    def size(self) -> int:
        ...
class GeoPointSource:
    """
    GeoPointSource contains common properties, functions
    for the point sources in Shyft.
    Typically it contains a GeoPoint (3d position), plus a time-series
    """
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, arg0: GeoPointSource) -> bool:
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, midpoint: shyft.time_series.GeoPoint, ts: shyft.time_series.TimeSeries) -> None:
        ...
    @typing.overload
    def __init__(self, geo_ts: shyft.time_series.GeoTimeSeries) -> None:
        ...
    def __ne__(self, arg0: GeoPointSource) -> bool:
        ...
    def mid_point(self) -> shyft.time_series.GeoPoint:
        """
        returns a copy of mid_point
        """
    @property
    def geo_ts(self) -> shyft.time_series.GeoTimeSeries:
        """
        GeoTimeSeries: returns a GeoTimeSeries  where the geo-point are copied, but time-series are by-reference
        """
    @property
    def mid_point_(self) -> shyft.time_series.GeoPoint:
        """
        reference to internal mid_point
        """
    @mid_point_.setter
    def mid_point_(self, arg0: shyft.time_series.GeoPoint) -> None:
        ...
    @property
    def ts(self) -> shyft.time_series.TimeSeries:
        """
        TimeSeries: time-series
        """
    @ts.setter
    def ts(self, arg1: shyft.time_series.TimeSeries) -> None:
        ...
    @property
    def uid(self) -> str:
        """
        str: user specified identifier, string
        """
    @uid.setter
    def uid(self, arg0: str) -> None:
        ...
class GeoPointSourceVector:
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __bool__(self) -> bool:
        """
        Check whether the list is nonempty
        """
    @typing.overload
    def __delitem__(self, arg0: int) -> None:
        """
        Delete the list elements at index ``i``
        """
    @typing.overload
    def __delitem__(self, arg0: slice) -> None:
        """
        Delete list elements using a slice object
        """
    def __eq__(self, arg0: GeoPointSourceVector) -> bool:
        ...
    @typing.overload
    def __getitem__(self, s: slice) -> GeoPointSourceVector:
        """
        Retrieve list elements using a slice object
        """
    @typing.overload
    def __getitem__(self, arg0: int) -> GeoPointSource:
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, arg0: GeoPointSourceVector) -> None:
        """
        Copy constructor
        """
    @typing.overload
    def __init__(self, arg0: typing.Iterable) -> None:
        ...
    def __iter__(self) -> typing.Iterator[GeoPointSource]:
        ...
    def __len__(self) -> int:
        ...
    def __ne__(self, arg0: GeoPointSourceVector) -> bool:
        ...
    @typing.overload
    def __setitem__(self, arg0: int, arg1: GeoPointSource) -> None:
        ...
    @typing.overload
    def __setitem__(self, arg0: slice, arg1: GeoPointSourceVector) -> None:
        """
        Assign list elements using a slice object
        """
    def append(self, x: GeoPointSource) -> None:
        """
        Add an item to the end of the list
        """
    def clear(self) -> None:
        """
        Clear the contents
        """
    @typing.overload
    def extend(self, L: GeoPointSourceVector) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    @typing.overload
    def extend(self, L: typing.Iterable) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    def insert(self, i: int, x: GeoPointSource) -> None:
        """
        Insert an item at a given position.
        """
    @typing.overload
    def pop(self) -> GeoPointSource:
        """
        Remove and return the last item
        """
    @typing.overload
    def pop(self, i: int) -> GeoPointSource:
        """
        Remove and return the item at index ``i``
        """
    def size(self) -> int:
        ...
    def values_at_time(self, arg0: shyft.time_series.time) -> list[float]:
        ...
class GlacierMeltParameter:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, dtf: float, direct_response: float = 0.0) -> None:
        ...
    def __repr__(self) -> str:
        ...
    def __str__(self) -> str:
        ...
    @property
    def direct_response(self) -> float:
        """
        float: fraction that goes as direct response, (1-fraction) is routed through soil/kirchner routine,default=0.0
        """
    @direct_response.setter
    def direct_response(self, arg0: float) -> None:
        ...
    @property
    def dtf(self) -> float:
        """
        float: degree timestep factor, default=6.0 [mm/day/degC]
        """
    @dtf.setter
    def dtf(self, arg0: float) -> None:
        ...
class GoalFunctionClient:
    """
    The client api for the shyft.hydrology goal-function model service.
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @staticmethod
    def get_model_infos(*args, **kwargs) -> list[shyft.time_series.ModelInfo]:
        """
        returns all or selected model-info objects based on model-identifiers(mids)
        
        Args:
            mids (IntVector): empty = all, or a list of known exisiting model-identifiers
        
            created_in (UtcPeriod): For which period you are interested in model-infos.
        
        Returns:
            ModelInfoVector: model_infos. Strongly typed list of ModelInfo
        """
    def __init__(self, host_port: str, timeout_ms: int) -> None:
        """
        Creates a python client that can communicate with the corresponding server
        """
    def close(self) -> None:
        """
        Close the connection. It will automatically reopen if needed.
        """
    def read_model(self, mid: int) -> GoalFunctionModel:
        """
        Read and return the model for specified model-identifier (mid)
        
        Args:
            mid (int): the model-identifer for the wanted model
        
        Returns:
            Model: m. The resulting model from the server
        """
    def read_models(self, mids: list[int]) -> GoalFunctionModelVector:
        """
        Read and return the model for specified model-identifier (mid)
        
        Args:
            mids (list[int]): A strongly typed list of ints, the model-identifers for the wanted models
        
        Returns:
            Model: m. The resulting model from the server
        """
    def remove_model(self, mid: int) -> int:
        """
        Remove the specified model bymodel-identifier (mid)
        
        Args:
            mid (int): the model-identifer for the wanted model
        
        Returns:
            int: ec. 0 or error-code?
        """
    def store_model(self, m: GoalFunctionModel, mi: shyft.time_series.ModelInfo) -> int:
        """
        Store the model to backend, if m.id==0 then a new unique model-info is created and used
        
        Args:
            m (Model): The model to store
        
            mi (ModelInfo): The model-info to store for the model
        
        Returns:
            int: mid. model-identifier for the stored model and model-info
        """
    def update_model_info(self, mid: int, mi: shyft.time_series.ModelInfo) -> bool:
        """
        Update the model-info for specified model-identifier(mid)
        
        Args:
            mid (int): model-identifer
        
            mi (ModelInfo): The new updated model-info
        
        Returns:
            bool: ok. true if success
        """
    @property
    def host_port(self) -> str:
        """
        str: Endpoint network address of the remote server.
        """
    @property
    def is_open(self) -> bool:
        """
        bool: If the connection to the remote server is (still) open.
        """
    @property
    def reconnect_count(self) -> int:
        """
        int: Number of reconnects to the remote server that have been performed.
        """
    @property
    def timeout_ms(self) -> int:
        """
        int: Timout for remote server operations, in number milliseconds.
        """
class GoalFunctionModel:
    """
    A model identifier and goal-funcions, aka `TargetSpecificationVector`  used in Shyft region model calibrations. This plays the role of the goal-function specification for shyft region-models, so that they
    can be calibrated, and measured against observed quantities.
    """
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, arg0: GoalFunctionModel) -> bool:
        ...
    def __init__(self) -> None:
        ...
    def __repr__(self) -> str:
        ...
    def __str__(self) -> str:
        ...
    @property
    def goal_functions(self) -> TargetSpecificationVector:
        """
        TargetSpecificationVector: The goal functions
        """
    @goal_functions.setter
    def goal_functions(self, arg0: TargetSpecificationVector) -> None:
        ...
    @property
    def id(self) -> int:
        """
        int: unique model id
        """
    @id.setter
    def id(self, arg0: int) -> None:
        ...
class GoalFunctionModelVector:
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __bool__(self) -> bool:
        """
        Check whether the list is nonempty
        """
    @typing.overload
    def __delitem__(self, arg0: int) -> None:
        """
        Delete the list elements at index ``i``
        """
    @typing.overload
    def __delitem__(self, arg0: slice) -> None:
        """
        Delete list elements using a slice object
        """
    def __eq__(self, arg0: GoalFunctionModelVector) -> bool:
        ...
    @typing.overload
    def __getitem__(self, s: slice) -> GoalFunctionModelVector:
        """
        Retrieve list elements using a slice object
        """
    @typing.overload
    def __getitem__(self, arg0: int) -> GoalFunctionModel:
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, arg0: GoalFunctionModelVector) -> None:
        """
        Copy constructor
        """
    @typing.overload
    def __init__(self, arg0: typing.Iterable) -> None:
        ...
    def __iter__(self) -> typing.Iterator[GoalFunctionModel]:
        ...
    def __len__(self) -> int:
        ...
    def __ne__(self, arg0: GoalFunctionModelVector) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    @typing.overload
    def __setitem__(self, arg0: int, arg1: GoalFunctionModel) -> None:
        ...
    @typing.overload
    def __setitem__(self, arg0: slice, arg1: GoalFunctionModelVector) -> None:
        """
        Assign list elements using a slice object
        """
    def __str__(self) -> str:
        ...
    def append(self, x: GoalFunctionModel) -> None:
        """
        Add an item to the end of the list
        """
    def clear(self) -> None:
        """
        Clear the contents
        """
    @typing.overload
    def extend(self, L: GoalFunctionModelVector) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    @typing.overload
    def extend(self, L: typing.Iterable) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    def insert(self, i: int, x: GoalFunctionModel) -> None:
        """
        Insert an item at a given position.
        """
    @typing.overload
    def pop(self) -> GoalFunctionModel:
        """
        Remove and return the last item
        """
    @typing.overload
    def pop(self, i: int) -> GoalFunctionModel:
        """
        Remove and return the item at index ``i``
        """
    def size(self) -> int:
        ...
class GoalFunctionServer:
    """
    The server-side component for the goal-function model repository.
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, root_dir: str, config: shyft.time_series.ServerConfig = ...) -> None:
        """
        Creates a server object that serves models from root_dir.
        The root_dir will be create if it does not exists.
        
        Args:
            root_dir (str): Path to the root-directory that keeps/will keep the model-files
        
            config (ServerConfig): Configuration of the server
        """
    def get_listening_port(self) -> int:
        """
        returns the port number it's listening at for serving incoming request
        """
    def get_max_connections(self) -> int:
        """
        returns the maximum number of connections to be served concurrently
        """
    def is_running(self) -> bool:
        """
        true if server is listening and running
        
        See also:
            start_server()
        """
    def set_listening_ip(self, ip: str) -> None:
        """
        set the listening port for the service
        
        Args:
            ip (str): ip or host-name to start listening on
        
        Returns:
            None: nothing. 
        """
    def set_listening_port(self, port_no: int) -> None:
        """
        set the listening port for the service
        
        Args:
            port_no (int): a valid and available tcp-ip port number to listen on.
            typically it could be 20000 (avoid using official reserved numbers)
        
        Returns:
            None: nothing. 
        """
    def set_max_connections(self, max_connect: int) -> None:
        """
        limits simultaneous connections to the server (it's multithreaded, and uses on thread pr. connect)
        
        Args:
            max_connect (int): maximum number of connections before denying more connections
        
        See also:
            get_max_connections()
        """
    def start_server(self) -> int:
        """
        start server listening in background, and processing messages
        
        See also:
            set_listening_port(port_no),is_running
        
        Returns:
            in: port_no. the port used for listening operations, either the value as by set_listening_port, or if it was unspecified, a new available port
        """
    def stop_server(self, timeout: int = 1000) -> None:
        """
        stop serving connections, gracefully.
        
        See also:
            start_server()
        """
    @property
    def stale_connection_close_count(self) -> int:
        """
        int: returns count of connection closed due to stale/no communication activity
        """
class HbvActualEvapotranspirationParameter:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, lp: float = 150.0) -> None:
        ...
    @property
    def lp(self) -> float:
        """
        float: typical value 150
        """
    @lp.setter
    def lp(self, arg0: float) -> None:
        ...
class HbvActualEvapotranspirationResponse:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self) -> None:
        ...
    @property
    def ae(self) -> float:
        """
        float: ae value
        """
    @ae.setter
    def ae(self, arg0: float) -> None:
        ...
class HbvPhysicalSnowCalculator:
    """
    Generalized quantile based HBV Physical Snow model method
    
    This algorithm uses arbitrary quartiles to model snow. No checks are performed to assert valid input.
    The starting points of the quantiles have to partition the unity, 
    include the end points 0 and 1 and must be given in ascending order.
    
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, parameter: HbvPhysicalSnowParameter) -> None:
        """
        creates a calculator with given parameter and initial state, notice that state is updated in this call(hmm)
        """
    def step(self, state: HbvPhysicalSnowState, response: HbvPhysicalSnowResponse, t: shyft.time_series.time, dt: shyft.time_series.time, temperature: float, rad: float, prec_mm_h: float, wind_speed: float, rel_hum: float) -> None:
        """
        steps the model forward from t to t+dt, updating state and response
        """
class HbvPhysicalSnowParameter:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self, tx: float = 0.0, lw: float = 0.1, cfr: float = 0.5, wind_scale: float = 2.0, wind_const: float = 1.0, surface_magnitude: float = 30.0, max_albedo: float = 0.9, min_albedo: float = 0.6, fast_albedo_decay_rate: float = 5.0, slow_albedo_decay_rate: float = 5.0, snowfall_reset_depth: float = 5.0, calculate_iso_pot_energy: bool = False) -> None:
        ...
    @typing.overload
    def __init__(self, snow_redist_factors: list[float], quantiles: list[float], tx: float = 0.0, lw: float = 0.1, cfr: float = 0.5, wind_scale: float = 2.0, wind_const: float = 1.0, surface_magnitude: float = 30.0, max_albedo: float = 0.9, min_albedo: float = 0.6, fast_albedo_decay_rate: float = 5.0, slow_albedo_decay_rate: float = 5.0, snowfall_reset_depth: float = 5.0, calculate_iso_pot_energy: bool = False) -> None:
        """
        create a parameter with snow re-distribution factors, quartiles and optionally the other parameters
        """
    def set_snow_quantiles(self, quantiles: list[float]) -> None:
        ...
    def set_snow_redistribution_factors(self, snow_redist_factors: list[float]) -> None:
        ...
    @property
    def calculate_iso_pot_energy(self) -> bool:
        """
        float: whether or not to calculate the potential energy flux
        """
    @calculate_iso_pot_energy.setter
    def calculate_iso_pot_energy(self, arg0: bool) -> None:
        ...
    @property
    def cfr(self) -> float:
        """
        float: cfr
        """
    @cfr.setter
    def cfr(self, arg0: float) -> None:
        ...
    @property
    def fast_albedo_decay_rate(self) -> float:
        """
        float: albedo decay rate during melt [days]
        """
    @fast_albedo_decay_rate.setter
    def fast_albedo_decay_rate(self, arg0: float) -> None:
        ...
    @property
    def intervals(self) -> list[float]:
        """
        DoubleVector: snow quantiles list default 0, 0.25 0.5 1.0
        """
    @intervals.setter
    def intervals(self, arg0: list[float]) -> None:
        ...
    @property
    def lw(self) -> float:
        """
        float: max liquid water content of the snow
        """
    @lw.setter
    def lw(self, arg0: float) -> None:
        ...
    @property
    def max_albedo(self) -> float:
        """
        float: maximum albedo value
        """
    @max_albedo.setter
    def max_albedo(self, arg0: float) -> None:
        ...
    @property
    def min_albedo(self) -> float:
        """
        float: minimum albedo value
        """
    @min_albedo.setter
    def min_albedo(self, arg0: float) -> None:
        ...
    @property
    def s(self) -> list[float]:
        """
        DoubleVector: snow redistribution factors,default =1.0..
        """
    @s.setter
    def s(self, arg0: list[float]) -> None:
        ...
    @property
    def slow_albedo_decay_rate(self) -> float:
        """
        float: albedo decay rate in cold conditions [days]
        """
    @slow_albedo_decay_rate.setter
    def slow_albedo_decay_rate(self, arg0: float) -> None:
        ...
    @property
    def snowfall_reset_depth(self) -> float:
        """
        float: snowfall required to reset albedo [mm]
        """
    @snowfall_reset_depth.setter
    def snowfall_reset_depth(self, arg0: float) -> None:
        ...
    @property
    def surface_magnitude(self) -> float:
        """
        float: surface layer magnitude
        """
    @surface_magnitude.setter
    def surface_magnitude(self, arg0: float) -> None:
        ...
    @property
    def tx(self) -> float:
        """
        float: threshold temperature determining if precipitation is rain or snow
        """
    @tx.setter
    def tx(self, arg0: float) -> None:
        ...
    @property
    def wind_const(self) -> float:
        """
        float: intercept in turbulent wind function
        """
    @wind_const.setter
    def wind_const(self, arg0: float) -> None:
        ...
    @property
    def wind_scale(self) -> float:
        """
        float: slope in turbulent wind function [m/s]
        """
    @wind_scale.setter
    def wind_scale(self, arg0: float) -> None:
        ...
class HbvPhysicalSnowResponse:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self) -> None:
        ...
    @property
    def hps_state(self) -> HbvPhysicalSnowState:
        """
        HbvPhysicalSnowState: current state instance
        """
    @hps_state.setter
    def hps_state(self, arg0: HbvPhysicalSnowState) -> None:
        ...
    @property
    def outflow(self) -> float:
        """
        float: from snow-routine in [mm]
        """
    @outflow.setter
    def outflow(self, arg0: float) -> None:
        ...
    @property
    def sca(self) -> float:
        """
        float: snow-covered area
        """
    @sca.setter
    def sca(self, arg0: float) -> None:
        ...
    @property
    def storage(self) -> float:
        """
        float: snow storage [mm]
        """
    @storage.setter
    def storage(self, arg0: float) -> None:
        ...
class HbvPhysicalSnowState:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, albedo: list[float], iso_pot_energy: list[float], surface_heat: float = 30000.0, swe: float = 0.0, sca: float = 0.0) -> None:
        ...
    def distribute(self, p: HbvPhysicalSnowParameter, force: bool = True) -> None:
        """
        Distribute state according to parameter settings.
        
        Args:
            p (HbvPhysicalSnowParameter): descr
        
            force (bool): default true, if false then only distribute if state vectors are of different size than parameters passed
        
        Returns:
            None: . 
        """
    @property
    def albedo(self) -> list[float]:
        """
        float: albedo (Broadband snow reflectivity fraction)
        """
    @albedo.setter
    def albedo(self, arg0: list[float]) -> None:
        ...
    @property
    def iso_pot_energy(self) -> list[float]:
        """
        float: iso_pot_energy (Accumulated energy assuming isothermal snow surface) [J/m2]
        """
    @iso_pot_energy.setter
    def iso_pot_energy(self, arg0: list[float]) -> None:
        ...
    @property
    def sca(self) -> float:
        """
        float: snow covered area [0..1]
        """
    @sca.setter
    def sca(self, arg0: float) -> None:
        ...
    @property
    def sp(self) -> list[float]:
        """
        DoubleVector: snow dry[mm]
        """
    @sp.setter
    def sp(self, arg0: list[float]) -> None:
        ...
    @property
    def surface_heat(self) -> float:
        """
        float: surface_heat (Snow surface cold content) [J/m2]
        """
    @surface_heat.setter
    def surface_heat(self, arg0: float) -> None:
        ...
    @property
    def sw(self) -> list[float]:
        """
        DoubleVector: snow water[mm]
        """
    @sw.setter
    def sw(self, arg0: list[float]) -> None:
        ...
    @property
    def swe(self) -> float:
        """
        float: snow water equivalent[mm]
        """
    @swe.setter
    def swe(self, arg0: float) -> None:
        ...
class HbvSnowCalculator:
    """
    Generalized quantile based HBV Snow model method
    
    This algorithm uses arbitrary quartiles to model snow. No checks are performed to assert valid input.
    The starting points of the quantiles have to partition the unity, 
    include the end points 0 and 1 and must be given in ascending order.
    
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, parameter: HbvSnowParameter) -> None:
        ...
    def step(self, state: HbvSnowState, response: HbvSnowResponse, t0: shyft.time_series.time, t1: shyft.time_series.time, precipitation: float, temperature: float) -> None:
        """
        steps the model forward from t0 to t1, updating state and response
        """
class HbvSnowParameter:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self, tx: float = 0.0, cx: float = 1.0, ts: float = 0.0, lw: float = 0.1, cfr: float = 0.5) -> None:
        ...
    @typing.overload
    def __init__(self, snow_redist_factors: list[float], quantiles: list[float], tx: float = 0.0, cx: float = 1.0, ts: float = 0.0, lw: float = 0.1, cfr: float = 0.5) -> None:
        """
        create a parameter with snow re-distribution factors, quartiles and optionally the other parameters
        """
    def set_snow_quantiles(self, quantiles: list[float]) -> None:
        ...
    def set_snow_redistribution_factors(self, snow_redist_factors: list[float]) -> None:
        ...
    @property
    def cfr(self) -> float:
        """
        float: cfr
        """
    @cfr.setter
    def cfr(self, arg0: float) -> None:
        ...
    @property
    def cx(self) -> float:
        """
        float: temperature index, i.e., melt = cx(t - ts) in mm per degree C
        """
    @cx.setter
    def cx(self, arg0: float) -> None:
        ...
    @property
    def intervals(self) -> list[float]:
        """
        DoubleVector: snow quantiles list default 0, 0.25 0.5 1.0
        """
    @intervals.setter
    def intervals(self, arg0: list[float]) -> None:
        ...
    @property
    def lw(self) -> float:
        """
        float: max liquid water content of the snow
        """
    @lw.setter
    def lw(self, arg0: float) -> None:
        ...
    @property
    def s(self) -> list[float]:
        """
        DoubleVector: snow redistribution factors,default =1.0..
        """
    @s.setter
    def s(self, arg0: list[float]) -> None:
        ...
    @property
    def ts(self) -> float:
        """
        float: threshold temperature for melt onset
        """
    @ts.setter
    def ts(self, arg0: float) -> None:
        ...
    @property
    def tx(self) -> float:
        """
        float: threshold temperature determining if precipitation is rain or snow
        """
    @tx.setter
    def tx(self, arg0: float) -> None:
        ...
class HbvSnowResponse:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self) -> None:
        ...
    @property
    def outflow(self) -> float:
        """
        float: from snow-routine in [mm]
        """
    @outflow.setter
    def outflow(self, arg0: float) -> None:
        ...
    @property
    def snow_state(self) -> HbvSnowState:
        """
        HbvSnowState: swe and snow covered area
        """
    @snow_state.setter
    def snow_state(self, arg0: HbvSnowState) -> None:
        ...
class HbvSnowState:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, swe: float = 0.0, sca: float = 0.0) -> None:
        ...
    def distribute(self, p: HbvSnowParameter, force: bool = True) -> None:
        """
        Distribute state according to parameter settings.
        
        Args:
            p (HbvSnowParameter): descr
        
            force (bool): default true, if false then only distribute if state vectors are of different size than parameters passed
        
        Returns:
            None: . 
        """
    @property
    def sca(self) -> float:
        """
        float: snow covered area [0..1]
        """
    @sca.setter
    def sca(self, arg0: float) -> None:
        ...
    @property
    def sp(self) -> list[float]:
        """
        DoubleVector: snow dry[mm]
        """
    @sp.setter
    def sp(self, arg0: list[float]) -> None:
        ...
    @property
    def sw(self) -> list[float]:
        """
        DoubleVector: snow water[mm]
        """
    @sw.setter
    def sw(self, arg0: list[float]) -> None:
        ...
    @property
    def swe(self) -> float:
        """
        float: snow water equivalent[mm]
        """
    @swe.setter
    def swe(self, arg0: float) -> None:
        ...
class HbvSoilCalculator:
    """
    Computing water through the soil moisture zone of the HBV model.
    
    
    Reference:
    
     * Nils Roar Sæhlthun: The Nordic HBV model 1996 https://publikasjoner.nve.no/publication/1996/publication1996_07.pdf
    
    
    
    
    Notes:
        Lake and glacier are treated in the stack
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, parameter: HbvSoilParameter, land_fraction: float) -> None:
        ...
    def step(self, state: HbvSoilState, response: HbvSoilResponse, insoil: float, pe: float, sca: float) -> None:
        """
        One step of the model, given state, parameters and input.
        
        Updates the state and response.
        
        
        Args:
            state (HbvSoilState):  param of type S, in/out, ref template parameters
        
            response (HbvSoilResponse):  param of type R, in/out, ref template parameters
        
            insoil (float): inflow to soil
        
            pe (float): potential evaporation
        
            sca (float): the response , out parameter.
        
            land_fraction (float): Land-fraction of the cell. Not directly used/desribed in ref. 'Nils Roar', but introduced here to adopt to the stack.
        """
class HbvSoilParameter:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, fc: float = 32.2755689, beta: float = 1.50738409) -> None:
        ...
    @property
    def beta(self) -> float:
        """
        float: (unitless) exponent in the nonlinear relationship between soil moisture and field capacity. default=1.50738409
        """
    @beta.setter
    def beta(self, arg0: float) -> None:
        ...
    @property
    def fc(self) -> float:
        """
        float: (mm) maximum water content of the soil (Field Capacity), default=32.2755689
        """
    @fc.setter
    def fc(self, arg0: float) -> None:
        ...
    @property
    def infmax(self) -> float:
        """
        float: (mm/h) maximum input rate to the soil moisture zone. Excess water will go directly to upper ground water zone. default=2.0
        """
    @infmax.setter
    def infmax(self, arg0: float) -> None:
        ...
    @property
    def lpdel(self) -> float:
        """
        float: (unitless) dimensionless parameter (<1), from which level evapotranspiration is potential. Some publications use fcdel instead. default= 0.848557794
        """
    @lpdel.setter
    def lpdel(self, arg0: float) -> None:
        ...
class HbvSoilResponse:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, inuz: float = 0.0, ae: float = 0.0) -> None:
        ...
    @property
    def ae(self) -> float:
        """
        float: (mm/h) Actual evapotranspiration. default=0
        """
    @ae.setter
    def ae(self, arg0: float) -> None:
        ...
    @property
    def inuz(self) -> float:
        """
        float: (mm/h) Perculation to upper ground water zone. Called `cuz` by Nils Roar. default=0
        """
    @inuz.setter
    def inuz(self, arg0: float) -> None:
        ...
class HbvSoilState:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, sm: float = 10.0) -> None:
        ...
    @property
    def sm(self) -> float:
        """
        float: Soil  moisture [mm]. default=10
        """
    @sm.setter
    def sm(self, arg0: float) -> None:
        ...
class HbvTankCalculator:
    """
    Computing water through the HBV ground water zone of the HBV model.
    
    
    Reference:
    
     * Nils Roar Sæhlthun: The Nordic HBV model 1996 https://publikasjoner.nve.no/publication/1996/publication1996_07.pdf
    
    
    
    
    Notes:
        Lake and glacier are treated in the stack
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, parameter: HbvTankParameter, lake_fraction: float) -> None:
        """
        creates a calculator with given parameters
        """
    def step(self, state: HbvTankState, response: HbvTankResponse, from_soil: float, precip: float, t2m: float) -> None:
        """
        One step of the model, given state, parameters and input.
        
        Updates the state and response.
        
        
        Args:
            state (HbvSoilState):  param of type S, in/out, ref template parameters
        
            response (HbvSoilResponse):  param of type R, in/out, ref template parameters
        
            from_soil (float): [mm/step] Inflow to upper zone (from soil moisture)
        
            precip (float): [mm/step] Routed precipitation. Assumed that it is corrected for lake fraction already.
        
            t2m (float): [deg C] Air-temperature at 2m (proxy for water temperature)
        
            lake_fraction (float): [unitless] Lake-fraction of the cell. Not directly used/desribed in ref.'Nils Roar', but introduced here to adopt to the stack.
        """
class HbvTankParameter:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, uz1: float = 10.0, uz2: float = 50.0, kuz0: float = 0.05, kuz1: float = 0.1, kuz2: float = 0.0, perc: float = 0.6, klz: float = 0.05, ce: float = 0.007083333333333334, cevpl: float = 1.1) -> None:
        ...
    @property
    def ce(self) -> float:
        """
        float: [ mm/deg/day ] Evapotranspiration constant (degree-day-factor). default=0.17/24
        """
    @ce.setter
    def ce(self, arg0: float) -> None:
        ...
    @property
    def cevpl(self) -> float:
        """
        float: [ unitless ], adjustment factor for potential evapotranspiration on lakes. default=1.1
        """
    @cevpl.setter
    def cevpl(self, arg0: float) -> None:
        ...
    @property
    def klz(self) -> float:
        """
        float: [ 1/step ] Slow response coefficient lower ground water zone. default=0.05
        """
    @klz.setter
    def klz(self, arg0: float) -> None:
        ...
    @property
    def kuz0(self) -> float:
        """
        float: [ 1/step ] Slow response coefficient upper ground water zone. default=0.05
        """
    @kuz0.setter
    def kuz0(self, arg0: float) -> None:
        ...
    @property
    def kuz1(self) -> float:
        """
        float: [ 1/step ] Mid response coefficient upper ground water zone. default=0.1
        """
    @kuz1.setter
    def kuz1(self, arg0: float) -> None:
        ...
    @property
    def kuz2(self) -> float:
        """
        float: [ 1/step ] Fast response coefficient upper ground water zone. default=0 (not active)
        """
    @kuz2.setter
    def kuz2(self, arg0: float) -> None:
        ...
    @property
    def perc(self) -> float:
        """
        float: [ mm/step ] Perculation to lower ground water zone. default=0.6
        """
    @perc.setter
    def perc(self, arg0: float) -> None:
        ...
    @property
    def uz1(self) -> float:
        """
        float: [ mm ] Mid-threshold for upper ground water zone. default=10
        """
    @uz1.setter
    def uz1(self, arg0: float) -> None:
        ...
    @property
    def uz2(self) -> float:
        """
        float: [ mm ] High-threshold for upper ground water zone. default=50
        """
    @uz2.setter
    def uz2(self, arg0: float) -> None:
        ...
class HbvTankResponse:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, quz0: float = 0.0, quz1: float = 0.0, quz2: float = 0.0, qlz: float = 0.0, elake: float = 0.0, perculation: float = 0.0) -> None:
        ...
    @property
    def elake(self) -> float:
        """
        float: [ mm/step ] Evaporation from lake
        """
    @elake.setter
    def elake(self, arg0: float) -> None:
        ...
    @property
    def perculation(self) -> float:
        """
        float: [ mm/step ] Effective perculation from upper to lower
        """
    @perculation.setter
    def perculation(self, arg0: float) -> None:
        ...
    @property
    def q(self) -> float:
        """
        float: [mm/step] Total discharge from tank
        """
    @property
    def qlz(self) -> float:
        """
        float: [mm/step] Total for lower zone
        """
    @qlz.setter
    def qlz(self, arg0: float) -> None:
        ...
    @property
    def quz(self) -> float:
        """
        float: [mm/step] Total for upper zone
        """
    @property
    def quz0(self) -> float:
        """
        float: [mm/step] Upper zone slow response
        """
    @quz0.setter
    def quz0(self, arg0: float) -> None:
        ...
    @property
    def quz1(self) -> float:
        """
        float: [mm/step] Upper zone mid response
        """
    @quz1.setter
    def quz1(self, arg0: float) -> None:
        ...
    @property
    def quz2(self) -> float:
        """
        float: [mm/step] Upper zone fast response
        """
    @quz2.setter
    def quz2(self, arg0: float) -> None:
        ...
class HbvTankState:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, uz: float = 10, lz: float = 10) -> None:
        """
        create a state with specified values
        """
    @property
    def lz(self) -> float:
        """
        float: Lower ground water zone content [mm]
        """
    @lz.setter
    def lz(self, arg0: float) -> None:
        ...
    @property
    def uz(self) -> float:
        """
        float: Upper ground water zone content [mm]
        """
    @uz.setter
    def uz(self, arg0: float) -> None:
        ...
class IDWParameter:
    """
    IDWParameter is a simple place-holder for IDW parameters used during interpolation process.
    The two most common is max_distance and max_members.
    Additionally it keep distance measure-factor,
    so that the IDW distance is computed as 1 over pow(euclid distance,distance_measure_factor).
    zscale is used to discriminate neighbors that are at different elevation than target point.
    """
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, arg0: IDWParameter) -> bool:
        ...
    def __init__(self, max_members: int = 10, max_distance: float = 200000.0, distance_measure_factor: float = 2.0) -> None:
        ...
    def __ne__(self, arg0: IDWParameter) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __str__(self) -> str:
        ...
    @property
    def distance_measure_factor(self) -> float:
        """
        float: IDW distance is computed as 1 over pow(euclid distance,distance_measure_factor), default=2.0
        """
    @distance_measure_factor.setter
    def distance_measure_factor(self, arg0: float) -> None:
        ...
    @property
    def max_distance(self) -> float:
        """
        float: [meter] only neighbours within max distance is used for each destination-cell,default= 200000.0
        """
    @max_distance.setter
    def max_distance(self, arg0: float) -> None:
        ...
    @property
    def max_members(self) -> int:
        """
        int: maximum members,neighbors used to interpolate into a point,default=10
        """
    @max_members.setter
    def max_members(self, arg0: int) -> None:
        ...
    @property
    def zscale(self) -> float:
        """
        float: Use to weight neighbors having same elevation, default=1.0
        """
    @zscale.setter
    def zscale(self, arg0: float) -> None:
        ...
class IDWPrecipitationParameter(IDWParameter):
    """
    For precipitation,the scaling model needs the scale_factor.
    adjusted_precipitation = precipitation* (scale_factor)^(z-distance-in-meters/100.0)
    Ref to IDWParameter for the other parameters
    
    """
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, arg0: IDWPrecipitationParameter) -> bool:
        ...
    def __init__(self, scale_factor: float = 1.02, max_members: int = 20, max_distance: float = 200000.0) -> None:
        ...
    def __ne__(self, arg0: IDWPrecipitationParameter) -> bool:
        ...
    @property
    def scale_factor(self) -> float:
        """
        float: ref. formula for adjusted_precipitation,  default=1.02
        """
    @scale_factor.setter
    def scale_factor(self, arg0: float) -> None:
        ...
class IDWTemperatureParameter(IDWParameter):
    """
    For temperature inverse distance, also provide default temperature gradient to be used
    when the gradient can not be computed, or is outside specified limits.
    if gradient_by_equation is set true, and number of points >3, the temperature gradient computer
    will try to use the 4 closest points and determine the 3d gradient including the vertical gradient.
    In scenarios with constant gradients(vertical/horizontal), this is accurate) 
    If the number of valide nearby points is to few to do the full gradient equation, or the gradient_by_equation is set to False,
    then the two points with maximum vertical distance (required to be more than the minimum_z_distance) is used to compute the gradient.
    If all of the above fails, the default-gradient is used.
    Please note:
    Carefully selecting the maximum horizontal distances, and number of neighbors is required to get
    optimal results.
    In case of non-grid, or sparse-grid scenarios, you might consider setting the parameters to enforce default-gradient
    or you can also consider using the bayesian temperature krieging algorithm if there are few temperature sources
    """
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, arg0: IDWTemperatureParameter) -> bool:
        ...
    def __init__(self, default_gradient: float = -0.006, max_members: int = 20, max_distance: float = 200000.0, gradient_by_equation: bool = False, minimum_z_distance: float = 50, gradient_min: float = -0.01, gradient_max: float = 0.01) -> None:
        """
        construct IDW for temperature as specified by arguments
        """
    def __ne__(self, arg0: IDWTemperatureParameter) -> bool:
        ...
    @property
    def default_temp_gradient(self) -> float:
        """
        float: [degC/m], default=-0.006
        """
    @default_temp_gradient.setter
    def default_temp_gradient(self, arg0: float) -> None:
        ...
    @property
    def gradient_by_equation(self) -> bool:
        """
        bool: if true, gradient is computed using 4 closest neighbors, solving equations to find 3D temperature gradients.
        """
    @gradient_by_equation.setter
    def gradient_by_equation(self, arg0: bool) -> None:
        ...
    @property
    def gradient_max(self) -> float:
        """
        float: [degC/m] default +0.01, if computed gradient larger than this, the default-gradient is used instead
        """
    @gradient_max.setter
    def gradient_max(self, arg0: float) -> None:
        ...
    @property
    def gradient_min(self) -> float:
        """
        float: [degC/m] default -0.01, if computed gradient less than this, the default-gradient is used instead
        """
    @gradient_min.setter
    def gradient_min(self, arg0: float) -> None:
        ...
    @property
    def minimum_z_distance(self) -> float:
        """
        float: [m] default 50m, the minimum vertical distance requirement for two points to be used for gradient computation
        """
    @minimum_z_distance.setter
    def minimum_z_distance(self, arg0: float) -> None:
        ...
class InterpolationParameter:
    """
    The InterpolationParameter keep parameters needed to perform the
    interpolation steps, IDW,BTK etc
    It is used as parameter  in the model.run_interpolation() method
    """
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, arg0: InterpolationParameter) -> bool:
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, temperature: BTKParameter, precipitation: IDWPrecipitationParameter, wind_speed: IDWParameter, radiation: IDWParameter, rel_hum: IDWParameter) -> None:
        ...
    @typing.overload
    def __init__(self, temperature: IDWTemperatureParameter, precipitation: IDWPrecipitationParameter, wind_speed: IDWParameter, radiation: IDWParameter, rel_hum: IDWParameter) -> None:
        ...
    def __ne__(self, arg0: InterpolationParameter) -> bool:
        ...
    @property
    def precipitation(self) -> IDWPrecipitationParameter:
        """
        IDWPrecipitationParameter: IDW parameters for precipitation
        """
    @precipitation.setter
    def precipitation(self, arg0: IDWPrecipitationParameter) -> None:
        ...
    @property
    def radiation(self) -> IDWParameter:
        """
        IDWParameter: IDW parameters for radiation
        """
    @radiation.setter
    def radiation(self, arg0: IDWParameter) -> None:
        ...
    @property
    def rel_hum(self) -> IDWParameter:
        """
        IDWParameter: IDW parameters for relative humidity
        """
    @rel_hum.setter
    def rel_hum(self, arg0: IDWParameter) -> None:
        ...
    @property
    def temperature(self) -> BTKParameter:
        """
        BTKParameter: BTK for temperature (in case .use_idw_for_temperature is false)
        """
    @temperature.setter
    def temperature(self, arg0: BTKParameter) -> None:
        ...
    @property
    def temperature_idw(self) -> IDWTemperatureParameter:
        """
        IDWTemperatureParameter: IDW for temperature(in case .use_idw_for_temperature is true)
        """
    @temperature_idw.setter
    def temperature_idw(self, arg0: IDWTemperatureParameter) -> None:
        ...
    @property
    def use_idw_for_temperature(self) -> bool:
        """
        bool: if true, the IDW temperature is used instead of BTK, useful for grid-input scenarios
        """
    @use_idw_for_temperature.setter
    def use_idw_for_temperature(self, arg0: bool) -> None:
        ...
    @property
    def wind_speed(self) -> IDWParameter:
        """
        IDWParameter: IDW parameters for wind_speed
        """
    @wind_speed.setter
    def wind_speed(self, arg0: IDWParameter) -> None:
        ...
class KalmanBiasPredictor:
    """
    A bias predictor using a daily pattern KalmanFilter for temperature (etc.)
    (tbd)
    """
    state: KalmanState
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @staticmethod
    def compute_running_bias_ts(bias_predictor: KalmanBiasPredictor, forecast_ts: shyft.time_series.TimeSeries, observation_ts: shyft.time_series.TimeSeries, time_axis: shyft.time_series.TimeAxis) -> shyft.time_series.TimeSeries:
        """
        compute the running bias timeseries,
        using one 'merged' - forecasts and one observation time - series.
        
        Before each day - period, the bias - values are copied out to form
        a continuous bias prediction time-series.
        
        
        Args:
            bias_predictor (KalmanBiasPredictor): The bias predictor object it self
        
            forecast_ts (TimeSeries): a merged forecast ts, with period covering the observation_ts and time_axis supplied
        
            observation_ts (TimeSeries): the observation time-series
        
            time_axis (TimeAxis): covering the period/timesteps to be updated, e.g. yesterday, 3h resolution steps, according to the points in the filter
        
        Returns:
            TimeSeries: bias_ts. With ts( time_axis,bias_vector,POINT_AVERAGE),  computed running bias-ts
        """
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, filter: KalmanFilter) -> None:
        ...
    @typing.overload
    def __init__(self, filter: KalmanFilter, state: KalmanState) -> None:
        ...
    def compute_running_bias(self, arg0: shyft.time_series.TimeSeries, arg1: shyft.time_series.TimeSeries, arg2: shyft.time_series.TimeAxis) -> shyft.time_series.TimeSeries:
        ...
    @typing.overload
    def update_with_forecast(self, arg0: TemperatureSourceVector, arg1: shyft.time_series.TimeSeries, arg2: shyft.time_series.TimeAxis) -> None:
        ...
    @typing.overload
    def update_with_forecast(self, arg0: list[shyft.time_series.TimeSeries], arg1: shyft.time_series.TimeSeries, arg2: shyft.time_series.TimeAxis) -> None:
        ...
    def update_with_forecast_vector(self, temperature_sources: list[shyft.time_series.TimeSeries], observation_ts: shyft.time_series.TimeSeries, time_axis: shyft.time_series.TimeAxis) -> None:
        """
        update the bias-predictor with forecasts and observation
        After the update, the state is updated with new kalman estimates for the bias, .state.x
        
        
        Args:
            temperature_sources (TsVector): a set of forecasts, in the order oldest to the newest.With the periods covering parts of the observation_ts and time_axis supplied
        
            observation_ts (TimeSeries): the observation time-series
        
            time_axis (TimeAxis): covering the period/timesteps to be updated, e.g. yesterday, 3h resolution steps, according to the points in the filter
        """
    def update_with_geo_forecast(self, temperature_sources: TemperatureSourceVector, observation_ts: shyft.time_series.TimeSeries, time_axis: shyft.time_series.TimeAxis) -> None:
        """
        update the bias-predictor with forecasts and observation
        After the update, the state is updated with new kalman estimates for the bias, .state.x
        
        
        Args:
            temperature_sources (TemperatureSourceVector): ta set of forecasts, in the order oldest to the newest.Note that the geo part of source is not used in this context, only the tswith periods covering parts of the observation_ts and time_axis supplied
        
            observation_ts (Timeseries): the observation time-series
        
            time_axis (TimeAxis): covering the period/timesteps to be updated,e.g. yesterday, 3h resolution steps, according to the points in the filter
        """
    @property
    def filter(self) -> KalmanFilter:
        ...
class KalmanFilter:
    """
    Specialized kalman filter for temperature (e.g.:solar-driven bias patterns)
    The observation point(t, v) is folded on to corresponding period of the
    day(number of periods in a day is parameterized, typically 8).
    A simplified kalman filter algorithm using the forecast bias as
    the state - variable.
    Observed bias(fc - obs) is feed into the filter and establishes the
    kalman best predicted estimates(x) for the bias.
    This bias can then be used as a correction to forecast in the future
    to compensate for systematic forecast errors.
    Credits: Thanks to met.no for providing the original source for this algorithm.
    
    
    See also:
        https://en.wikipedia.org/wiki/Kalman_filter
    """
    parameter: KalmanParameter
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, p: KalmanParameter = ...) -> None:
        ...
    def create_initial_state(self) -> KalmanState:
        ...
    def update(self, observed_bias: float, t: shyft.time_series.time, state: KalmanState) -> None:
        """
        update the with the observed_bias for a specific period starting at utctime t
        
        Args:
            observed_bias (float): nan if no observation is available otherwise obs - fc
        
            t (time): utctime of the start of the observation period, this filter utilizes daily solar patterns, so time in day - cycle is the only important aspect.
        
            state (KalmanState): contains the kalman state,that is updated by the function upon return
        """
class KalmanParameter:
    """
    Defines the parameters that is used to tune the kalman-filter algorithm for temperature type of signals
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self, n_daily_observations: int = 8, hourly_correlation: float = 0.93, covariance_init: float = 0.5, std_error_bias_measurements: float = 2.0, ratio_std_w_over_v: float = 0.15) -> None:
        """
        Constructs KalmanParameter with default or supplied values
        """
    @typing.overload
    def __init__(self, const_ref: KalmanParameter) -> None:
        """
        clone the supplied KalmanParameter
        """
    def __repr__(self) -> str:
        ...
    def __str__(self) -> str:
        ...
    @property
    def covariance_init(self) -> float:
        """
        float:  default=0.5,  for the error covariance P matrix start-values
        """
    @covariance_init.setter
    def covariance_init(self, arg0: float) -> None:
        ...
    @property
    def hourly_correlation(self) -> float:
        """
        float: default=0.93, correlation from one-hour to the next
        """
    @hourly_correlation.setter
    def hourly_correlation(self, arg0: float) -> None:
        ...
    @property
    def n_daily_observations(self) -> int:
        """
        int:  default = 8 each 24hour, every 3 hour
        """
    @n_daily_observations.setter
    def n_daily_observations(self, arg0: int) -> None:
        ...
    @property
    def ratio_std_w_over_v(self) -> float:
        """
        float: default=0.06, st.dev W /st.dev V ratio
        """
    @ratio_std_w_over_v.setter
    def ratio_std_w_over_v(self, arg0: float) -> None:
        ...
    @property
    def std_error_bias_measurements(self) -> float:
        """
        float: default=2.0, st.dev for the bias measurements
        """
    @std_error_bias_measurements.setter
    def std_error_bias_measurements(self, arg0: float) -> None:
        ...
class KalmanState:
    """
    keeps the state of the specialized kalman-filter:
      * x : vector[n=n_daily_observations] best estimate
      * k : vector[n], gain factors
      * P : matrix[nxn], covariance
      * W : noise[nxn]
    
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @staticmethod
    def get_P(arg0: KalmanState) -> list[float]:
        ...
    @staticmethod
    def get_W(arg0: KalmanState) -> list[float]:
        ...
    @staticmethod
    def get_k(arg0: KalmanState) -> list[float]:
        ...
    @staticmethod
    def get_x(arg0: KalmanState) -> list[float]:
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, n_daily_observations: int, covariance_init: float, hourly_correlation: float, process_noise_init: float) -> None:
        ...
    @typing.overload
    def __init__(self, clone: KalmanState) -> None:
        """
        clone the supplied state
        """
    def size(self) -> int:
        """
        returns the size of the state, corresponding to n_daily_observations
        """
    @property
    def P(self) -> numpy.ndarray:
        ...
    @property
    def W(self) -> numpy.ndarray:
        ...
    @property
    def k(self) -> numpy.ndarray:
        ...
    @property
    def x(self) -> numpy.ndarray:
        ...
class KirchnerCalculator:
    """
    Kirchner model for computing the discharge based on precipitation and evapotranspiration data.
    
    This algorithm is based on the log transform of the ode formulation of the time change in discharge as a function
    of measured precipitation, evapo-transpiration and discharge, i.e. equation 19 in the publication
    Catchments as simple dynamic systems: Catchment characterization, rainfall-runoff modeling, and doing
    'hydrology backward' by James W. Kirchner, published in Water Resources Research, vol. 45, W02429,
    doi: 10.1029/2008WR006912, 2009.
    
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self, param: KirchnerParameter) -> None:
        ...
    @typing.overload
    def __init__(self, abs_err: float, rel_err: float, param: KirchnerParameter) -> None:
        ...
    def step(self, state: KirchnerState, response: KirchnerResponse, t0: shyft.time_series.time, t1: shyft.time_series.time, precipitation: float, evapotranspiration: float) -> None:
        """
        step Kirchner model forward from time t0 to time t1
        
        note: If the supplied q (state) is less than min_q(0.00001, it represents mm water..), it is forced to min_q to ensure numerical stability
        
        
        Args:
            state (KirchnerState):  current state, updated on return
        
            response (KirchnerResponse): the response , out parameter.
        
            t0 (time): start time
        
            t1 (time): end time
        
            precipitation (float): in mm/h
        
            evapotranspiration (float): in mm/h
        """
class KirchnerParameter:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, c1: float = 2.439, c2: float = 0.966, c3: float = -0.1) -> None:
        ...
    @property
    def c1(self) -> float:
        """
        float: default =2.439
        """
    @c1.setter
    def c1(self, arg0: float) -> None:
        ...
    @property
    def c2(self) -> float:
        """
        float: default= 0.966
        """
    @c2.setter
    def c2(self, arg0: float) -> None:
        ...
    @property
    def c3(self) -> float:
        """
        float: default = -0.10
        """
    @c3.setter
    def c3(self, arg0: float) -> None:
        ...
class KirchnerResponse:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self) -> None:
        ...
    @property
    def q_avg(self) -> float:
        """
        float: average discharge over time-step in [mm/h]
        """
    @q_avg.setter
    def q_avg(self, arg0: float) -> None:
        ...
class KirchnerState:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, q: float = 0.0001) -> None:
        ...
    @property
    def q(self) -> float:
        """
        float: state water 'content' in [mm/h], it defaults to 0.0001 mm, zero is not a reasonable valid value
        """
    @q.setter
    def q(self, arg0: float) -> None:
        ...
class LandTypeFractions:
    """
    LandTypeFractions are used to describe type of land,
    like glacier, lake, reservoir and forest.
    It is designed as a part of GeoCellData
    """
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, arg0: LandTypeFractions) -> bool:
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, glacier: float, lake: float, reservoir: float, forest: float, unspecified: float) -> None:
        """
        construct LandTypeFraction specifying the area of each type
        """
    def __ne__(self, arg0: LandTypeFractions) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __str__(self) -> str:
        ...
    def forest(self) -> float:
        """
        returns the forest part
        """
    def glacier(self) -> float:
        """
        returns the glacier part
        """
    def lake(self) -> float:
        """
        returns the lake part
        """
    def reservoir(self) -> float:
        """
        returns the reservoir part
        """
    def set_fractions(self, glacier: float, lake: float, reservoir: float, forest: float) -> None:
        """
        set the fractions explicit, each a value in range 0..1, sum should be 1.0
        """
    def snow_storage(self) -> float:
        """
        returns the area where snow can build up, 1.0-lake-reservoir
        """
    def unspecified(self) -> float:
        """
        returns the unspecified part
        """
class MethodStackParameter:
    """
    Contains the parameters for the method-stack,
    related to inter-method and routing behaviour
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, clone: MethodStackParameter) -> None:
        """
        make a clone
        """
    def __repr__(self) -> str:
        ...
    def __str__(self) -> str:
        ...
    @property
    def reservoir_direct_response_fraction(self) -> float:
        """
        float: range 0..1, default 1.0, e.g. all precipitation on a reservoir goes to direct response
         - set to 0.0, then all precipitation is routed as pr. standard for the stack
        """
    @reservoir_direct_response_fraction.setter
    def reservoir_direct_response_fraction(self, arg0: float) -> None:
        ...
class OKCovarianceType:
    """
    Members:
    
      GAUSSIAN
    
      EXPONENTIAL
    """
    EXPONENTIAL: typing.ClassVar[OKCovarianceType]  # value = <OKCovarianceType.EXPONENTIAL: 1>
    GAUSSIAN: typing.ClassVar[OKCovarianceType]  # value = <OKCovarianceType.GAUSSIAN: 0>
    __members__: typing.ClassVar[dict[str, OKCovarianceType]]  # value = {'GAUSSIAN': <OKCovarianceType.GAUSSIAN: 0>, 'EXPONENTIAL': <OKCovarianceType.EXPONENTIAL: 1>}
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
class OKParameter:
    """
    Ordinary Kriging Parameter, keeps parameters that controls the ordinary kriging calculation
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, c: float = 1.0, a: float = 10000.0, cov_type: OKCovarianceType = OKCovarianceType.EXPONENTIAL, z_scale: float = 1.0) -> None:
        ...
    @property
    def a(self) -> float:
        """
        float: the a-constant, range or distance, value in the covariance formula
        """
    @a.setter
    def a(self, arg0: float) -> None:
        ...
    @property
    def c(self) -> float:
        """
        float: the c-constant, sill value in the covariance formula
        """
    @c.setter
    def c(self, arg0: float) -> None:
        ...
    @property
    def cov_type(self) -> OKCovarianceType:
        """
        OKCovarianceType: covariance type EXPONENTIAL|GAUSSIAN to be used
        """
    @cov_type.setter
    def cov_type(self, arg0: OKCovarianceType) -> None:
        ...
    @property
    def z_scale(self) -> float:
        """
        float: z_scale to be used for range|distance calculations
        """
    @z_scale.setter
    def z_scale(self, arg0: float) -> None:
        ...
class OptimizerMethod:
    """
    Shyft calibration allow the user to select between one of these parameter optimizer methods.
    
    
    Members:
    
      BOBYQA
    
      GLOBAL
    
      DREAM
    
      SCEUA
    """
    BOBYQA: typing.ClassVar[OptimizerMethod]  # value = <OptimizerMethod.BOBYQA: 0>
    DREAM: typing.ClassVar[OptimizerMethod]  # value = <OptimizerMethod.DREAM: 2>
    GLOBAL: typing.ClassVar[OptimizerMethod]  # value = <OptimizerMethod.GLOBAL: 1>
    SCEUA: typing.ClassVar[OptimizerMethod]  # value = <OptimizerMethod.SCEUA: 3>
    __members__: typing.ClassVar[dict[str, OptimizerMethod]]  # value = {'BOBYQA': <OptimizerMethod.BOBYQA: 0>, 'GLOBAL': <OptimizerMethod.GLOBAL: 1>, 'DREAM': <OptimizerMethod.DREAM: 2>, 'SCEUA': <OptimizerMethod.SCEUA: 3>}
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
class ParameterClient:
    """
    The client api for the shyft.hydrology parameter model service.
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @staticmethod
    def get_model_infos(*args, **kwargs) -> list[shyft.time_series.ModelInfo]:
        """
        returns all or selected model-info objects based on model-identifiers(mids)
        
        Args:
            mids (IntVector): empty = all, or a list of known exisiting model-identifiers
        
            created_in (UtcPeriod): For which period you are interested in model-infos.
        
        Returns:
            ModelInfoVector: model_infos. Strongly typed list of ModelInfo
        """
    def __init__(self, host_port: str, timeout_ms: int) -> None:
        """
        Creates a python client that can communicate with the corresponding server
        """
    def close(self) -> None:
        """
        Close the connection. It will automatically reopen if needed.
        """
    def read_model(self, mid: int) -> ParameterModel:
        """
        Read and return the model for specified model-identifier (mid)
        
        Args:
            mid (int): the model-identifer for the wanted model
        
        Returns:
            Model: m. The resulting model from the server
        """
    def read_models(self, mids: list[int]) -> ParameterModelVector:
        """
        Read and return the model for specified model-identifier (mid)
        
        Args:
            mids (list[int]): A strongly typed list of ints, the model-identifers for the wanted models
        
        Returns:
            Model: m. The resulting model from the server
        """
    def remove_model(self, mid: int) -> int:
        """
        Remove the specified model bymodel-identifier (mid)
        
        Args:
            mid (int): the model-identifer for the wanted model
        
        Returns:
            int: ec. 0 or error-code?
        """
    def store_model(self, m: ParameterModel, mi: shyft.time_series.ModelInfo) -> int:
        """
        Store the model to backend, if m.id==0 then a new unique model-info is created and used
        
        Args:
            m (Model): The model to store
        
            mi (ModelInfo): The model-info to store for the model
        
        Returns:
            int: mid. model-identifier for the stored model and model-info
        """
    def update_model_info(self, mid: int, mi: shyft.time_series.ModelInfo) -> bool:
        """
        Update the model-info for specified model-identifier(mid)
        
        Args:
            mid (int): model-identifer
        
            mi (ModelInfo): The new updated model-info
        
        Returns:
            bool: ok. true if success
        """
    @property
    def host_port(self) -> str:
        """
        str: Endpoint network address of the remote server.
        """
    @property
    def is_open(self) -> bool:
        """
        bool: If the connection to the remote server is (still) open.
        """
    @property
    def reconnect_count(self) -> int:
        """
        int: Number of reconnects to the remote server that have been performed.
        """
    @property
    def timeout_ms(self) -> int:
        """
        int: Timout for remote server operations, in number milliseconds.
        """
class ParameterModel:
    """
    A model identifier and a strongly typed dict, cpp-map,  of stack parameters, suitable for use with a region-model matching the type.
    """
    __hash__: typing.ClassVar[None] = None
    id: int
    parameters: pt_gs_k.PTGSKParameterMap | pt_ss_k.PTSSKParameterMap | pt_hs_k.PTHSKParameterMap | pt_hps_k.PTHPSKParameterMap | r_pm_gs_k.RPMGSKParameterMap | pt_st_k.PTSTKParameterMap | pt_st_hbv.PTSTHBVParameterMap | r_pt_gs_k.RPTGSKParameterMap | r_pm_st_k.RPMSTKParameterMap | r_pmv_st_k.RPMVSTKParameterMap
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, arg0: ParameterModel) -> bool:
        ...
    def __init__(self) -> None:
        ...
    def __repr__(self) -> str:
        ...
    def __str__(self) -> str:
        ...
class ParameterModelVector:
    """
    A strongly typed list of ParameterModels
    """
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __bool__(self) -> bool:
        """
        Check whether the list is nonempty
        """
    @typing.overload
    def __delitem__(self, arg0: int) -> None:
        """
        Delete the list elements at index ``i``
        """
    @typing.overload
    def __delitem__(self, arg0: slice) -> None:
        """
        Delete list elements using a slice object
        """
    def __eq__(self, arg0: ParameterModelVector) -> bool:
        ...
    @typing.overload
    def __getitem__(self, s: slice) -> ParameterModelVector:
        """
        Retrieve list elements using a slice object
        """
    @typing.overload
    def __getitem__(self, arg0: int) -> ParameterModel:
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, arg0: ParameterModelVector) -> None:
        """
        Copy constructor
        """
    @typing.overload
    def __init__(self, arg0: typing.Iterable) -> None:
        ...
    def __iter__(self) -> typing.Iterator[ParameterModel]:
        ...
    def __len__(self) -> int:
        ...
    def __ne__(self, arg0: ParameterModelVector) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    @typing.overload
    def __setitem__(self, arg0: int, arg1: ParameterModel) -> None:
        ...
    @typing.overload
    def __setitem__(self, arg0: slice, arg1: ParameterModelVector) -> None:
        """
        Assign list elements using a slice object
        """
    def __str__(self) -> str:
        ...
    def append(self, x: ParameterModel) -> None:
        """
        Add an item to the end of the list
        """
    def clear(self) -> None:
        """
        Clear the contents
        """
    @typing.overload
    def extend(self, L: ParameterModelVector) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    @typing.overload
    def extend(self, L: typing.Iterable) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    def insert(self, i: int, x: ParameterModel) -> None:
        """
        Insert an item at a given position.
        """
    @typing.overload
    def pop(self) -> ParameterModel:
        """
        Remove and return the last item
        """
    @typing.overload
    def pop(self, i: int) -> ParameterModel:
        """
        Remove and return the item at index ``i``
        """
    def size(self) -> int:
        ...
class ParameterOptimizer:
    """
    The ParameterOptimizer allows for testing and comparing optimizer methods for the same inital parameters and goal function.
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, p_min: list[float], p_max: list[float], fx: typing.Callable[[list[float]], float]) -> None:
        """
        Initialises a ParameterOptimizer instance.
        
            p_min (Union[DoubleVector[float], DoubleVector[int]]): the lower parameter bound in at least one-dimensional space.
        
            p_max (Union[DoubleVector[float], DoubleVector[int]]): the upper parameter bound in at least one-dimensional space.
        
            fx (Callable): the goal function to be used in the parameter optimization problem.
        """
    def __repr__(self) -> str:
        ...
    def __str__(self) -> str:
        ...
    def goal_function(self, x: list[float]) -> float:
        """
        The goal function to be used in the optimization problem.
        
            x (Union[DoubleVector[float], DoubleVector[int]]): the point in at least one-dimensional space at which the goal function is evaluated.
        
        Returns:
            DoubleVector[float]: y. The function value f(x) at point x.
        """
    def optimize(self, x: list[float], calibration_option: CalibrationOption) -> list[float]:
        """
        Run the optimizer for given parameters and configuration.
        
            x (Union[DoubleVector[float], DoubleVector[int]]): the initial point in at least one-dimensional space for the optimization problem.
        
            calibration_option (CalibrationOption): the calibration for the various optimization methods.
        
        Returns:
            DoubleVector[float]: x_opt. The optimal parameters for which the optimum of f(x) is found.
        
        See also:
            CalibrationOption
        """
    @property
    def fx(self) -> typing.Callable[[list[float]], float]:
        """
        Callable[[],float]: goal function for which parameters are optimized
        """
    @fx.setter
    def fx(self, arg0: typing.Callable[[list[float]], float]) -> None:
        ...
    @property
    def p_max(self) -> list[float]:
        """
        DoubleVector: upper parameter bound in at least one-dimensional space
        """
    @property
    def p_min(self) -> list[float]:
        """
        DoubleVector: lower parameter bound in at least one-dimensional space
        """
class ParameterServer:
    """
    The server-side component for the shyft.hydrology parameter model repository.
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, root_dir: str, config: shyft.time_series.ServerConfig = ...) -> None:
        """
        Creates a server object that serves models from root_dir.
        The root_dir will be create if it does not exists.
        
        Args:
            root_dir (str): Path to the root-directory that keeps/will keep the model-files
        
            config (ServerConfig): Configuration of the server
        """
    def get_listening_port(self) -> int:
        """
        returns the port number it's listening at for serving incoming request
        """
    def get_max_connections(self) -> int:
        """
        returns the maximum number of connections to be served concurrently
        """
    def is_running(self) -> bool:
        """
        true if server is listening and running
        
        See also:
            start_server()
        """
    def set_listening_ip(self, ip: str) -> None:
        """
        set the listening port for the service
        
        Args:
            ip (str): ip or host-name to start listening on
        
        Returns:
            None: nothing. 
        """
    def set_listening_port(self, port_no: int) -> None:
        """
        set the listening port for the service
        
        Args:
            port_no (int): a valid and available tcp-ip port number to listen on.
            typically it could be 20000 (avoid using official reserved numbers)
        
        Returns:
            None: nothing. 
        """
    def set_max_connections(self, max_connect: int) -> None:
        """
        limits simultaneous connections to the server (it's multithreaded, and uses on thread pr. connect)
        
        Args:
            max_connect (int): maximum number of connections before denying more connections
        
        See also:
            get_max_connections()
        """
    def start_server(self) -> int:
        """
        start server listening in background, and processing messages
        
        See also:
            set_listening_port(port_no),is_running
        
        Returns:
            in: port_no. the port used for listening operations, either the value as by set_listening_port, or if it was unspecified, a new available port
        """
    def stop_server(self, timeout: int = 1000) -> None:
        """
        stop serving connections, gracefully.
        
        See also:
            start_server()
        """
    @property
    def stale_connection_close_count(self) -> int:
        """
        int: returns count of connection closed due to stale/no communication activity
        """
class PenmanMonteithCalculator:
    """
    Evapotranspiration model, Penman-Monteith equation, PM
    reference ::
    
      ASCE-EWRI The ASCE Standardized Reference Evapotranspiration Equation Environmental and
         * Water Resources Institute (EWRI) of the American Society of Civil Engineers Task Com- mittee on Standardization of
         * Reference Evapotranspiration Calculation, ASCE, Washington, DC, Environmental and Water Resources Institute (EWRI) of
         * the American Society of Civil Engineers Task Com- mittee on Standardization of Reference Evapotranspiration Calculation,
         * ASCE, Washington, DC, 2005
    
    calculating reference evapotranspiration
    This function is plain and simple, taking albedo and turbidity
    into the constructor and provides function:
    reference_evapotranspiration;
    [mm/s] units.
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, param: PenmanMonteithParameter) -> None:
        ...
    def reference_evapotranspiration(self, response: PenmanMonteithResponse, dt: shyft.time_series.time, net_radiation: float, tempmax: float, tempmin: float, rhumidity: float, elevation: float, windspeed: float) -> None:
        """
        calculates reference evapotranspiration (standard or full based on full_model param), updating response
        """
class PenmanMonteithParameter:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, height_veg: float, height_ws: float, height_t: float, rl: float, full_model: bool) -> None:
        """
        a new object with specified parameters
        """
    @property
    def full_model(self) -> bool:
        """
        bool: boolean to specify if full penman-monteith required, set to false by default, so it is asce-ewri standard
        """
    @full_model.setter
    def full_model(self, arg0: bool) -> None:
        ...
    @property
    def height_t(self) -> float:
        """
        float: typical value 2.0
        """
    @height_t.setter
    def height_t(self, arg0: float) -> None:
        ...
    @property
    def height_veg(self) -> float:
        """
        float: grass 0.15
        """
    @height_veg.setter
    def height_veg(self, arg0: float) -> None:
        ...
    @property
    def height_ws(self) -> float:
        """
        float: typical value 2.0
        """
    @height_ws.setter
    def height_ws(self, arg0: float) -> None:
        ...
    @property
    def rl(self) -> float:
        """
        float: stomatal resistance, for full PM only, 140
        """
    @rl.setter
    def rl(self, arg0: float) -> None:
        ...
class PenmanMonteithResponse:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self) -> None:
        ...
    @property
    def et_ref(self) -> float:
        """
        float: ..
        """
    @et_ref.setter
    def et_ref(self, arg0: float) -> None:
        ...
    @property
    def soil_heat(self) -> float:
        """
        float: ..
        """
    @soil_heat.setter
    def soil_heat(self, arg0: float) -> None:
        ...
class PenmanMonteithVegetationCalculator:
    """
    Evapotranspiration model, Penman-Monteith equation with vegetation properties, PM
    reference ::
    
        L. Dingman Stewart-Penmann-Monteith evapotranspiration
    [mm/s] units.
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, param: PenmanMonteithVegetationParameter) -> None:
        ...
    def evapotranspiration(self, response: PenmanMonteithVegetationResponse, dt: shyft.time_series.time, swin_radiation: float, lw_radiation: float, tempmax: float, tempmin: float, rhumidity: float, lai: float, albedo: float, hvegetation: float, cleaf_max: float, ks: float, soil_moisture_deficit: float, elevation: float, windspeed: float) -> None:
        """
        calculates evapotranspiration, updating response
        """
class PenmanMonteithVegetationParameter:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, height_ws: float, height_t: float) -> None:
        ...
    @property
    def height_t(self) -> float:
        """
        float: typical value 2.0
        """
    @height_t.setter
    def height_t(self, arg0: float) -> None:
        ...
    @property
    def height_ws(self) -> float:
        """
        float: typical value 2.0
        """
    @height_ws.setter
    def height_ws(self, arg0: float) -> None:
        ...
class PenmanMonteithVegetationResponse:
    et: float
    soil_heat: float
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
class PrecipitationCorrectionCalculator:
    """
    Scales precipitation with the specified scale factor
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, scale_factor: float) -> None:
        ...
    def calc(self, precipitation: float) -> float:
        """
        returns scale_factor*precipitation
        """
class PrecipitationCorrectionParameter:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, scale_factor: float = 1.0) -> None:
        ...
    @property
    def scale_factor(self) -> float:
        """
        float: default =1.0
        """
    @scale_factor.setter
    def scale_factor(self, arg0: float) -> None:
        ...
class PrecipitationSource(GeoPointSource):
    """
    geo located precipitation[mm/h]
    """
    vector_t = PrecipitationSourceVector
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self, midpoint: shyft.time_series.GeoPoint, ts: shyft.time_series.TimeSeries) -> None:
        ...
    @typing.overload
    def __init__(self, geo_ts: shyft.time_series.GeoTimeSeries) -> None:
        ...
class PrecipitationSourceVector:
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @staticmethod
    def from_geo_and_ts_vector(geo_points: shyft.time_series.GeoPointVector, tsv: shyft.time_series.TsVector) -> PrecipitationSourceVector:
        """
        Create from a geo_points and corresponding ts-vectors
        
        Args:
            geo_points (GeoPointVector): the geo-points
        
        Returns:
            SourceVector: src_vector. a newly created geo-located vector of specified type
        
            tsv (TsVector): the corresponding time-series located at corresponding geo-point
        """
    def __bool__(self) -> bool:
        """
        Check whether the list is nonempty
        """
    @typing.overload
    def __delitem__(self, arg0: int) -> None:
        """
        Delete the list elements at index ``i``
        """
    @typing.overload
    def __delitem__(self, arg0: slice) -> None:
        """
        Delete list elements using a slice object
        """
    def __eq__(self, arg0: PrecipitationSourceVector) -> bool:
        ...
    @typing.overload
    def __getitem__(self, s: slice) -> PrecipitationSourceVector:
        """
        Retrieve list elements using a slice object
        """
    @typing.overload
    def __getitem__(self, arg0: int) -> PrecipitationSource:
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, arg0: PrecipitationSourceVector) -> None:
        """
        Copy constructor
        """
    @typing.overload
    def __init__(self, arg0: typing.Iterable) -> None:
        ...
    @typing.overload
    def __init__(self, geo_ts_vector: shyft.time_series.GeoTimeSeriesVector) -> None:
        """
        Construct from a GeoTimeSeriesVector,
        Reference to the shyft.time_series.GeoTsMatrix.extract_geo_ts_vector() to see how to create one from the dtss/geo extensions
        """
    def __iter__(self) -> typing.Iterator[PrecipitationSource]:
        ...
    def __len__(self) -> int:
        ...
    def __ne__(self, arg0: PrecipitationSourceVector) -> bool:
        ...
    @typing.overload
    def __setitem__(self, arg0: int, arg1: PrecipitationSource) -> None:
        ...
    @typing.overload
    def __setitem__(self, arg0: slice, arg1: PrecipitationSourceVector) -> None:
        """
        Assign list elements using a slice object
        """
    def append(self, x: PrecipitationSource) -> None:
        """
        Add an item to the end of the list
        """
    def clear(self) -> None:
        """
        Clear the contents
        """
    @typing.overload
    def extend(self, L: PrecipitationSourceVector) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    @typing.overload
    def extend(self, L: typing.Iterable) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    def insert(self, i: int, x: PrecipitationSource) -> None:
        """
        Insert an item at a given position.
        """
    @typing.overload
    def pop(self) -> PrecipitationSource:
        """
        Remove and return the last item
        """
    @typing.overload
    def pop(self, i: int) -> PrecipitationSource:
        """
        Remove and return the item at index ``i``
        """
    def size(self) -> int:
        ...
    def values_at_time(self, arg0: shyft.time_series.time) -> list[float]:
        ...
    @property
    def geo_tsvector(self) -> shyft.time_series.GeoTimeSeriesVector:
        """
        GeoTimeSeriesVector: returns a GeoTimeSeriesVector containing GeoTimeSeries where the geo-points are copied, but time-series are by-reference
        """
class PriestleyTaylorCalculator:
    """
    PriestleyTaylor,PT, (google for PriestleyTaylor)
    primitive implementation for calculating the potential evaporation.
    This function is plain and simple, taking land_albedo and PT.alpha
    into the constructor and provides a function that calculates potential evapotransporation
    [mm/s] units.
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, land_albedo: float, alpha: float) -> None:
        ...
    def potential_evapotranspiration(self, temperature: float, global_radiation: float, rhumidity: float) -> float:
        """
        Calculate PotentialEvapotranspiration, given specified parameters
        
           * param temperature in [degC]
           * param global_radiation [W/m2]
           * param rhumidity in interval [0,1]
           * return PotentialEvapotranspiration in [mm/s] units
           *
        """
class PriestleyTaylorParameter:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, albedo: float = 0.2, alpha: float = 1.26) -> None:
        ...
    @property
    def albedo(self) -> float:
        """
        float: typical value 0.2
        """
    @albedo.setter
    def albedo(self, arg0: float) -> None:
        ...
    @property
    def alpha(self) -> float:
        """
        float: typical value 1.26
        """
    @alpha.setter
    def alpha(self, arg0: float) -> None:
        ...
class PriestleyTaylorResponse:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @property
    def pot_evapotranspiration(self) -> float:
        """
        float: response
        """
    @pot_evapotranspiration.setter
    def pot_evapotranspiration(self, arg0: float) -> None:
        ...
class RadiationCalculator:
    """
    Radiation,R,
    
    reference ::
    
      Allen, R. G.; Trezza, R. & Tasumi, M. Analytical integrated functions for daily solar radiation on slopes Agricultural and Forest Meteorology, 2006, 139, 55-73)
      primitive implementation for calculating predicted clear-sky short-wave solar radiation for inclined surfaces
    
    
    This function is plain and simple, taking albedo and turbidity
    into the constructor and provides 2 functions:
    
     * net_radiation calculates predicted solar radiation (if no measured data available) or translates measured data into the slope plus adds the longwave radiation: instantaneously;
     * net_radiation_step calculates predicted solar radiation or/and translates measured horizontal radiation into sloped surface for the time period between tstart and tend plus adds the lw radiation
    
    Recommended usage is the net_radiation_step for 24h-steps; it was also tested with 1h and 3h steps
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, param: RadiationParameter) -> None:
        ...
    def net_radiation(self, response: RadiationResponse, latitude: float, t1: shyft.time_series.time, dt: shyft.time_series.time, slope: float, aspect: float, temperature: float, rhumidity: float, elevation: float, rsm: float) -> None:
        """
        calculates net radiation using either step or instantaneous method, updating response
        """
    def net_radiation_inst(self, response: RadiationResponse, latitude: float, t: shyft.time_series.time, slope: float, aspect: float, temperature: float, rhumidity: float, elevation: float, rsm: float) -> None:
        """
        calculates net radiation using instantaneous method, updating response
        """
    def net_radiation_step(self, response: RadiationResponse, latitude: float, t1: shyft.time_series.time, dt: shyft.time_series.time, slope: float, aspect: float, temperature: float, rhumidity: float, elevation: float, rsm: float) -> None:
        """
        calculates net radiation, updating response
        """
    def net_radiation_step_asce_st(self, response: RadiationResponse, latitude: float, t1: shyft.time_series.time, dt: shyft.time_series.time, slope: float, aspect: float, temperature: float, rhumidity: float, elevation: float, rsm: float) -> None:
        """
        calculates net radiation, updating response
        """
class RadiationParameter:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, albedo: float = 0.2, turbidity: float = 1.0) -> None:
        ...
    @property
    def albedo(self) -> float:
        """
        float: typical value 0.2
        """
    @albedo.setter
    def albedo(self, arg0: float) -> None:
        ...
    @property
    def turbidity(self) -> float:
        """
        float: typical value 1.0
        """
    @turbidity.setter
    def turbidity(self, arg0: float) -> None:
        ...
class RadiationResponse:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self) -> None:
        ...
    @property
    def net(self) -> float:
        """
        float: ..
        """
    @net.setter
    def net(self, arg0: float) -> None:
        ...
    @property
    def net_lw(self) -> float:
        """
        float: ..
        """
    @net_lw.setter
    def net_lw(self, arg0: float) -> None:
        ...
    @property
    def net_sw(self) -> float:
        """
        float: ..
        """
    @net_sw.setter
    def net_sw(self, arg0: float) -> None:
        ...
    @property
    def ra(self) -> float:
        """
        float: ..
        """
    @ra.setter
    def ra(self, arg0: float) -> None:
        ...
    @property
    def sw_cs_p(self) -> float:
        """
        float: ..
        """
    @sw_cs_p.setter
    def sw_cs_p(self, arg0: float) -> None:
        ...
    @property
    def sw_t(self) -> float:
        """
        float: ..
        """
    @sw_t.setter
    def sw_t(self, arg0: float) -> None:
        ...
class RadiationSource(GeoPointSource):
    """
    geo located radiation[W/m2]
    """
    vector_t = RadiationSourceVector
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self, midpoint: shyft.time_series.GeoPoint, ts: shyft.time_series.TimeSeries) -> None:
        ...
    @typing.overload
    def __init__(self, geo_ts: shyft.time_series.GeoTimeSeries) -> None:
        ...
class RadiationSourceVector:
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @staticmethod
    def from_geo_and_ts_vector(geo_points: shyft.time_series.GeoPointVector, tsv: shyft.time_series.TsVector) -> RadiationSourceVector:
        """
        Create from a geo_points and corresponding ts-vectors
        
        Args:
            geo_points (GeoPointVector): the geo-points
        
        Returns:
            SourceVector: src_vector. a newly created geo-located vector of specified type
        
            tsv (TsVector): the corresponding time-series located at corresponding geo-point
        """
    def __bool__(self) -> bool:
        """
        Check whether the list is nonempty
        """
    @typing.overload
    def __delitem__(self, arg0: int) -> None:
        """
        Delete the list elements at index ``i``
        """
    @typing.overload
    def __delitem__(self, arg0: slice) -> None:
        """
        Delete list elements using a slice object
        """
    def __eq__(self, arg0: RadiationSourceVector) -> bool:
        ...
    @typing.overload
    def __getitem__(self, s: slice) -> RadiationSourceVector:
        """
        Retrieve list elements using a slice object
        """
    @typing.overload
    def __getitem__(self, arg0: int) -> RadiationSource:
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, arg0: RadiationSourceVector) -> None:
        """
        Copy constructor
        """
    @typing.overload
    def __init__(self, arg0: typing.Iterable) -> None:
        ...
    @typing.overload
    def __init__(self, geo_ts_vector: shyft.time_series.GeoTimeSeriesVector) -> None:
        """
        Construct from a GeoTimeSeriesVector,
        Reference to the shyft.time_series.GeoTsMatrix.extract_geo_ts_vector() to see how to create one from the dtss/geo extensions
        """
    def __iter__(self) -> typing.Iterator[RadiationSource]:
        ...
    def __len__(self) -> int:
        ...
    def __ne__(self, arg0: RadiationSourceVector) -> bool:
        ...
    @typing.overload
    def __setitem__(self, arg0: int, arg1: RadiationSource) -> None:
        ...
    @typing.overload
    def __setitem__(self, arg0: slice, arg1: RadiationSourceVector) -> None:
        """
        Assign list elements using a slice object
        """
    def append(self, x: RadiationSource) -> None:
        """
        Add an item to the end of the list
        """
    def clear(self) -> None:
        """
        Clear the contents
        """
    @typing.overload
    def extend(self, L: RadiationSourceVector) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    @typing.overload
    def extend(self, L: typing.Iterable) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    def insert(self, i: int, x: RadiationSource) -> None:
        """
        Insert an item at a given position.
        """
    @typing.overload
    def pop(self) -> RadiationSource:
        """
        Remove and return the last item
        """
    @typing.overload
    def pop(self, i: int) -> RadiationSource:
        """
        Remove and return the item at index ``i``
        """
    def size(self) -> int:
        ...
    def values_at_time(self, arg0: shyft.time_series.time) -> list[float]:
        ...
    @property
    def geo_tsvector(self) -> shyft.time_series.GeoTimeSeriesVector:
        """
        GeoTimeSeriesVector: returns a GeoTimeSeriesVector containing GeoTimeSeries where the geo-points are copied, but time-series are by-reference
        """
class RegionModelType:
    """
    Ref to DrmClient, used do specify what remote region-model type to create
    
    Members:
    
      PT_GS_K
    
      PT_GS_K_OPT
    
      PT_SS_K
    
      PT_SS_K_OPT
    
      PT_HS_K
    
      PT_HS_K_OPT
    
      PT_HPS_K
    
      PT_HPS_K_OPT
    
      R_PM_GS_K
    
      R_PM_GS_K_OPT
    
      PT_ST_K
    
      PT_ST_K_OPT
    
      PT_ST_HBV
    
      PT_ST_K_HBV
    
      R_PT_GS_K
    
      R_PT_GS_K_OPT
    
      R_PM_ST_K
    
      R_PM_ST_K_OPT
    
      R_PMV_ST_K
    
      R_PMV_ST_K_OPT
    """
    PT_GS_K: typing.ClassVar[RegionModelType]  # value = <RegionModelType.PT_GS_K: 0>
    PT_GS_K_OPT: typing.ClassVar[RegionModelType]  # value = <RegionModelType.PT_GS_K_OPT: 1>
    PT_HPS_K: typing.ClassVar[RegionModelType]  # value = <RegionModelType.PT_HPS_K: 6>
    PT_HPS_K_OPT: typing.ClassVar[RegionModelType]  # value = <RegionModelType.PT_HPS_K_OPT: 7>
    PT_HS_K: typing.ClassVar[RegionModelType]  # value = <RegionModelType.PT_HS_K: 4>
    PT_HS_K_OPT: typing.ClassVar[RegionModelType]  # value = <RegionModelType.PT_HS_K_OPT: 5>
    PT_SS_K: typing.ClassVar[RegionModelType]  # value = <RegionModelType.PT_SS_K: 2>
    PT_SS_K_OPT: typing.ClassVar[RegionModelType]  # value = <RegionModelType.PT_SS_K_OPT: 3>
    PT_ST_HBV: typing.ClassVar[RegionModelType]  # value = <RegionModelType.PT_ST_HBV: 12>
    PT_ST_K: typing.ClassVar[RegionModelType]  # value = <RegionModelType.PT_ST_K: 10>
    PT_ST_K_HBV: typing.ClassVar[RegionModelType]  # value = <RegionModelType.PT_ST_K_HBV: 13>
    PT_ST_K_OPT: typing.ClassVar[RegionModelType]  # value = <RegionModelType.PT_ST_K_OPT: 11>
    R_PMV_ST_K: typing.ClassVar[RegionModelType]  # value = <RegionModelType.R_PMV_ST_K: 18>
    R_PMV_ST_K_OPT: typing.ClassVar[RegionModelType]  # value = <RegionModelType.R_PMV_ST_K_OPT: 19>
    R_PM_GS_K: typing.ClassVar[RegionModelType]  # value = <RegionModelType.R_PM_GS_K: 8>
    R_PM_GS_K_OPT: typing.ClassVar[RegionModelType]  # value = <RegionModelType.R_PM_GS_K_OPT: 9>
    R_PM_ST_K: typing.ClassVar[RegionModelType]  # value = <RegionModelType.R_PM_ST_K: 16>
    R_PM_ST_K_OPT: typing.ClassVar[RegionModelType]  # value = <RegionModelType.R_PM_ST_K_OPT: 17>
    R_PT_GS_K: typing.ClassVar[RegionModelType]  # value = <RegionModelType.R_PT_GS_K: 14>
    R_PT_GS_K_OPT: typing.ClassVar[RegionModelType]  # value = <RegionModelType.R_PT_GS_K_OPT: 15>
    __members__: typing.ClassVar[dict[str, RegionModelType]]  # value = {'PT_GS_K': <RegionModelType.PT_GS_K: 0>, 'PT_GS_K_OPT': <RegionModelType.PT_GS_K_OPT: 1>, 'PT_SS_K': <RegionModelType.PT_SS_K: 2>, 'PT_SS_K_OPT': <RegionModelType.PT_SS_K_OPT: 3>, 'PT_HS_K': <RegionModelType.PT_HS_K: 4>, 'PT_HS_K_OPT': <RegionModelType.PT_HS_K_OPT: 5>, 'PT_HPS_K': <RegionModelType.PT_HPS_K: 6>, 'PT_HPS_K_OPT': <RegionModelType.PT_HPS_K_OPT: 7>, 'R_PM_GS_K': <RegionModelType.R_PM_GS_K: 8>, 'R_PM_GS_K_OPT': <RegionModelType.R_PM_GS_K_OPT: 9>, 'PT_ST_K': <RegionModelType.PT_ST_K: 10>, 'PT_ST_K_OPT': <RegionModelType.PT_ST_K_OPT: 11>, 'PT_ST_HBV': <RegionModelType.PT_ST_HBV: 12>, 'PT_ST_K_HBV': <RegionModelType.PT_ST_K_HBV: 13>, 'R_PT_GS_K': <RegionModelType.R_PT_GS_K: 14>, 'R_PT_GS_K_OPT': <RegionModelType.R_PT_GS_K_OPT: 15>, 'R_PM_ST_K': <RegionModelType.R_PM_ST_K: 16>, 'R_PM_ST_K_OPT': <RegionModelType.R_PM_ST_K_OPT: 17>, 'R_PMV_ST_K': <RegionModelType.R_PMV_ST_K: 18>, 'R_PMV_ST_K_OPT': <RegionModelType.R_PMV_ST_K_OPT: 19>}
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
class RelHumSource(GeoPointSource):
    """
    geo located relative humidity[%rh], range 0..1
    """
    vector_t = RelHumSourceVector
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self, midpoint: shyft.time_series.GeoPoint, ts: shyft.time_series.TimeSeries) -> None:
        ...
    @typing.overload
    def __init__(self, geo_ts: shyft.time_series.GeoTimeSeries) -> None:
        ...
class RelHumSourceVector:
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @staticmethod
    def from_geo_and_ts_vector(geo_points: shyft.time_series.GeoPointVector, tsv: shyft.time_series.TsVector) -> RelHumSourceVector:
        """
        Create from a geo_points and corresponding ts-vectors
        
        Args:
            geo_points (GeoPointVector): the geo-points
        
        Returns:
            SourceVector: src_vector. a newly created geo-located vector of specified type
        
            tsv (TsVector): the corresponding time-series located at corresponding geo-point
        """
    def __bool__(self) -> bool:
        """
        Check whether the list is nonempty
        """
    @typing.overload
    def __delitem__(self, arg0: int) -> None:
        """
        Delete the list elements at index ``i``
        """
    @typing.overload
    def __delitem__(self, arg0: slice) -> None:
        """
        Delete list elements using a slice object
        """
    def __eq__(self, arg0: RelHumSourceVector) -> bool:
        ...
    @typing.overload
    def __getitem__(self, s: slice) -> RelHumSourceVector:
        """
        Retrieve list elements using a slice object
        """
    @typing.overload
    def __getitem__(self, arg0: int) -> RelHumSource:
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, arg0: RelHumSourceVector) -> None:
        """
        Copy constructor
        """
    @typing.overload
    def __init__(self, arg0: typing.Iterable) -> None:
        ...
    @typing.overload
    def __init__(self, geo_ts_vector: shyft.time_series.GeoTimeSeriesVector) -> None:
        """
        Construct from a GeoTimeSeriesVector,
        Reference to the shyft.time_series.GeoTsMatrix.extract_geo_ts_vector() to see how to create one from the dtss/geo extensions
        """
    def __iter__(self) -> typing.Iterator[RelHumSource]:
        ...
    def __len__(self) -> int:
        ...
    def __ne__(self, arg0: RelHumSourceVector) -> bool:
        ...
    @typing.overload
    def __setitem__(self, arg0: int, arg1: RelHumSource) -> None:
        ...
    @typing.overload
    def __setitem__(self, arg0: slice, arg1: RelHumSourceVector) -> None:
        """
        Assign list elements using a slice object
        """
    def append(self, x: RelHumSource) -> None:
        """
        Add an item to the end of the list
        """
    def clear(self) -> None:
        """
        Clear the contents
        """
    @typing.overload
    def extend(self, L: RelHumSourceVector) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    @typing.overload
    def extend(self, L: typing.Iterable) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    def insert(self, i: int, x: RelHumSource) -> None:
        """
        Insert an item at a given position.
        """
    @typing.overload
    def pop(self) -> RelHumSource:
        """
        Remove and return the last item
        """
    @typing.overload
    def pop(self, i: int) -> RelHumSource:
        """
        Remove and return the item at index ``i``
        """
    def size(self) -> int:
        ...
    def values_at_time(self, arg0: shyft.time_series.time) -> list[float]:
        ...
    @property
    def geo_tsvector(self) -> shyft.time_series.GeoTimeSeriesVector:
        """
        GeoTimeSeriesVector: returns a GeoTimeSeriesVector containing GeoTimeSeries where the geo-points are copied, but time-series are by-reference
        """
class River:
    """
    A river that we use for routing, its a single piece of a RiverNetwork
    
    The routing river have flow from:
    
      a) zero or more 'cell_nodes',  typically a cell_model type, lateral flow,like cell.rc.average_discharge [m3/s]
      b) zero or more upstream connected rivers, taking their .output_m3s()
    
    then a routing river can *optionally* be connected to a down-stream river
    providing a routing function (currently just a convolution of a uhg).
    
    This definition is recursive, and we use RiverNetwork to ensure the routing graph
    is directed and with no cycles.
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, id: int, downstream: RoutingInfo = ..., parameter: UHGParameter = ...) -> None:
        """
        a new object with specified parameters, notice that a valid river-id|routing-id must be >0
        """
    def uhg(self, dt: shyft.time_series.time) -> list[float]:
        """
        create the hydro-graph for this river, taking specified delta-t, dt,
        static hydrological distance as well as the shape parameters
        alpha and beta used to form the gamma-function.
        The length of the uhg (delay) is determined by the downstream-distance,
        and the velocity parameter. 
        The shape of the uhg is determined by alpha&beta parameters.
        """
    @property
    def downstream(self) -> RoutingInfo:
        """
        RoutingInfo: routing information for downstream, target-id, and hydrological distance
        """
    @downstream.setter
    def downstream(self, arg0: RoutingInfo) -> None:
        ...
    @property
    def id(self) -> int:
        """
        int: a valid identifier >0 for the river|routing element
        """
    @property
    def parameter(self) -> UHGParameter:
        """
        UHGParameter: describing the downstream propagation 
        """
    @parameter.setter
    def parameter(self, arg0: UHGParameter) -> None:
        ...
class RiverNetwork:
    """
    A RiverNetwork takes care of the routing
    see also description of River
    The RiverNetwork provides all needed functions to build
    routing into the region model
    It ensures safe manipulation of rivers:
    
     * no cycles,
     * no duplicate object-id's etc.
    
    
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, clone: RiverNetwork) -> None:
        """
        make a clone of river-network
        """
    def add(self, river: River) -> RiverNetwork:
        """
        add a river to the network, verifies river id, no cycles etc.
        
        raises exception on error
        tip: build your river-network from downstream to upstream order
        """
    def downstream_by_id(self, id: int) -> int:
        """
        return id of downstream river, 0 if none
        """
    def network_contains_directed_cycle(self) -> bool:
        """
        True if network have cycles detected
        """
    def remove_by_id(self, id: int) -> None:
        """
        disconnect and remove river for network
        """
    def river_by_id(self, id: int) -> River:
        """
        get river by id
        """
    def set_downstream_by_id(self, id: int, downstream_id: int) -> None:
        """
        set downstream target for specified river id
        """
    def upstreams_by_id(self, id: int) -> list[int]:
        """
        returns a list(vector) of id of upstream rivers from the specified one
        """
class RoutingInfo:
    """
    Describe the hydrological distance and the id of the target routing element (river)
    """
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, arg0: RoutingInfo) -> bool:
        ...
    def __init__(self, id: int = 0, distance: float = 0.0) -> None:
        ...
    def __ne__(self, arg0: RoutingInfo) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __str__(self) -> str:
        ...
    @property
    def distance(self) -> float:
        """
        float: the hydrological distance, in unit of [m]
        """
    @distance.setter
    def distance(self, arg0: float) -> None:
        ...
    @property
    def id(self) -> int:
        """
        int: id of the target,down-stream river
        """
    @id.setter
    def id(self, arg0: int) -> None:
        ...
class SkaugenCalculator:
    """
    Skaugen snow model method
    
    This algorithm uses theory from Skaugen 
    
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def step(self, delta_t: shyft.time_series.time, parameter: SkaugenParameter, temperature: float, precipitation: float, radiation: float, wind_speed: float, state: SkaugenState, response: SkaugenResponse) -> None:
        """
        steps the model forward delta_t seconds, using specified input, updating state and response
        """
class SkaugenParameter:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self, alpha_0: float = 40.77, d_range: float = 113.0, unit_size: float = 0.1, max_water_fraction: float = 0.1, tx: float = 0.16, cx: float = 2.5, ts: float = 0.14, cfr: float = 0.01) -> None:
        ...
    @typing.overload
    def __init__(self, p: SkaugenParameter) -> None:
        """
        create a clone of p
        """
    @property
    def alpha_0(self) -> float:
        """
        float: default = 40.77
        """
    @alpha_0.setter
    def alpha_0(self, arg0: float) -> None:
        ...
    @property
    def cfr(self) -> float:
        """
        float: default = 0.01
        """
    @cfr.setter
    def cfr(self, arg0: float) -> None:
        ...
    @property
    def cx(self) -> float:
        """
        float: default = 2.5
        """
    @cx.setter
    def cx(self, arg0: float) -> None:
        ...
    @property
    def d_range(self) -> float:
        """
        float: default = 113.0
        """
    @d_range.setter
    def d_range(self, arg0: float) -> None:
        ...
    @property
    def max_water_fraction(self) -> float:
        """
        float: default = 0.1
        """
    @max_water_fraction.setter
    def max_water_fraction(self, arg0: float) -> None:
        ...
    @property
    def ts(self) -> float:
        """
        float: default = 0.14
        """
    @ts.setter
    def ts(self, arg0: float) -> None:
        ...
    @property
    def tx(self) -> float:
        """
        float: default = 0.16
        """
    @tx.setter
    def tx(self, arg0: float) -> None:
        ...
    @property
    def unit_size(self) -> float:
        """
        float: default = 0.1
        """
    @unit_size.setter
    def unit_size(self, arg0: float) -> None:
        ...
class SkaugenResponse:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @property
    def outflow(self) -> float:
        """
        float: from snow-routine in [mm/h] over cell-area
        """
    @outflow.setter
    def outflow(self, arg0: float) -> None:
        ...
    @property
    def sca(self) -> float:
        """
        float: snow-covered area in fraction
        """
    @sca.setter
    def sca(self, arg0: float) -> None:
        ...
    @property
    def swe(self) -> float:
        """
        float: snow water equivalient [mm] over cell-area
        """
    @swe.setter
    def swe(self, arg0: float) -> None:
        ...
    @property
    def total_stored_water(self) -> float:
        """
        float: same as swe(deprecated)
        """
    @total_stored_water.setter
    def total_stored_water(self, arg0: float) -> None:
        ...
class SkaugenState:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self, nu: float = 4.077, alpha: float = 40.77, sca: float = 0.0, swe: float = 0.0, free_water: float = 0.0, residual: float = 0.0, num_units: float = 0) -> None:
        ...
    @typing.overload
    def __init__(self, s: SkaugenState) -> None:
        """
        create a clone of s
        """
    @property
    def alpha(self) -> float:
        """
        float: ..
        """
    @alpha.setter
    def alpha(self, arg0: float) -> None:
        ...
    @property
    def free_water(self) -> float:
        """
        float: ..
        """
    @free_water.setter
    def free_water(self, arg0: float) -> None:
        ...
    @property
    def nu(self) -> float:
        """
        float: ..
        """
    @nu.setter
    def nu(self, arg0: float) -> None:
        ...
    @property
    def num_units(self) -> int:
        """
        int: ..
        """
    @num_units.setter
    def num_units(self, arg0: int) -> None:
        ...
    @property
    def residual(self) -> float:
        """
        float: ..
        """
    @residual.setter
    def residual(self, arg0: float) -> None:
        ...
    @property
    def sca(self) -> float:
        """
        float: ..
        """
    @sca.setter
    def sca(self, arg0: float) -> None:
        ...
    @property
    def swe(self) -> float:
        """
        float: ..
        """
    @swe.setter
    def swe(self, arg0: float) -> None:
        ...
class SnowTilesCalculator:
    """
    Tile based snow model method
    
    This algorithm uses ...
    
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, parameter: SnowTilesParameter) -> None:
        ...
    def step(self, state: SnowTilesState, response: SnowTilesResponse, t0: shyft.time_series.time, t1: shyft.time_series.time, precipitation: float, temperature: float) -> None:
        """
        steps the model forward from t0 to t1, updating state and response
        """
class SnowTilesParameter:
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, arg0: SnowTilesParameter) -> bool:
        ...
    def __init__(self, shape: float = 2.0, tx: float = 0.0, cx: float = 1.0, ts: float = 0.0, lwmax: float = 0.1, cfr: float = 0.5, area_fractions: list[float] = [0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1]) -> None:
        ...
    def __ne__(self, arg0: SnowTilesParameter) -> bool:
        ...
    @property
    def _multiply(self) -> numpy.ndarray:
        """
        DoubleVector: multiplication factors for distributing solid precipitation between the tiles (mean of elements = 1)
        """
    @property
    def area_fractions(self) -> list[float]:
        """
        DoubleVector: area fractions of the individual tiles, unit (-), range per tile [0.01, 1], the sum of the elements in the vector must equal 1
        """
    @area_fractions.setter
    def area_fractions(self, arg1: list[float]) -> None:
        ...
    @property
    def cfr(self) -> float:
        """
        float: refreeze coefficient, unit (-), range [0, 1]
        """
    @cfr.setter
    def cfr(self, arg0: float) -> None:
        ...
    @property
    def cx(self) -> float:
        """
        float: degree-day melt factor, unit (mm/C/day), range [0, 30]
        """
    @cx.setter
    def cx(self, arg0: float) -> None:
        ...
    @property
    def lwmax(self) -> float:
        """
        float: max liquid water content given as a fraction of ice in the snowpack, unit (-), range [0, 1]
        """
    @lwmax.setter
    def lwmax(self, arg0: float) -> None:
        ...
    @property
    def shape(self) -> float:
        """
        float: shape parameter of the Gamma distribution defining the multiplication factors for snowfall, unit (-), range [0.1, inf]
        """
    @shape.setter
    def shape(self, arg1: float) -> None:
        ...
    @property
    def ts(self) -> float:
        """
        float: threshold temperature for melt onset, unit (C), range [-4, 4]
        """
    @ts.setter
    def ts(self, arg0: float) -> None:
        ...
    @property
    def tx(self) -> float:
        """
        float: threshold temperature determining precipitation phase, unit (C), range [-4, 4]
        """
    @tx.setter
    def tx(self, arg0: float) -> None:
        ...
class SnowTilesResponse:
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self) -> None:
        ...
    @property
    def outflow(self) -> float:
        """
        average outflow, rain and snowmelt, for the cell, unit (mm/hour)
        """
    @outflow.setter
    def outflow(self, arg0: float) -> None:
        ...
    @property
    def sca(self) -> float:
        """
        float: average sca as a fraction of total cell area, unit (-)
        """
    @sca.setter
    def sca(self, arg0: float) -> None:
        ...
    @property
    def swe(self) -> float:
        """
        float: average swe for the cell, unit (mm)
        """
    @swe.setter
    def swe(self, arg0: float) -> None:
        ...
class SnowTilesState:
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, arg0: SnowTilesState) -> bool:
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, fw: list[float], lw: list[float]) -> None:
        ...
    def __ne__(self, arg0: SnowTilesState) -> bool:
        ...
    @property
    def fw(self) -> list[float]:
        """
        DoubleVector: frozen water in the snowpack, unit (mm)
        """
    @fw.setter
    def fw(self, arg0: list[float]) -> None:
        ...
    @property
    def lw(self) -> list[float]:
        """
        DoubleVector: liquid water in the snowpack, unit (mm)
        """
    @lw.setter
    def lw(self, arg0: list[float]) -> None:
        ...
class StateClient:
    """
    The client api for the shyft.hydrology state model service.
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @staticmethod
    def get_model_infos(*args, **kwargs) -> list[shyft.time_series.ModelInfo]:
        """
        returns all or selected model-info objects based on model-identifiers(mids)
        
        Args:
            mids (IntVector): empty = all, or a list of known exisiting model-identifiers
        
            created_in (UtcPeriod): For which period you are interested in model-infos.
        
        Returns:
            ModelInfoVector: model_infos. Strongly typed list of ModelInfo
        """
    def __init__(self, host_port: str, timeout_ms: int) -> None:
        """
        Creates a python client that can communicate with the corresponding server
        """
    def close(self) -> None:
        """
        Close the connection. It will automatically reopen if needed.
        """
    def read_model(self, mid: int) -> StateModel:
        """
        Read and return the model for specified model-identifier (mid)
        
        Args:
            mid (int): the model-identifer for the wanted model
        
        Returns:
            Model: m. The resulting model from the server
        """
    def read_models(self, mids: list[int]) -> StateModelVector:
        """
        Read and return the model for specified model-identifier (mid)
        
        Args:
            mids (list[int]): A strongly typed list of ints, the model-identifers for the wanted models
        
        Returns:
            Model: m. The resulting model from the server
        """
    def remove_model(self, mid: int) -> int:
        """
        Remove the specified model bymodel-identifier (mid)
        
        Args:
            mid (int): the model-identifer for the wanted model
        
        Returns:
            int: ec. 0 or error-code?
        """
    def store_model(self, m: StateModel, mi: shyft.time_series.ModelInfo) -> int:
        """
        Store the model to backend, if m.id==0 then a new unique model-info is created and used
        
        Args:
            m (Model): The model to store
        
            mi (ModelInfo): The model-info to store for the model
        
        Returns:
            int: mid. model-identifier for the stored model and model-info
        """
    def update_model_info(self, mid: int, mi: shyft.time_series.ModelInfo) -> bool:
        """
        Update the model-info for specified model-identifier(mid)
        
        Args:
            mid (int): model-identifer
        
            mi (ModelInfo): The new updated model-info
        
        Returns:
            bool: ok. true if success
        """
    @property
    def host_port(self) -> str:
        """
        str: Endpoint network address of the remote server.
        """
    @property
    def is_open(self) -> bool:
        """
        bool: If the connection to the remote server is (still) open.
        """
    @property
    def reconnect_count(self) -> int:
        """
        int: Number of reconnects to the remote server that have been performed.
        """
    @property
    def timeout_ms(self) -> int:
        """
        int: Timout for remote server operations, in number milliseconds.
        """
class StateModel:
    """
    A model identifier and a StateVector, where the vector can keep an uniform set of StateId of specified type
    """
    __hash__: typing.ClassVar[None] = None
    states: pt_gs_k.PTGSKStateWithIdVector | pt_ss_k.PTSSKStateWithIdVector | pt_hs_k.PTHSKStateWithIdVector | pt_hps_k.PTHPSKStateWithIdVector | r_pm_gs_k.RPMGSKStateWithIdVector | pt_st_k.PTSTKStateWithIdVector | pt_st_hbv.PTSTHBVStateWithIdVector | r_pt_gs_k.RPTGSKStateWithIdVector | r_pm_st_k.RPMSTKStateWithIdVector | r_pmv_st_k.RPMVSTKStateWithIdVector
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, arg0: StateModel) -> bool:
        ...
    def __init__(self) -> None:
        ...
    def __repr__(self) -> str:
        ...
    def __str__(self) -> str:
        ...
    @property
    def id(self) -> int:
        """
        int: unique model id
        """
    @id.setter
    def id(self, arg0: int) -> None:
        ...
class StateModelVector:
    """
    A strongly typed list of StateModel's
    """
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __bool__(self) -> bool:
        """
        Check whether the list is nonempty
        """
    @typing.overload
    def __delitem__(self, arg0: int) -> None:
        """
        Delete the list elements at index ``i``
        """
    @typing.overload
    def __delitem__(self, arg0: slice) -> None:
        """
        Delete list elements using a slice object
        """
    def __eq__(self, arg0: StateModelVector) -> bool:
        ...
    @typing.overload
    def __getitem__(self, s: slice) -> StateModelVector:
        """
        Retrieve list elements using a slice object
        """
    @typing.overload
    def __getitem__(self, arg0: int) -> StateModel:
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, arg0: StateModelVector) -> None:
        """
        Copy constructor
        """
    @typing.overload
    def __init__(self, arg0: typing.Iterable) -> None:
        ...
    def __iter__(self) -> typing.Iterator[StateModel]:
        ...
    def __len__(self) -> int:
        ...
    def __ne__(self, arg0: StateModelVector) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    @typing.overload
    def __setitem__(self, arg0: int, arg1: StateModel) -> None:
        ...
    @typing.overload
    def __setitem__(self, arg0: slice, arg1: StateModelVector) -> None:
        """
        Assign list elements using a slice object
        """
    def __str__(self) -> str:
        ...
    def append(self, x: StateModel) -> None:
        """
        Add an item to the end of the list
        """
    def clear(self) -> None:
        """
        Clear the contents
        """
    @typing.overload
    def extend(self, L: StateModelVector) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    @typing.overload
    def extend(self, L: typing.Iterable) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    def insert(self, i: int, x: StateModel) -> None:
        """
        Insert an item at a given position.
        """
    @typing.overload
    def pop(self) -> StateModel:
        """
        Remove and return the last item
        """
    @typing.overload
    def pop(self, i: int) -> StateModel:
        """
        Remove and return the item at index ``i``
        """
    def size(self) -> int:
        ...
class StateServer:
    """
    The server-side component for the shyft.hydrology state model repository.
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, root_dir: str, config: shyft.time_series.ServerConfig = ...) -> None:
        """
        Creates a server object that serves models from root_dir.
        The root_dir will be create if it does not exists.
        
        Args:
            root_dir (str): Path to the root-directory that keeps/will keep the model-files
        
            config (ServerConfig): Configuration of the server
        """
    def get_listening_port(self) -> int:
        """
        returns the port number it's listening at for serving incoming request
        """
    def get_max_connections(self) -> int:
        """
        returns the maximum number of connections to be served concurrently
        """
    def is_running(self) -> bool:
        """
        true if server is listening and running
        
        See also:
            start_server()
        """
    def set_listening_ip(self, ip: str) -> None:
        """
        set the listening port for the service
        
        Args:
            ip (str): ip or host-name to start listening on
        
        Returns:
            None: nothing. 
        """
    def set_listening_port(self, port_no: int) -> None:
        """
        set the listening port for the service
        
        Args:
            port_no (int): a valid and available tcp-ip port number to listen on.
            typically it could be 20000 (avoid using official reserved numbers)
        
        Returns:
            None: nothing. 
        """
    def set_max_connections(self, max_connect: int) -> None:
        """
        limits simultaneous connections to the server (it's multithreaded, and uses on thread pr. connect)
        
        Args:
            max_connect (int): maximum number of connections before denying more connections
        
        See also:
            get_max_connections()
        """
    def start_server(self) -> int:
        """
        start server listening in background, and processing messages
        
        See also:
            set_listening_port(port_no),is_running
        
        Returns:
            in: port_no. the port used for listening operations, either the value as by set_listening_port, or if it was unspecified, a new available port
        """
    def stop_server(self, timeout: int = 1000) -> None:
        """
        stop serving connections, gracefully.
        
        See also:
            start_server()
        """
    @property
    def stale_connection_close_count(self) -> int:
        """
        int: returns count of connection closed due to stale/no communication activity
        """
class TargetSpecCalcType:
    """
    Members:
    
      NASH_SUTCLIFFE
    
      KLING_GUPTA
    
      ABS_DIFF
    
      RMSE
    """
    ABS_DIFF: typing.ClassVar[TargetSpecCalcType]  # value = <TargetSpecCalcType.ABS_DIFF: 2>
    KLING_GUPTA: typing.ClassVar[TargetSpecCalcType]  # value = <TargetSpecCalcType.KLING_GUPTA: 1>
    NASH_SUTCLIFFE: typing.ClassVar[TargetSpecCalcType]  # value = <TargetSpecCalcType.NASH_SUTCLIFFE: 0>
    RMSE: typing.ClassVar[TargetSpecCalcType]  # value = <TargetSpecCalcType.RMSE: 3>
    __members__: typing.ClassVar[dict[str, TargetSpecCalcType]]  # value = {'NASH_SUTCLIFFE': <TargetSpecCalcType.NASH_SUTCLIFFE: 0>, 'KLING_GUPTA': <TargetSpecCalcType.KLING_GUPTA: 1>, 'ABS_DIFF': <TargetSpecCalcType.ABS_DIFF: 2>, 'RMSE': <TargetSpecCalcType.RMSE: 3>}
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
class TargetSpecificationPts:
    """
    To guide the model calibration, we have a goal-function that we try to minimize
    This class contains the needed specification of this goal-function so that we can:
    
     1. from simulations, collect time-series at catchment level for (DISCHARGE|SNOW_COVERED_AREA|SNOW_WATER_EQUIVALENT)
     2. from observations, have a user-specified series the expression above should be equal to
     3. use user specified kling-gupta factors to evaluate kind-of-difference between target and simulated
     4. scale-factor to put a weight on this specific target-specification compared to other(we can have multiple)
     5. a user specified id, uid, a string to identify the external source for calibration
     6. a user specified time-series expression with symbolic ref to `sim`, like `TimeSeries('sim').log()`
    
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, ts: shyft.time_series.TsFixed, cids: list[int], scale_factor: float, calc_mode: TargetSpecCalcType, s_r: float, s_a: float, s_b: float, catchment_property: CatchmentPropertyType, uid: str) -> None:
        """
        construct a target specification filled in with supplied parameters
        
        Args:
            ts (TsFixed): time-series containing the target time-series
        
            cids (IntVector): A list of catchment id's(cids) that together adds up into same as the target-ts
        
            scale_factor (float): the weight of this target-specification
        
            calc_mode (TargetSpecCalcType): specifies how to calculate the goal function, NS, KG, Abs,RMSE method
        
            s_r (float): KG scalefactor for correlation
        
            s_a (float): KG scalefactor for alpha(variance)
        
            s_b (float): KG scalefactor for beta(bias)
        
            catchment_property (CatchmentPropertyType): what to extract from catchment(DISCHARGE|SNOW_COVERED_AREA|SNOW_WATER_EQUIVALENT|ROUTED_DISCHARGE|CELL_CHARGE)
        
            uid (str): user specified string/id to help integration efforts
        """
    @typing.overload
    def __init__(self, ts: shyft.time_series.TimeSeries, cids: list[int], scale_factor: float, calc_mode: TargetSpecCalcType, s_r: float, s_a: float, s_b: float, catchment_property: CatchmentPropertyType, uid: str) -> None:
        """
        construct a target specification filled in with supplied parameters
        
        Args:
            ts (TimeSeries): time-series containing the target time-series, note that the time-axis of this ts must be a fixed-interval type
        
            cids (IntVector): A list of catchment id's(cids) that together adds up into same as the target-ts
        
            scale_factor (float): the weight of this target-specification
        
            calc_mode (TargetSpecCalcType): specifies how to calculate the goal function, NS, KG, Abs method
        
            s_r (float): KG scalefactor for correlation
        
            s_a (float): KG scalefactor for alpha(variance)
        
            s_b (float): KG scalefactor for beta(bias)
        
            catchment_property (CatchmentPropertyType): what to extract from catchment(DISCHARGE|SNOW_COVERED_AREA|SNOW_WATER_EQUIVALENT|ROUTED_DISCHARGE|CELL_CHARGE)
        
            uid (str): user specified string/id to help integration efforts
        """
    @typing.overload
    def __init__(self, ts: shyft.time_series.TsFixed, cids: list[int], scale_factor: float, calc_mode: TargetSpecCalcType) -> None:
        """
        construct a target specification filled in with supplied parameters
        
        Args:
            ts (TsFixed): time-series containing the target time-series
        
            cids (IntVector): A list of catchment id's(cids) that together adds up into same as the target-ts
        
            scale_factor (float): the weight of this target-specification
        
            calc_mode (TargetSpecCalcType): specifies how to calculate the goal function, NS, KG, Abs method
        """
    @typing.overload
    def __init__(self, ts: shyft.time_series.TimeSeries, cids: list[int], scale_factor: float, calc_mode: TargetSpecCalcType) -> None:
        """
        construct a target specification filled in with supplied parameters
        
        Args:
            ts (TimeSeries): time-series containing the target time-series, note the time-axis needs to be fixed_dt!
        
            cids (IntVector): A list of catchment id's(cids) that together adds up into same as the target-ts
        
            scale_factor (float): the weight of this target-specification
        
            calc_mode (TargetSpecCalcType): specifies how to calculate the goal function, NS, KG, Abs method
        """
    @typing.overload
    def __init__(self, ts: shyft.time_series.TsFixed, rid: int, scale_factor: float, calc_mode: TargetSpecCalcType, s_r: float, s_a: float, s_b: float, uid: str) -> None:
        """
        construct a target specification filled in with supplied parameters
        
        Args:
            ts (TsFixed): time-series containing the target time-series
        
            rid (int): A river-id identifying the point of flow in the river-network
        
            scale_factor (float): the weight of this target-specification
        
            calc_mode (TargetSpecCalcType): specifies how to calculate the goal function, NS, KG, Abs method
        
            s_r (float): KG scalefactor for correlation
        
            s_a (float): KG scalefactor for alpha(variance)
        
            s_b (float): KG scalefactor for beta(bias)
        
            uid (str): user specified string/id to help integration efforts
        """
    @typing.overload
    def __init__(self, ts: shyft.time_series.TimeSeries, rid: int, scale_factor: float, calc_mode: TargetSpecCalcType, s_r: float, s_a: float, s_b: float, uid: str) -> None:
        """
        construct a target specification filled in with supplied parameters
        
        Args:
            ts (TimeSeries): time-series containing the target time-series, note time-axis required to be fixed-dt type
        
            rid (int): A river-id identifying the point of flow in the river-network
        
            scale_factor (float): the weight of this target-specification
        
            calc_mode (TargetSpecCalcType): specifies how to calculate the goal function, NS, KG, Abs method
        
            s_r (float): KG scalefactor for correlation
        
            s_a (float): KG scalefactor for alpha(variance)
        
            s_b (float): KG scalefactor for beta(bias)
        
            uid (str): user specified string/id to help integration efforts
        """
    def __repr__(self) -> str:
        ...
    def __str__(self) -> str:
        ...
    def evaluate_fx(self, ts: shyft.time_series.TimeSeries) -> shyft.time_series.TimeSeries:
        """
        Compute the evaluated result using the `fx` expression.
        
        Args:
            ts (TimeSeries): 
        
        Returns:
            TimeSeries: result. The evaluated result using the passed time-series as substitute to symbolic reference(s).
        """
    @property
    def calc_mode(self) -> TargetSpecCalcType:
        """
        TargetSpecCalcType: NASH_SUTCLIFFE, KLING_GUPTA,ABS_DIFF,RMSE
        """
    @calc_mode.setter
    def calc_mode(self, arg0: TargetSpecCalcType) -> None:
        ...
    @property
    def catchment_indexes(self) -> list[int]:
        """
        IntVector: catchment indexes, 'cids'
        """
    @catchment_indexes.setter
    def catchment_indexes(self, arg0: list[int]) -> None:
        ...
    @property
    def catchment_property(self) -> CatchmentPropertyType:
        """
        CatchmentPropertyType: DISCHARGE,SNOW_COVERED_AREA, SNOW_WATER_EQUIVALENT
        """
    @catchment_property.setter
    def catchment_property(self, arg0: CatchmentPropertyType) -> None:
        ...
    @property
    def fx(self) -> shyft.time_series.TimeSeries:
        """
        TimeSeries: an expression to be applied to the simulated ts, e.g.:`TimeSeries('sim').log()`
        """
    @fx.setter
    def fx(self, arg1: shyft.time_series.TimeSeries) -> None:
        ...
    @property
    def river_id(self) -> int:
        """
        int: river identifier for routed discharge calibration
        """
    @river_id.setter
    def river_id(self, arg0: int) -> None:
        ...
    @property
    def s_a(self) -> float:
        """
        float: KG-scalefactor for alpha (variance)
        """
    @s_a.setter
    def s_a(self, arg0: float) -> None:
        ...
    @property
    def s_b(self) -> float:
        """
        float: KG-scalefactor for beta (bias)
        """
    @s_b.setter
    def s_b(self, arg0: float) -> None:
        ...
    @property
    def s_r(self) -> float:
        """
        float: KG-scalefactor for correlation
        """
    @s_r.setter
    def s_r(self, arg0: float) -> None:
        ...
    @property
    def scale_factor(self) -> float:
        """
        float: the scale factor to be used when considering multiple target_specifications
        """
    @scale_factor.setter
    def scale_factor(self, arg0: float) -> None:
        ...
    @property
    def ts(self) -> shyft.time_series.TimeSeries:
        """
        TimeSeries: target ts
        """
    @ts.setter
    def ts(self, arg0: shyft.time_series.TimeSeries) -> None:
        ...
    @property
    def uid(self) -> str:
        """
        str: user specified identifier:string
        """
    @uid.setter
    def uid(self, arg0: str) -> None:
        ...
class TargetSpecificationVector:
    """
    A list of (weighted) target specifications to be used for model calibration
    """
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __bool__(self) -> bool:
        """
        Check whether the list is nonempty
        """
    @typing.overload
    def __delitem__(self, arg0: int) -> None:
        """
        Delete the list elements at index ``i``
        """
    @typing.overload
    def __delitem__(self, arg0: slice) -> None:
        """
        Delete list elements using a slice object
        """
    def __eq__(self, arg0: TargetSpecificationVector) -> bool:
        ...
    @typing.overload
    def __getitem__(self, s: slice) -> TargetSpecificationVector:
        """
        Retrieve list elements using a slice object
        """
    @typing.overload
    def __getitem__(self, arg0: int) -> TargetSpecificationPts:
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, arg0: TargetSpecificationVector) -> None:
        """
        Copy constructor
        """
    @typing.overload
    def __init__(self, arg0: typing.Iterable) -> None:
        ...
    def __iter__(self) -> typing.Iterator[TargetSpecificationPts]:
        ...
    def __len__(self) -> int:
        ...
    def __ne__(self, arg0: TargetSpecificationVector) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    @typing.overload
    def __setitem__(self, arg0: int, arg1: TargetSpecificationPts) -> None:
        ...
    @typing.overload
    def __setitem__(self, arg0: slice, arg1: TargetSpecificationVector) -> None:
        """
        Assign list elements using a slice object
        """
    def __str__(self) -> str:
        ...
    def append(self, x: TargetSpecificationPts) -> None:
        """
        Add an item to the end of the list
        """
    def clear(self) -> None:
        """
        Clear the contents
        """
    @typing.overload
    def extend(self, L: TargetSpecificationVector) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    @typing.overload
    def extend(self, L: typing.Iterable) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    def insert(self, i: int, x: TargetSpecificationPts) -> None:
        """
        Insert an item at a given position.
        """
    @typing.overload
    def pop(self) -> TargetSpecificationPts:
        """
        Remove and return the last item
        """
    @typing.overload
    def pop(self, i: int) -> TargetSpecificationPts:
        """
        Remove and return the item at index ``i``
        """
    def size(self) -> int:
        ...
class TemperatureSource(GeoPointSource):
    """
    geo located temperatures[deg Celcius]
    """
    vector_t = TemperatureSourceVector
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self, midpoint: shyft.time_series.GeoPoint, ts: shyft.time_series.TimeSeries) -> None:
        ...
    @typing.overload
    def __init__(self, geo_ts: shyft.time_series.GeoTimeSeries) -> None:
        ...
class TemperatureSourceVector:
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @staticmethod
    def from_geo_and_ts_vector(geo_points: shyft.time_series.GeoPointVector, tsv: shyft.time_series.TsVector) -> TemperatureSourceVector:
        """
        Create from a geo_points and corresponding ts-vectors
        
        Args:
            geo_points (GeoPointVector): the geo-points
        
        Returns:
            SourceVector: src_vector. a newly created geo-located vector of specified type
        
            tsv (TsVector): the corresponding time-series located at corresponding geo-point
        """
    def __bool__(self) -> bool:
        """
        Check whether the list is nonempty
        """
    @typing.overload
    def __delitem__(self, arg0: int) -> None:
        """
        Delete the list elements at index ``i``
        """
    @typing.overload
    def __delitem__(self, arg0: slice) -> None:
        """
        Delete list elements using a slice object
        """
    def __eq__(self, arg0: TemperatureSourceVector) -> bool:
        ...
    @typing.overload
    def __getitem__(self, s: slice) -> TemperatureSourceVector:
        """
        Retrieve list elements using a slice object
        """
    @typing.overload
    def __getitem__(self, arg0: int) -> TemperatureSource:
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, arg0: TemperatureSourceVector) -> None:
        """
        Copy constructor
        """
    @typing.overload
    def __init__(self, arg0: typing.Iterable) -> None:
        ...
    @typing.overload
    def __init__(self, geo_ts_vector: shyft.time_series.GeoTimeSeriesVector) -> None:
        """
        Construct from a GeoTimeSeriesVector,
        Reference to the shyft.time_series.GeoTsMatrix.extract_geo_ts_vector() to see how to create one from the dtss/geo extensions
        """
    def __iter__(self) -> typing.Iterator[TemperatureSource]:
        ...
    def __len__(self) -> int:
        ...
    def __ne__(self, arg0: TemperatureSourceVector) -> bool:
        ...
    @typing.overload
    def __setitem__(self, arg0: int, arg1: TemperatureSource) -> None:
        ...
    @typing.overload
    def __setitem__(self, arg0: slice, arg1: TemperatureSourceVector) -> None:
        """
        Assign list elements using a slice object
        """
    def append(self, x: TemperatureSource) -> None:
        """
        Add an item to the end of the list
        """
    def clear(self) -> None:
        """
        Clear the contents
        """
    @typing.overload
    def extend(self, L: TemperatureSourceVector) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    @typing.overload
    def extend(self, L: typing.Iterable) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    def insert(self, i: int, x: TemperatureSource) -> None:
        """
        Insert an item at a given position.
        """
    @typing.overload
    def pop(self) -> TemperatureSource:
        """
        Remove and return the last item
        """
    @typing.overload
    def pop(self, i: int) -> TemperatureSource:
        """
        Remove and return the item at index ``i``
        """
    def size(self) -> int:
        ...
    def values_at_time(self, arg0: shyft.time_series.time) -> list[float]:
        ...
    @property
    def geo_tsvector(self) -> shyft.time_series.GeoTimeSeriesVector:
        """
        GeoTimeSeriesVector: returns a GeoTimeSeriesVector containing GeoTimeSeries where the geo-points are copied, but time-series are by-reference
        """
class TsTransform:
    """
    transform the supplied time-series, f(t) interpreted according to its point_interpretation() policy
    into a new shyft core TsFixed time-series,
    that represents the true average for each of the n intervals of length dt, starting at start.
    the result ts will have the policy is set to POINT_AVERAGE_VALUE
    note:  that the resulting ts is a fresh new ts, not connected to the source ts
    
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self) -> None:
        ...
    @typing.overload
    def to_average(self, start: shyft.time_series.time, dt: shyft.time_series.time, n: int, src: shyft.time_series.TimeSeries) -> shyft.time_series.TsFixed:
        ...
    @typing.overload
    def to_average(self, start: shyft.time_series.time, dt: shyft.time_series.time, n: int, src: shyft.time_series.TimeSeries) -> shyft.time_series.TsFixed:
        ...
    @typing.overload
    def to_average(self, start: shyft.time_series.time, dt: shyft.time_series.time, n: int, src: shyft.time_series.TsFixed) -> shyft.time_series.TsFixed:
        ...
    @typing.overload
    def to_average(self, start: shyft.time_series.time, dt: shyft.time_series.time, n: int, src: shyft.time_series.TsFixed) -> shyft.time_series.TsFixed:
        ...
    @typing.overload
    def to_average(self, start: int, dt: int, n: int, src: shyft.time_series.TimeSeries) -> shyft.time_series.TsFixed:
        ...
    @typing.overload
    def to_average(self, start: int, dt: int, n: int, src: shyft.time_series.TimeSeries) -> shyft.time_series.TsFixed:
        ...
    @typing.overload
    def to_average(self, start: int, dt: int, n: int, src: shyft.time_series.TsFixed) -> shyft.time_series.TsFixed:
        ...
    @typing.overload
    def to_average(self, start: int, dt: int, n: int, src: shyft.time_series.TsFixed) -> shyft.time_series.TsFixed:
        ...
class UHGParameter:
    """
    The Unit Hydro Graph Parameter contains sufficient
    description to create a unit hydro graph, that have a shape
    and a discretized 'time-length' according to the model time-step resolution.
    Currently we use a gamma function for the shape:
    <a ref='https://en.wikipedia.org/wiki/Gamma_distribution'>gamma distribution</a>
    
    """
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __init__(self, velocity: float = 1.0, alpha: float = 7.0, beta: float = 0.0) -> None:
        ...
    def __repr__(self) -> str:
        ...
    def __str__(self) -> str:
        ...
    @property
    def alpha(self) -> float:
        """
        float: default 3.0,unit-less, ref. shape of gamma-function
        """
    @alpha.setter
    def alpha(self, arg0: float) -> None:
        ...
    @property
    def beta(self) -> float:
        """
        float: default 0.0, unit-less, added to pdf of gamma(alpha,1.0)
        """
    @beta.setter
    def beta(self, arg0: float) -> None:
        ...
    @property
    def velocity(self) -> float:
        """
        float: default 1.0, unit [m/s]
        """
    @velocity.setter
    def velocity(self, arg0: float) -> None:
        ...
class WindSpeedSource(GeoPointSource):
    """
    geo located wind speeds[m/s]
    """
    vector_t = WindSpeedSourceVector
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @typing.overload
    def __init__(self, midpoint: shyft.time_series.GeoPoint, ts: shyft.time_series.TimeSeries) -> None:
        ...
    @typing.overload
    def __init__(self, geo_ts: shyft.time_series.GeoTimeSeries) -> None:
        ...
class WindSpeedSourceVector:
    __hash__: typing.ClassVar[None] = None
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    @staticmethod
    def from_geo_and_ts_vector(geo_points: shyft.time_series.GeoPointVector, tsv: shyft.time_series.TsVector) -> WindSpeedSourceVector:
        """
        Create from a geo_points and corresponding ts-vectors
        
        Args:
            geo_points (GeoPointVector): the geo-points
        
        Returns:
            SourceVector: src_vector. a newly created geo-located vector of specified type
        
            tsv (TsVector): the corresponding time-series located at corresponding geo-point
        """
    def __bool__(self) -> bool:
        """
        Check whether the list is nonempty
        """
    @typing.overload
    def __delitem__(self, arg0: int) -> None:
        """
        Delete the list elements at index ``i``
        """
    @typing.overload
    def __delitem__(self, arg0: slice) -> None:
        """
        Delete list elements using a slice object
        """
    def __eq__(self, arg0: WindSpeedSourceVector) -> bool:
        ...
    @typing.overload
    def __getitem__(self, s: slice) -> WindSpeedSourceVector:
        """
        Retrieve list elements using a slice object
        """
    @typing.overload
    def __getitem__(self, arg0: int) -> WindSpeedSource:
        ...
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, arg0: WindSpeedSourceVector) -> None:
        """
        Copy constructor
        """
    @typing.overload
    def __init__(self, arg0: typing.Iterable) -> None:
        ...
    @typing.overload
    def __init__(self, geo_ts_vector: shyft.time_series.GeoTimeSeriesVector) -> None:
        """
        Construct from a GeoTimeSeriesVector,
        Reference to the shyft.time_series.GeoTsMatrix.extract_geo_ts_vector() to see how to create one from the dtss/geo extensions
        """
    def __iter__(self) -> typing.Iterator[WindSpeedSource]:
        ...
    def __len__(self) -> int:
        ...
    def __ne__(self, arg0: WindSpeedSourceVector) -> bool:
        ...
    @typing.overload
    def __setitem__(self, arg0: int, arg1: WindSpeedSource) -> None:
        ...
    @typing.overload
    def __setitem__(self, arg0: slice, arg1: WindSpeedSourceVector) -> None:
        """
        Assign list elements using a slice object
        """
    def append(self, x: WindSpeedSource) -> None:
        """
        Add an item to the end of the list
        """
    def clear(self) -> None:
        """
        Clear the contents
        """
    @typing.overload
    def extend(self, L: WindSpeedSourceVector) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    @typing.overload
    def extend(self, L: typing.Iterable) -> None:
        """
        Extend the list by appending all the items in the given list
        """
    def insert(self, i: int, x: WindSpeedSource) -> None:
        """
        Insert an item at a given position.
        """
    @typing.overload
    def pop(self) -> WindSpeedSource:
        """
        Remove and return the last item
        """
    @typing.overload
    def pop(self, i: int) -> WindSpeedSource:
        """
        Remove and return the item at index ``i``
        """
    def size(self) -> int:
        ...
    def values_at_time(self, arg0: shyft.time_series.time) -> list[float]:
        ...
    @property
    def geo_tsvector(self) -> shyft.time_series.GeoTimeSeriesVector:
        """
        GeoTimeSeriesVector: returns a GeoTimeSeriesVector containing GeoTimeSeries where the geo-points are copied, but time-series are by-reference
        """
class stat_scope:
    """
    Defines the scope of the indexes passed to statistics functions
    .cell      : the indexes are considered as the i'th cell
    .catchment : the indexes are considered catchment identifiers
    
    The statistics is then covering the cells that matches the selected criteria
    
    
    Members:
    
      cell
    
      catchment
    """
    __members__: typing.ClassVar[dict[str, stat_scope]]  # value = {'cell': <stat_scope.cell: 0>, 'catchment': <stat_scope.catchment: 1>}
    catchment: typing.ClassVar[stat_scope]  # value = <stat_scope.catchment: 1>
    cell: typing.ClassVar[stat_scope]  # value = <stat_scope.cell: 0>
    @staticmethod
    def _pybind11_conduit_v1_(*args, **kwargs):
        ...
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
def ActualEvapotranspirationCalculate_step(water_level: float, potential_evapotranspiration: float, scale_factor: float, snow_fraction: float, dt: shyft.time_series.time) -> float:
    """
    actual_evapotranspiration calculates actual evapotranspiration, returning same unit as input pot.evap
    based on supplied parameters
    
    Args:
        water_level (): [mm] water level eqvivalent in ground, ae goes to zero if ground is drying out
    
        potential_evapotranspiration (): [mm/x], x time-unit
    
        scale_factor (): typically 1.5
    
        snow_fraction ():  0..1 there is to ae over snow surface, so snow_fraction 1.0, yields 0.0
    
        dt (): [s] timestep length, currently not part of the formula
    
    Returns:
        : actual evapotranspiration. 
    """
def HbvActualEvapotranspirationCalculate_step(soil_moisture: float, potential_evapotranspiration: float, lp: float, snow_fraction: float, dt: shyft.time_series.time) -> float:
    """
    actual_evapotranspiration calculates actual evapotranspiration, returning same unit as input pot.evap
    based on supplied parameters, formula like (1.0 - snow_fraction)*(soil_moisture < lp ? pot_evapo*(soil_moisture / lp):pot_evapo)
    
    Args:
        soil_moisture (): [mm] water level eqvivalent in ground, ae goes to zero if ground is drying out
    
        potential_evapotranspiration (): [mm/x], x time-unit
    
        lp (): [mm] soil_moisture threshold, lp typically 150
    
        snow_fraction ():  0..1 there is to ae over snow surface, so snow_fraction 1.0, yields 0.0
    
        dt (): [s] timestep length, currently not part of the formula
    
    Returns:
        : actual evapotranspiration. 
    """
def bayesian_kriging_temperature(src: TemperatureSourceVector, dst: shyft.time_series.GeoPointVector, time_axis: shyft.time_series.TimeAxisFixedDeltaT, btk_parameter: BTKParameter) -> TemperatureSourceVector:
    """
    Runs kriging for temperature sources and project the temperatures out to the destination geo-timeseries
    
    Notice that bayesian kriging is currently not very efficient for large grid inputs,
    using only one thread, and considering all source-timeseries (entire grid) for all destinations
    For few sources, spread out on a grid, it's quite efficient should work well
    
    
    
    Args:
        src ( TemperatureSourceVector): input a geo-located list of temperature time-series with filled in values (some might be nan etc.)
    
        dst (GeoPointVector): the GeoPoints,(x,y,z) locations to interpolate into
    
        time_axis ( TimeAxis): the destination time-axis, recall that the inputs can be any-time-axis and they are transformed and interpolated into the destination-timeaxis
    
        btk_parameter (BTKParameter): the parameters to be used during interpolation
    
    Returns:
        TemperatureSourceVector: result. with filled in temperatures according to their position, the idw_parameters and time_axis
    """
@typing.overload
def compute_geo_ts_values_at_time(geo_ts_vector: GeoPointSourceVector, t: shyft.time_series.time) -> list[float]:
    """
    compute the ts-values of the GeoPointSourceVector for the specified time t and return DoubleVector
    
    Args:
        geo_ts_vector (GeoPointSourceVector): Any kind of GeoPointSource vector
    
    Returns:
        DoubleValue: values. List of extracted values at same size/position as the geo_ts_vector
    
        t (int): timestamp in utc seconds since epoch
    """
@typing.overload
def compute_geo_ts_values_at_time(geo_ts_vector: TemperatureSourceVector, t: shyft.time_series.time) -> list[float]:
    """
    compute the ts-values of the GeoPointSourceVector type for the specified time t and return DoubleVector
    
    Args:
        geo_ts_vector (GeoPointSourceVector): Any kind of GeoPointSource vector
    
    Returns:
        DoubleValue: values. List of extracted values at same size/position as the geo_ts_vector
    
        t (int): timestamp in utc seconds since epoch
    """
@typing.overload
def compute_geo_ts_values_at_time(geo_ts_vector: PrecipitationSourceVector, t: shyft.time_series.time) -> list[float]:
    """
    compute the ts-values of the GeoPointSourceVector type for the specified time t and return DoubleVector
    
    Args:
        geo_ts_vector (GeoPointSourceVector): Any kind of GeoPointSource vector
    
    Returns:
        DoubleValue: values. List of extracted values at same size/position as the geo_ts_vector
    
        t (int): timestamp in utc seconds since epoch
    """
@typing.overload
def compute_geo_ts_values_at_time(geo_ts_vector: WindSpeedSourceVector, t: shyft.time_series.time) -> list[float]:
    """
    compute the ts-values of the GeoPointSourceVector type for the specified time t and return DoubleVector
    
    Args:
        geo_ts_vector (GeoPointSourceVector): Any kind of GeoPointSource vector
    
    Returns:
        DoubleValue: values. List of extracted values at same size/position as the geo_ts_vector
    
        t (int): timestamp in utc seconds since epoch
    """
@typing.overload
def compute_geo_ts_values_at_time(geo_ts_vector: RelHumSourceVector, t: shyft.time_series.time) -> list[float]:
    """
    compute the ts-values of the GeoPointSourceVector type for the specified time t and return DoubleVector
    
    Args:
        geo_ts_vector (GeoPointSourceVector): Any kind of GeoPointSource vector
    
    Returns:
        DoubleValue: values. List of extracted values at same size/position as the geo_ts_vector
    
        t (int): timestamp in utc seconds since epoch
    """
@typing.overload
def compute_geo_ts_values_at_time(geo_ts_vector: RadiationSourceVector, t: shyft.time_series.time) -> list[float]:
    """
    compute the ts-values of the GeoPointSourceVector type for the specified time t and return DoubleVector
    
    Args:
        geo_ts_vector (GeoPointSourceVector): Any kind of GeoPointSource vector
    
    Returns:
        DoubleValue: values. List of extracted values at same size/position as the geo_ts_vector
    
        t (int): timestamp in utc seconds since epoch
    """
def create_precipitation_source_vector_from_np_array(time_axis: shyft.time_series.TimeAxis, geo_points: shyft.time_series.GeoPointVector, np_array: numpy.ndarray[numpy.float64], point_fx: shyft.time_series.point_interpretation_policy) -> PrecipitationSourceVector:
    """
    Create a PrecipitationSourceVector from specified time_axis,geo_points, 2-d np_array and point_fx.
    
    Args:
        time_axis (TimeAxis): time-axis that matches in length to 2nd dim of np_array
    
        geo_points (GeoPointVector): the geo-positions for the time-series, should be of length n_ts
    
        np_array (np.ndarray): numpy array of dtype=np.float64, and shape(n_ts,n_points)
    
        point_fx (point interpretation): one of POINT_AVERAGE_VALUE|POINT_INSTANT_VALUE
    
    Returns:
        PrecipitationSourceVector: tsv. a PrecipitationSourceVector of length first np_array dim, n_ts, each with geo-point and time-series with time-axis, values and point_fx
    """
def create_radiation_source_vector_from_np_array(time_axis: shyft.time_series.TimeAxis, geo_points: shyft.time_series.GeoPointVector, np_array: numpy.ndarray[numpy.float64], point_fx: shyft.time_series.point_interpretation_policy) -> RadiationSourceVector:
    """
    Create a RadiationSourceVector from specified time_axis,geo_points, 2-d np_array and point_fx.
    
    Args:
        time_axis (TimeAxis): time-axis that matches in length to 2nd dim of np_array
    
        geo_points (GeoPointVector): the geo-positions for the time-series, should be of length n_ts
    
        np_array (np.ndarray): numpy array of dtype=np.float64, and shape(n_ts,n_points)
    
        point_fx (point interpretation): one of POINT_AVERAGE_VALUE|POINT_INSTANT_VALUE
    
    Returns:
        RadiationSourceVector: tsv. a RadiationSourceVector of length first np_array dim, n_ts, each with geo-point and time-series with time-axis, values and point_fx
    """
def create_rel_hum_source_vector_from_np_array(time_axis: shyft.time_series.TimeAxis, geo_points: shyft.time_series.GeoPointVector, np_array: numpy.ndarray[numpy.float64], point_fx: shyft.time_series.point_interpretation_policy) -> RelHumSourceVector:
    """
    Create a RelHumSourceVector from specified time_axis,geo_points, 2-d np_array and point_fx.
    
    Args:
        time_axis (TimeAxis): time-axis that matches in length to 2nd dim of np_array
    
        geo_points (GeoPointVector): the geo-positions for the time-series, should be of length n_ts
    
        np_array (np.ndarray): numpy array of dtype=np.float64, and shape(n_ts,n_points)
    
        point_fx (point interpretation): one of POINT_AVERAGE_VALUE|POINT_INSTANT_VALUE
    
    Returns:
        RelHumSourceVector: tsv. a RelHumSourceVector of length first np_array dim, n_ts, each with geo-point and time-series with time-axis, values and point_fx
    """
def create_temperature_source_vector_from_np_array(time_axis: shyft.time_series.TimeAxis, geo_points: shyft.time_series.GeoPointVector, np_array: numpy.ndarray[numpy.float64], point_fx: shyft.time_series.point_interpretation_policy) -> TemperatureSourceVector:
    """
    Create a TemperatureSourceVector from specified time_axis,geo_points, 2-d np_array and point_fx.
    
    Args:
        time_axis (TimeAxis): time-axis that matches in length to 2nd dim of np_array
    
        geo_points (GeoPointVector): the geo-positions for the time-series, should be of length n_ts
    
        np_array (np.ndarray): numpy array of dtype=np.float64, and shape(n_ts,n_points)
    
        point_fx (point interpretation): one of POINT_AVERAGE_VALUE|POINT_INSTANT_VALUE
    
    Returns:
        TemperatureSourceVector: tsv. a TemperatureSourceVector of length first np_array dim, n_ts, each with geo-point and time-series with time-axis, values and point_fx
    """
def create_wind_speed_source_vector_from_np_array(time_axis: shyft.time_series.TimeAxis, geo_points: shyft.time_series.GeoPointVector, np_array: numpy.ndarray[numpy.float64], point_fx: shyft.time_series.point_interpretation_policy) -> WindSpeedSourceVector:
    """
    Create a WindSpeedSourceVector from specified time_axis,geo_points, 2-d np_array and point_fx.
    
    Args:
        time_axis (TimeAxis): time-axis that matches in length to 2nd dim of np_array
    
        geo_points (GeoPointVector): the geo-positions for the time-series, should be of length n_ts
    
        np_array (np.ndarray): numpy array of dtype=np.float64, and shape(n_ts,n_points)
    
        point_fx (point interpretation): one of POINT_AVERAGE_VALUE|POINT_INSTANT_VALUE
    
    Returns:
        WindSpeedSourceVector: tsv. a WindSpeedSourceVector of length first np_array dim, n_ts, each with geo-point and time-series with time-axis, values and point_fx
    """
def glacier_melt_step(dtf: float, temperature: float, sca: float, glacier_fraction: float) -> float:
    """
    Calculates outflow from glacier melt rate [mm/h].
    
    Args:
        dtf (float): degree timestep factor [mm/day/deg.C]; lit. values for Norway: 5.5 - 6.4 in Hock, R. (2003), J. Hydrol., 282, 104-115.
    
        temperature (float): degC, considered constant over timestep dt
    
        sca (float): fraction of snow cover in cell [0..1], glacier melt occurs only if glacier fraction > snow fraction
    
        glacier_fraction (float): glacier fraction [0..1] in the total area
    
    Returns:
        float: glacier_melt. output from glacier, melt rate [mm/h]
    """
def idw_precipitation(src: PrecipitationSourceVector, dst: shyft.time_series.GeoPointVector, time_axis: shyft.time_series.TimeAxisFixedDeltaT, idw_param: IDWPrecipitationParameter) -> PrecipitationSourceVector:
    """
    Runs inverse distance interpolation to project precipitation sources out to the destination geo-timeseries
    
    Args:
        src (PrecipitationSourceVector): a geo-located list of precipitation time-series with filled in values (some might be nan etc.)
    
        dst (GeoPointVector): the GeoPoints,(x,y,z) locations to interpolate into
    
        time_axis (TimeAxis): the destination time-axis, recall that the inputs can be any-time-axis, they are transformed and interpolated into the destination-timeaxis
    
        idw_param (IDWPrecipitationParameter): the parameters to be used during interpolation
    
    Returns:
        PrecipitationSourceVector: precipitation. with filled in precipitations according to their position, the idw_parameters and time_axis
    """
def idw_radiation(src: RadiationSourceVector, dst: shyft.time_series.GeoPointVector, time_axis: shyft.time_series.TimeAxisFixedDeltaT, idw_param: IDWParameter, slope_factors: list[float]) -> RadiationSourceVector:
    """
    Runs inverse distance interpolation to project radiation sources out to the destination geo-timeseries
    
    Args:
        src (RadiationSourceVector): input a geo-located list of precipitation time-series with filled in values (some might be nan etc.)
    
        dst (GeoPointVector): the GeoPoints,(x,y,z) locations to interpolate into
    
        time_axis (TimeAxis): the destination time-axis, recall that the inputs can be any-time-axis, they are transformed and interpolated into the destination-timeaxis
    
        idw_param (IDWParameter): the parameters to be used during interpolation
    
        slope_factors (DoubleVector): the slope-factor corresponding to geopoints, typical 0.9
    
    Returns:
        RadiationSourceVector: radiation. with filled in radiation according to their position, the idw_parameters and time_axis
    """
def idw_relative_humidity(src: RelHumSourceVector, dst: shyft.time_series.GeoPointVector, time_axis: shyft.time_series.TimeAxisFixedDeltaT, idw_param: IDWParameter) -> RelHumSourceVector:
    """
    Runs inverse distance interpolation to project relative humidity sources out to the destination geo-timeseries
    
    Args:
        src (RelHumSourceVector): input a geo-located list of precipitation time-series with filled in values (some might be nan etc.)
    
        dst (GeoPointVector): the GeoPoints,(x,y,z) locations to interpolate into
    
        time_axis (TimeAxis): the destination time-axis, recall that the inputs can be any-time-axis, they are transformed and interpolated into the destination-timeaxis
    
        idw_param (IDWParameter): the parameters to be used during interpolation
    
    Returns:
        RelHumSourceVector: relhum. with filled in relative humidity according to their position, the idw_parameters and time_axis
    """
def idw_temperature(src: TemperatureSourceVector, dst: shyft.time_series.GeoPointVector, time_axis: shyft.time_series.TimeAxisFixedDeltaT, idw_param: IDWTemperatureParameter) -> TemperatureSourceVector:
    """
    Runs inverse distance interpolation to project temperature sources out to the destination geo-timeseries
    
    Args:
        src (TemperatureSourceVectorn): input a geo-located list of temperature time-series with filled in values (some might be nan etc.)
    
        dst (GeoPointVector): the GeoPoints,(x,y,z) locations to interpolate into
    
        time_axis (TimeAxis): the destination time-axis, recall that the inputs can be any-time-axis, they are transformed and interpolated into the destination-timeaxis
    
        idw_param (IDWTemperatureParameter): the parameters to be used during interpolation, defines the behaviour of the interpolation. Please use help on the IDWTemperatureParameter for details.
    
    Returns:
        TemperatureSourceVector: temperatures. with filled in temperatures according to their position, the idw_parameters and time_axis
    """
def idw_wind_speed(src: WindSpeedSourceVector, dst: shyft.time_series.GeoPointVector, time_axis: shyft.time_series.TimeAxisFixedDeltaT, idw_param: IDWParameter, shear_factor: float = ..., hub_height: float = ..., reference_height: float = ...) -> WindSpeedSourceVector:
    """
    Runs inverse distance interpolation to project geo located wind input sources out to the destination geo-timeseries
    
    
    Args:
        src (WindspeedSourceVector): input a geo-located list of wind speed time-series with filled in values (some might be nan etc.)
    
        dst (GeoPointVector): the GeoPoints,(x,y,z) locations to interpolate into
    
        time_axis (TimeAxis): the destination time-axis, recall that the inputs can be any-time-axis, they are transformed and interpolated into the destination-timeaxis
    
        idw_param (IDWParametern): the parameters to be used during interpolation
    
        shear_factor (float): optional:wind shear coefficient to estimate the wind speed at higher elevations, typical 0.11 onshore and 0.15 offshore,>0.0
    
        hub_height (float): optional:distance from the ground to the middle of the turbine's rotor, >0.0 
    
        reference_height (float): optional: height above ground of measured wind speed, >0.0
    
    Returns:
        WindSpeedSourceVector: windspeed. with filled in wind speed according to their position, the idw_parameters and time_axis
    """
def make_uhg_from_gamma(n_steps: int, alpha: float, beta: float) -> list[float]:
    """
    make_uhg_from_gamma a simple function to create a uhg (unit hydro graph) weight vector
    containing n_steps, given the gamma shape factor alpha and beta.
    ensuring the sum of the weight vector is 1.0
    and that it has a min-size of one element (1.0)
    
    
    Args:
        n_steps (int): number of time-steps, elements, in the vectorn
    
        alpha (float): the gamma_distribution gamma-factor
    
        beta (float): the base-line, added to pdf(gamma(alpha,1))
    
    Returns:
        : unit hydro graph factors.  - normalized to sum 1.0
    """
def ordinary_kriging(src: GeoPointSourceVector, dst: shyft.time_series.GeoPointVector, time_axis: shyft.time_series.TimeAxisFixedDeltaT, parameter: OKParameter) -> GeoPointSourceVector:
    """
    Runs ordinary kriging for geo sources and project the source out to the destination geo-timeseries
    Notice that kriging is currently not very efficient for large grid inputs,
    using only one thread, and considering all source-timeseries (entire grid) for all destinations
    For few sources, spread out on a grid, it's quite efficient should work well
    Also note that this function currently does not elicite observations with nan-data
    most useful when you have control on the inputs, providing full set of data.
    
    
    Args:
        src (GeoSourceVector): input a geo-located list of time-series with filled in values
    
        dst (GeoPointVector): the GeoPoints,(x,y,z) locations to interpolate into
    
        time_axis (TimeAxis): the destination time-axis, recall that the inputs can be any-time-axis, and they are transformed and interpolated into the destination-timeaxis
    
        parameter (OKParameter): the parameters to be used during interpolation
    
    Returns:
        GeoSourceVector: result. with filled in ts-values according to their position, the parameters and time_axis
    """
def parse_cf_time(reference: str) -> shyft.time_series.UtcPeriod:
    """
    parses cf time unit string like hours since 1970-01-01 00:00:00
    and returs an UtcPeriod, where the .start is the reference time
    and the .timespan() is the delta time, e.g. 3600.0
    If the reference is illformed, then the returned period is not .valid()
    The intended usage is to get the reference time and time-delta of netcdf files,
    replacing the need of cftime python library.
    
    Args:
        reference (str): netcdf time-reference
    
    Returns:
        UtcPeriod: p. p.valid()==True if well formed,p.start equal to the reference time, and p.timespan() equal to the time-delta specified
    """
def version() -> str:
    ...
ABS_DIFF: TargetSpecCalcType  # value = <TargetSpecCalcType.ABS_DIFF: 2>
BOBYQA: OptimizerMethod  # value = <OptimizerMethod.BOBYQA: 0>
CELL_CHARGE: CatchmentPropertyType  # value = <CatchmentPropertyType.CELL_CHARGE: 4>
DISCHARGE: CatchmentPropertyType  # value = <CatchmentPropertyType.DISCHARGE: 0>
DREAM: OptimizerMethod  # value = <OptimizerMethod.DREAM: 2>
EXPONENTIAL: OKCovarianceType  # value = <OKCovarianceType.EXPONENTIAL: 1>
GAUSSIAN: OKCovarianceType  # value = <OKCovarianceType.GAUSSIAN: 0>
GLOBAL: OptimizerMethod  # value = <OptimizerMethod.GLOBAL: 1>
KLING_GUPTA: TargetSpecCalcType  # value = <TargetSpecCalcType.KLING_GUPTA: 1>
NASH_SUTCLIFFE: TargetSpecCalcType  # value = <TargetSpecCalcType.NASH_SUTCLIFFE: 0>
PT_GS_K: RegionModelType  # value = <RegionModelType.PT_GS_K: 0>
PT_GS_K_OPT: RegionModelType  # value = <RegionModelType.PT_GS_K_OPT: 1>
PT_HPS_K: RegionModelType  # value = <RegionModelType.PT_HPS_K: 6>
PT_HPS_K_OPT: RegionModelType  # value = <RegionModelType.PT_HPS_K_OPT: 7>
PT_HS_K: RegionModelType  # value = <RegionModelType.PT_HS_K: 4>
PT_HS_K_OPT: RegionModelType  # value = <RegionModelType.PT_HS_K_OPT: 5>
PT_SS_K: RegionModelType  # value = <RegionModelType.PT_SS_K: 2>
PT_SS_K_OPT: RegionModelType  # value = <RegionModelType.PT_SS_K_OPT: 3>
PT_ST_HBV: RegionModelType  # value = <RegionModelType.PT_ST_HBV: 12>
PT_ST_K: RegionModelType  # value = <RegionModelType.PT_ST_K: 10>
PT_ST_K_HBV: RegionModelType  # value = <RegionModelType.PT_ST_K_HBV: 13>
PT_ST_K_OPT: RegionModelType  # value = <RegionModelType.PT_ST_K_OPT: 11>
RMSE: TargetSpecCalcType  # value = <TargetSpecCalcType.RMSE: 3>
ROUTED_DISCHARGE: CatchmentPropertyType  # value = <CatchmentPropertyType.ROUTED_DISCHARGE: 3>
R_PMV_ST_K: RegionModelType  # value = <RegionModelType.R_PMV_ST_K: 18>
R_PMV_ST_K_OPT: RegionModelType  # value = <RegionModelType.R_PMV_ST_K_OPT: 19>
R_PM_GS_K: RegionModelType  # value = <RegionModelType.R_PM_GS_K: 8>
R_PM_GS_K_OPT: RegionModelType  # value = <RegionModelType.R_PM_GS_K_OPT: 9>
R_PM_ST_K: RegionModelType  # value = <RegionModelType.R_PM_ST_K: 16>
R_PM_ST_K_OPT: RegionModelType  # value = <RegionModelType.R_PM_ST_K_OPT: 17>
R_PT_GS_K: RegionModelType  # value = <RegionModelType.R_PT_GS_K: 14>
R_PT_GS_K_OPT: RegionModelType  # value = <RegionModelType.R_PT_GS_K_OPT: 15>
SCEUA: OptimizerMethod  # value = <OptimizerMethod.SCEUA: 3>
SNOW_COVERED_AREA: CatchmentPropertyType  # value = <CatchmentPropertyType.SNOW_COVERED_AREA: 1>
SNOW_WATER_EQUIVALENT: CatchmentPropertyType  # value = <CatchmentPropertyType.SNOW_WATER_EQUIVALENT: 2>
catchment: stat_scope  # value = <stat_scope.catchment: 1>
cell: stat_scope  # value = <stat_scope.cell: 0>
shyftdata_dir: str = 'C:\\gitlab\\projects\\builds\\h8NJAAPv\\0\\shyft-os\\shyft\\build\\ci\\_python_stubs\\shyft-data'
